(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     17674,        540]
NotebookOptionsPosition[     13477,        421]
NotebookOutlinePosition[     15488,        467]
CellTagsIndexPosition[     15445,        464]
WindowTitle->LieART: Lie Algebras and Representation Theory - Wolfram \
Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["LIE A R T PACKAGE GUIDE", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Algebra\"\>":>Documentation`HelpLookup[
       "paclet:LieART/ref/Algebra"], "\<\"CartanMatrix\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/CartanMatrix"], "\<\"DecomposeIrrep\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/DecomposeIrrep"], "\<\"DecomposeProduct\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/DecomposeProduct"], "\<\"Dim\"\>":>
       Documentation`HelpLookup["paclet:LieART/ref/Dim"], "\<\"DimName\"\>":>
       Documentation`HelpLookup["paclet:LieART/ref/DimName"], "\<\"Index\"\>":>
       Documentation`HelpLookup["paclet:LieART/ref/Index"], "\<\"Irrep\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/Irrep"], "\<\"IrrepPlus\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/IrrepPlus"], "\<\"LaTeXForm\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/LaTeXForm"], "\<\"MetricTensor\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/MetricTensor"], "\<\"OrthogonalSimpleRoots\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/OrthogonalSimpleRoots"], "\<\"PositiveRoots\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/PositiveRoots"], "\<\"ProductAlgebra\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/ProductAlgebra"], "\<\"ProductIrrep\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/ProductIrrep"], "\<\"RootSystem\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/RootSystem"], "\<\"WeightSystem\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/WeightSystem"], "\<\"YoungTableau\"\>":>
       Documentation`HelpLookup["paclet:LieART/ref/YoungTableau"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Representation Properties\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/Tables/IrrepProperties"], "\<\"Tensor Products\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/Tables/TensorProducts"], "\<\"Branching Rules\"\>":>
       Documentation`HelpLookup["paclet:LieART/Tables/BranchingRules"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Tutorials \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"Quick Start Tutorial\"\>":>
       Documentation`HelpLookup["paclet:LieART/tutorial/QuickStartTutorial"]},
       
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"LieART/Guides/LieART\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["LieART/Guides/LieART"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/LieART/Guides/\
LieART.html"], StandardForm]], "Input", TextClipboardType -> "PlainText"]}, 
            Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "LieART/Guides/LieART", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 CellID->1],

Cell[CellGroupData[{

Cell["LieART: Lie Algebras and Representation Theory", "GuideTitle",
 CellID->942062912],

Cell["\<\
The primary purpose of the LieART application is the tensor product and \
subalgebra decomposition of irreducible representations (irreps) of Lie \
algebras. These tasks are frequently needed in particle physics, where \
multiparticle fields are assigned to irreps. Tensor products of irreps are \
thus commonly used to find singlets suitable for the Lagrangian. Spontaneous \
symmetry breaking corresponds to the subalgebra decomposition.\
\>", "GuideAbstract",
 CellID->2001916300],

Cell[CellGroupData[{

Cell["Algebras", "GuideFunctionsSubsection",
 CellID->1219780944],

Cell[TextData[{
 Cell[TextData[ButtonBox["Algebra",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/Algebra"]], "InlineFunctionSans"],
 "\[NonBreakingSpace]",
 StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", 
  "InlineSeparator"],
 " ",
 Cell[TextData[ButtonBox["ProductAlgebra",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/ProductAlgebra"]], "InlineFunctionSans"],
 "\[NonBreakingSpace]",
 StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", 
  "InlineSeparator"],
 " ",
 Cell[TextData[ButtonBox["CartanMatrix",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/CartanMatrix"]], "InlineFunctionSans"],
 "\[NonBreakingSpace]",
 StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", 
  "InlineSeparator"],
 " ",
 Cell[TextData[ButtonBox["MetricTensor",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/MetricTensor"]], "InlineFunctionSans"]
}], "InlineGuideFunctionListing",
 CellID->329721484]
}, Open  ]],

Cell[CellGroupData[{

Cell["Roots", "GuideFunctionsSubsection",
 CellID->1654006133],

Cell[TextData[{
 Cell[TextData[ButtonBox["RootSystem",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/RootSystem"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " ",
 "Complete root system of an algebra"
}], "GuideText",
 CellID->1474453966],

Cell[TextData[{
 Cell[TextData[ButtonBox["PositiveRoots",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/PositiveRoots"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " ",
 "Only the positive roots of an algebra"
}], "GuideText",
 CellID->595719226],

Cell[TextData[{
 Cell[TextData[ButtonBox["OrthogonalSimpleRoots",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/OrthogonalSimpleRoots"]], 
  "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " ",
 "Simple roots of an algebra in orthogonal coordinates"
}], "GuideText",
 CellID->1369365639]
}, Open  ]],

Cell[CellGroupData[{

Cell["Weights", "GuideFunctionsSubsection",
 CellID->2016205067],

Cell[TextData[{
 Cell[TextData[ButtonBox["WeightSystem",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/WeightSystem"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " ",
 "Complete weight system of a representation"
}], "GuideText",
 CellID->1342091672]
}, Open  ]],

Cell[CellGroupData[{

Cell["Representations", "GuideFunctionsSubsection",
 CellID->775384787],

Cell[TextData[{
 Cell[TextData[ButtonBox["Irrep",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/Irrep"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " Irreducible representation"
}], "GuideText",
 CellID->361439325],

Cell[TextData[{
 Cell[TextData[ButtonBox["Dim",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/Dim"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " ",
 "Compute the dimension of a representation"
}], "GuideText",
 CellID->146551704],

Cell[TextData[{
 Cell[TextData[ButtonBox["YoungTableau",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/YoungTableau"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " ",
 "Displays the Young tableau corresponding to an SU(N) representation"
}], "GuideText",
 CellID->447911115],

Cell[TextData[{
 Cell[TextData[ButtonBox["ProductIrrep",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/ProductIrrep"]], "InlineFunctionSans"],
 "\[NonBreakingSpace]",
 StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", 
  "InlineSeparator"],
 " ",
 Cell[TextData[ButtonBox["DimName",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/DimName"]], "InlineFunctionSans"],
 "\[NonBreakingSpace]",
 StyleBox["\[MediumSpace]\[FilledVerySmallSquare]\[MediumSpace]", 
  "InlineSeparator"],
 " ",
 Cell[TextData[ButtonBox["Index",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/Index"]], "InlineFunctionSans"]
}], "InlineGuideFunctionListing",
 CellID->641609725]
}, Open  ]],

Cell[CellGroupData[{

Cell["Decompositions", "GuideFunctionsSubsection",
 CellID->760568900],

Cell[TextData[{
 Cell[TextData[ButtonBox["DecomposeProduct",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/DecomposeProduct"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " ",
 "Decompose tensor products of irreps"
}], "GuideText",
 CellID->931894448],

Cell[TextData[{
 Cell[TextData[ButtonBox["DecomposeIrrep",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/DecomposeIrrep"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " Decompose irrep to subalgebras"
}], "GuideText",
 CellID->424177561]
}, Open  ]],

Cell[CellGroupData[{

Cell["Displaying Output", "GuideFunctionsSubsection",
 CellID->1103951756],

Cell[TextData[{
 Cell[TextData[ButtonBox["IrrepPlus",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/IrrepPlus"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " ",
 "Textbook style sum of a list irreps"
}], "GuideText",
 CellID->1445608321],

Cell[TextData[{
 Cell[TextData[ButtonBox["LaTeXForm",
  BaseStyle->"Link",
  ButtonData->"paclet:LieART/ref/LaTeXForm"]], "InlineFunctionSans"],
 " ",
 StyleBox["\[LongDash]", "GuideEmDash"],
 " ",
 "Formatting suitable for copy&paste into a ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    RowBox[{"L", 
     StyleBox[
      AdjustmentBox["A",
       BoxBaselineShift->-0.4,
       BoxMargins->{{-0.5, -0.3}, {0, 0}}],
      FontSize->Smaller], "T", 
     AdjustmentBox["E",
      BoxBaselineShift->0.5,
      BoxMargins->{{-0.3, 0}, {0, 0}}], "X"}], "TR",
    SingleLetterItalics->False], TraditionalForm]]],
 " document"
}], "GuideText",
 CellID->490235135]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TUTORIALS",
 CellFrame->{{0, 0}, {0, 2}},
 ShowCellBracket->Automatic,
 CellMargins->{{24, 24}, {9, 18}},
 CellElementSpacings->{"ClosedGroupTopMargin"->28},
 CellGroupingRules->{"SectionGrouping", 30},
 CellFrameMargins->4,
 CellFrameColor->RGBColor[
  0.48235300000000003`, 0.5176470000000001, 0.8196080000000001],
 StyleMenuListing->None,
 FontFamily->"Verdana",
 FontSize->9,
 FontTracking->"Wide",
 CellID->1804949407],

Cell[TextData[ButtonBox["Quick Start Tutorial",
 BaseStyle->"Link",
 ButtonData->"paclet:LieART/tutorial/QuickStartTutorial"]], "GuideTutorial",
 CellID->1275349391]
}, Open  ]],

Cell[CellGroupData[{

Cell["TABLES",
 Editable->False,
 CellFrame->{{0, 0}, {0, 2}},
 ShowCellBracket->Automatic,
 CellMargins->{{24, 24}, {9, 18}},
 CellElementSpacings->{"ClosedGroupTopMargin"->28},
 CellGroupingRules->{"SectionGrouping", 30},
 CellFrameMargins->4,
 CellFrameColor->RGBColor[
  0.48235300000000003`, 0.5176470000000001, 0.8196080000000001],
 StyleMenuListing->None,
 FontFamily->"Verdana",
 FontSize->9,
 FontTracking->"Wide"],

Cell[TextData[ButtonBox["Representation Properties",
 BaseStyle->"Link",
 ButtonData->"paclet:LieART/Tables/IrrepProperties"]], "GuideMoreAbout",
 CellID->427798698],

Cell[TextData[ButtonBox["Tensor Products",
 BaseStyle->"Link",
 ButtonData->"paclet:LieART/Tables/TensorProducts"]], "GuideMoreAbout",
 CellID->240618869],

Cell[TextData[ButtonBox["Branching Rules",
 BaseStyle->"Link",
 ButtonData->"paclet:LieART/Tables/BranchingRules"]], "GuideMoreAbout",
 CellID->359825597]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"LieART: Lie Algebras and Representation Theory - Wolfram \
Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "GuideColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        StyleBox[
         ButtonBox[
         "Mathematica", ButtonData -> "paclet:guide/Mathematica", 
          BaseStyle -> {"Link", "DockedLinkTrail"}], FontSlant -> "Italic"], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "Metadata" -> {
   "built" -> "{2014, 8, 6, 10, 23, 8.466952}", 
    "history" -> {"0.9", "", "", ""}, "context" -> "LieART`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Lie A R T Package Guide", "language" -> "en", "paclet" -> 
    "LieART Package", "status" -> "None", "summary" -> 
    "The primary purpose of the LieART application is the tensor product and \
subalgebra decomposition of irreducible representations (irreps) of Lie \
algebras. These tasks are frequently needed in particle physics, where \
multiparticle fields are assigned to irreps. Tensor products of irreps are \
thus commonly used to find singlets suitable for the Lagrangian. Spontaneous \
symmetry breaking corresponds to the subalgebra decomposition.", 
    "synonyms" -> {}, "title" -> 
    "LieART: Lie Algebras and Representation Theory", "titlemodifier" -> "", 
    "windowtitle" -> "LieART: Lie Algebras and Representation Theory", "type" -> 
    "Guide", "uri" -> "LieART/Guides/LieART"}, "SearchTextTranslated" -> ""},
FrontEndVersion->"9.0 for Linux x86 (64-bit) (February 7, 2013)",
StyleDefinitions->FrontEnd`FileName[{"Wolfram"}, "Reference.nb", 
  CharacterEncoding -> "UTF-8"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[641, 22, 4813, 104, 49, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[5479, 130, 88, 1, 76, "GuideTitle",
 CellID->942062912],
Cell[5570, 133, 493, 8, 123, "GuideAbstract",
 CellID->2001916300],
Cell[CellGroupData[{
Cell[6088, 145, 65, 1, 29, "GuideFunctionsSubsection",
 CellID->1219780944],
Cell[6156, 148, 959, 26, 20, "InlineGuideFunctionListing",
 CellID->329721484]
}, Open  ]],
Cell[CellGroupData[{
Cell[7152, 179, 62, 1, 29, "GuideFunctionsSubsection",
 CellID->1654006133],
Cell[7217, 182, 275, 9, 21, "GuideText",
 CellID->1474453966],
Cell[7495, 193, 283, 9, 21, "GuideText",
 CellID->595719226],
Cell[7781, 204, 318, 10, 21, "GuideText",
 CellID->1369365639]
}, Open  ]],
Cell[CellGroupData[{
Cell[8136, 219, 64, 1, 29, "GuideFunctionsSubsection",
 CellID->2016205067],
Cell[8203, 222, 287, 9, 21, "GuideText",
 CellID->1342091672]
}, Open  ]],
Cell[CellGroupData[{
Cell[8527, 236, 71, 1, 29, "GuideFunctionsSubsection",
 CellID->775384787],
Cell[8601, 239, 251, 8, 21, "GuideText",
 CellID->361439325],
Cell[8855, 249, 267, 9, 21, "GuideText",
 CellID->146551704],
Cell[9125, 260, 311, 9, 21, "GuideText",
 CellID->447911115],
Cell[9439, 271, 687, 19, 20, "InlineGuideFunctionListing",
 CellID->641609725]
}, Open  ]],
Cell[CellGroupData[{
Cell[10163, 295, 70, 1, 29, "GuideFunctionsSubsection",
 CellID->760568900],
Cell[10236, 298, 287, 9, 21, "GuideText",
 CellID->931894448],
Cell[10526, 309, 273, 8, 21, "GuideText",
 CellID->424177561]
}, Open  ]],
Cell[CellGroupData[{
Cell[10836, 322, 74, 1, 29, "GuideFunctionsSubsection",
 CellID->1103951756],
Cell[10913, 325, 274, 9, 21, "GuideText",
 CellID->1445608321],
Cell[11190, 336, 657, 23, 70, "GuideText",
 CellID->490235135]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[11896, 365, 429, 13, 70, InheritFromParent,
 CellGroupingRules->{"SectionGrouping", 30},
 CellID->1804949407],
Cell[12328, 380, 165, 3, 70, "GuideTutorial",
 CellID->1275349391]
}, Open  ]],
Cell[CellGroupData[{
Cell[12530, 388, 423, 13, 70, InheritFromParent,
 CellGroupingRules->{"SectionGrouping", 30}],
Cell[12956, 403, 165, 3, 70, "GuideMoreAbout",
 CellID->427798698],
Cell[13124, 408, 154, 3, 70, "GuideMoreAbout",
 CellID->240618869],
Cell[13281, 413, 154, 3, 70, "GuideMoreAbout",
 CellID->359825597]
}, Open  ]],
Cell[13450, 419, 23, 0, 70, "FooterCell"]
}
]
*)

(* End of internal cache information *)

