(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     17047,        565]
NotebookOptionsPosition[     12443,        423]
NotebookOutlinePosition[     14517,        479]
CellTagsIndexPosition[     14432,        474]
WindowTitle->OrthogonalSimpleRoots - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["LIE A R T SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"LieART: Lie Algebras and Representation Theory\"\>":>
       Documentation`HelpLookup["paclet:LieART/guide/LieART"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"LieART/ref/OrthogonalSimpleRoots\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["LieART/ref/OrthogonalSimpleRoots"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/LieART/ref/\
OrthogonalSimpleRoots.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "LieART/ref/OrthogonalSimpleRoots", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell["OrthogonalSimpleRoots", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"OrthogonalSimpleRoots", "[", 
       StyleBox["algebra", "TI"], "]"}]], "InlineFormula"],
     " \[LineSeparator]gives the simple roots of ",
     Cell[BoxData[
      StyleBox["algebra", "TI"]], "InlineFormula"],
     " in the orthogonal basis."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}},
 CellID->982511436],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 WholeCellGroupOpener->True,
 CellTags->"PrimaryExamplesSection",
 CellID->63422717],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(2)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->1598686097],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<LieART`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->667351211],

Cell[CellGroupData[{

Cell["\<\
Simple roots of all rank 4 classical Lie algebras in the orthogonal basis:\
\>", "ExampleText",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellID->432688663],

Cell[BoxData[
 RowBox[{"OrthogonalSimpleRoots", "[", "A4", "]"}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellLabel->"In[2]:=",
 CellID->1503630565],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox["\<\"(1, -1, 0, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 1, -1, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, 1, -1, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, 0, 1, -1)\"\>",
     TraditionalForm]}], "}"}], TraditionalForm]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 ImageSize->{422, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->49766444],

Cell[BoxData[
 RowBox[{"OrthogonalSimpleRoots", "[", "B4", "]"}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellLabel->"In[3]:=",
 CellID->1000317780],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox["\<\"(1, -1, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 1, -1, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, 1, -1)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, 0, 1)\"\>",
     TraditionalForm]}], "}"}], TraditionalForm]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 ImageSize->{345, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->546677541],

Cell[BoxData[
 RowBox[{"OrthogonalSimpleRoots", "[", "C4", "]"}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellLabel->"In[4]:=",
 CellID->1089582623],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox["\<\"(1, -1, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 1, -1, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, 1, -1)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, 0, 2)\"\>",
     TraditionalForm]}], "}"}], TraditionalForm]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 ImageSize->{346, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->1318456167],

Cell[BoxData[
 RowBox[{"OrthogonalSimpleRoots", "[", "D4", "]"}]], "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellLabel->"In[5]:=",
 CellID->326623444],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox["\<\"(1, -1, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 1, -1, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, 1, -1)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, 1, 1)\"\>",
     TraditionalForm]}], "}"}], TraditionalForm]], "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 ImageSize->{345, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->967405781]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[Cell["\t", "ExampleDelimiter"],
  $Line = 0; Null]], "ExampleDelimiter",
 CellID->71220346],

Cell["\<\
Simple roots of all exceptional Lie algebras in the orthogonal basis:\
\>", "ExampleText",
 CellID->1687553946],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OrthogonalSimpleRoots", "[", "E6", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1519463992],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox["\<\"(1/2, -1/2, -1/2, -1/2, -1/2, -1/2, -1/2, 1/2)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(-1, 1, 0, 0, 0, 0, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, -1, 1, 0, 0, 0, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, -1, 1, 0, 0, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, 0, -1, 1, 0, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(1, 1, 0, 0, 0, 0, 0, 0)\"\>",
     TraditionalForm]}], "}"}], TraditionalForm]], "Output",
 ImageSize->{471, 54},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[1]=",
 CellID->900921066]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OrthogonalSimpleRoots", "[", "E7", "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->1034770548],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox["\<\"(1/2, -1/2, -1/2, -1/2, -1/2, -1/2, -1/2, 1/2)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(-1, 1, 0, 0, 0, 0, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, -1, 1, 0, 0, 0, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, -1, 1, 0, 0, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, 0, -1, 1, 0, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, 0, 0, -1, 1, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(1, 1, 0, 0, 0, 0, 0, 0)\"\>",
     TraditionalForm]}], "}"}], TraditionalForm]], "Output",
 ImageSize->{333, 73},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->975780727]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OrthogonalSimpleRoots", "[", "E8", "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->2111439842],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox["\<\"(1/2, -1/2, -1/2, -1/2, -1/2, -1/2, -1/2, 1/2)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(-1, 1, 0, 0, 0, 0, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, -1, 1, 0, 0, 0, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, -1, 1, 0, 0, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, 0, -1, 1, 0, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, 0, 0, -1, 1, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, 0, 0, 0, -1, 1, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(1, 1, 0, 0, 0, 0, 0, 0)\"\>",
     TraditionalForm]}], "}"}], TraditionalForm]], "Output",
 ImageSize->{333, 92},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->652264783]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OrthogonalSimpleRoots", "[", "F4", "]"}]], "Input",
 CellLabel->"In[4]:=",
 CellID->1130387038],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox["\<\"(1, -1, 0, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 1, -1, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(0, 0, 1, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(-1/2, -1/2, -1/2, -1/2)\"\>",
     TraditionalForm]}], "}"}], TraditionalForm]], "Output",
 ImageSize->{271, 35},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[4]=",
 CellID->1989954993]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OrthogonalSimpleRoots", "[", "G2", "]"}]], "Input",
 CellLabel->"In[5]:=",
 CellID->667380572],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox["\<\"(1, -1, 0)\"\>",
     TraditionalForm], ",", 
    FormBox["\<\"(-2, 1, 1)\"\>",
     TraditionalForm]}], "}"}], TraditionalForm]], "Output",
 ImageSize->{147, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[5]=",
 CellID->247295215]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 WholeCellGroupOpener->True,
 CellID->38303248],

Cell[TextData[ButtonBox["LieART: Lie Algebras and Representation Theory",
 BaseStyle->"Link",
 ButtonData->"paclet:LieART/guide/LieART"]], "MoreAbout",
 CellID->1930509961]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"OrthogonalSimpleRoots - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "SymbolColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        ButtonBox[
        "LieART: Lie Algebras and Representation Theory", ButtonData -> 
         "paclet:LieART/guide/LieART", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "Metadata" -> {
   "built" -> "{2014, 8, 6, 10, 23, 54.588314}", 
    "history" -> {"0.9", "", "", ""}, "context" -> "LieART`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Lie A R T Symbol", "language" -> "en", "paclet" -> "LieART", "status" -> 
    "None", "summary" -> 
    "OrthogonalSimpleRoots[algebra] gives the simple roots of algebra in the \
orthogonal basis.", "synonyms" -> {}, "title" -> "OrthogonalSimpleRoots", 
    "titlemodifier" -> "", "windowtitle" -> "OrthogonalSimpleRoots", "type" -> 
    "Symbol", "uri" -> "LieART/ref/OrthogonalSimpleRoots"}, 
  "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"9.0 for Linux x86 (64-bit) (February 7, 2013)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "9.0 for Linux x86 (64-bit) (February 7, 2013)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[3594, 99, 489, 14, 29, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->63422717]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 14291, 467}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[614, 21, 2377, 54, 49, "AnchorBarGrid",
 CellID->1],
Cell[2994, 77, 64, 1, 42, "ObjectName",
 CellID->1224892054],
Cell[3061, 80, 508, 15, 86, "Usage",
 CellID->982511436],
Cell[CellGroupData[{
Cell[3594, 99, 489, 14, 29, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->63422717],
Cell[CellGroupData[{
Cell[4108, 117, 196, 6, 25, "ExampleSection",
 CellID->1598686097],
Cell[4307, 125, 115, 3, 28, "Input",
 CellID->667351211],
Cell[CellGroupData[{
Cell[4447, 132, 178, 4, 34, "ExampleText",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellID->432688663],
Cell[4628, 138, 172, 4, 28, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellID->1503630565],
Cell[4803, 144, 542, 17, 37, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellID->49766444],
Cell[5348, 163, 172, 4, 28, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellID->1000317780],
Cell[5523, 169, 530, 17, 37, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellID->546677541],
Cell[6056, 188, 172, 4, 28, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellID->1089582623],
Cell[6231, 194, 531, 17, 37, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellID->1318456167],
Cell[6765, 213, 171, 4, 28, "Input",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellID->326623444],
Cell[6939, 219, 530, 17, 37, "Output",
 CellGroupingRules->{GroupTogetherGrouping, 10000.},
 CellID->967405781]
}, Open  ]],
Cell[CellGroupData[{
Cell[7506, 241, 124, 3, 7, "ExampleDelimiter",
 CellID->71220346],
Cell[7633, 246, 121, 3, 34, "ExampleText",
 CellID->1687553946],
Cell[CellGroupData[{
Cell[7779, 253, 119, 3, 28, "Input",
 CellID->1519463992],
Cell[7901, 258, 702, 20, 75, "Output",
 CellID->900921066]
}, Open  ]],
Cell[CellGroupData[{
Cell[8640, 283, 119, 3, 28, "Input",
 CellID->1034770548],
Cell[8762, 288, 780, 22, 94, "Output",
 CellID->975780727]
}, Open  ]],
Cell[CellGroupData[{
Cell[9579, 315, 119, 3, 28, "Input",
 CellID->2111439842],
Cell[9701, 320, 858, 24, 113, "Output",
 CellID->652264783]
}, Open  ]],
Cell[CellGroupData[{
Cell[10596, 349, 119, 3, 28, "Input",
 CellID->1130387038],
Cell[10718, 354, 489, 16, 56, "Output",
 CellID->1989954993]
}, Open  ]],
Cell[CellGroupData[{
Cell[11244, 375, 118, 3, 28, "Input",
 CellID->667380572],
Cell[11365, 380, 340, 12, 37, "Output",
 CellID->247295215]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[11778, 400, 448, 13, 29, "MoreAboutSection",
 CellID->38303248],
Cell[12229, 415, 172, 3, 18, "MoreAbout",
 CellID->1930509961]
}, Open  ]],
Cell[12416, 421, 23, 0, 44, "FooterCell"]
}
]
*)

(* End of internal cache information *)

