(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     10305,        341]
NotebookOptionsPosition[      7294,        249]
NotebookOutlinePosition[      9323,        304]
CellTagsIndexPosition[      9239,        299]
WindowTitle->ProductAlgebra - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["LIE A R T SYMBOL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"More About \[RightGuillemet]\"\>",
        StripOnInput->
         False], {"\<\"LieART: Lie Algebras and Representation Theory\"\>":>
       Documentation`HelpLookup["paclet:LieART/guide/LieART"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"LieART/ref/ProductAlgebra\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["LieART/ref/ProductAlgebra"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/LieART/ref/\
ProductAlgebra.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "LieART/ref/ProductAlgebra", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell["ProductAlgebra", "ObjectName",
 CellID->1224892054],

Cell[BoxData[GridBox[{
   {"", Cell[TextData[{
     Cell[BoxData[
      RowBox[{"ProductAlgebra", "[", 
       StyleBox["algebras", "TI"], "]"}]], "InlineFormula"],
     " \[LineSeparator]gathers simple Lie ",
     Cell[BoxData[
      StyleBox["algebras", "TI"]], "InlineFormula"],
     " to a product algebra."
    }]]}
  }]], "Usage",
 GridBoxOptions->{
 GridBoxBackground->{
  "Columns" -> {{None}}, "ColumnsIndexed" -> {}, "Rows" -> {{None}}, 
   "RowsIndexed" -> {}}},
 CellID->982511436],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["EXAMPLES",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "PrimaryExamplesSection",
 WholeCellGroupOpener->True,
 CellTags->"PrimaryExamplesSection",
 CellID->1576967512],

Cell[CellGroupData[{

Cell[TextData[{
 "Basic Examples",
 "\[NonBreakingSpace]\[NonBreakingSpace]",
 Cell["(1)", "ExampleCount"]
}], "ExampleSection", "ExampleSection",
 WholeCellGroupOpener->True,
 CellID->503424393],

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<LieART`\>\"", "]"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->184741312],

Cell["\<\
Entering the product algebra corresponding to the Standard-Model gauge group:\
\
\>", "ExampleText",
 CellID->1691444968],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ProductAlgebra", "[", 
  RowBox[{"SU3", ",", "SU2", ",", "U1"}], "]"}]], "Input",
 CellLabel->"In[2]:=",
 CellID->435008270],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"SU(3)\"\>",
    TraditionalForm], "\[Times]", 
   FormBox["\<\"SU(2)\"\>",
    TraditionalForm], "\[Times]", 
   FormBox["\<\"U(1)\"\>",
    TraditionalForm]}], TraditionalForm]], "Output",
 ImageSize->{136, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[2]=",
 CellID->538414704]
}, Open  ]],

Cell["\<\
ProductAlgebra is mostly used to specify the semi-simple or non-semisimple \
subalgebra in subalgebra decompositions:\
\>", "ExampleText",
 CellID->172946751],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DecomposeIrrep", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Irrep", "[", "SU5", "]"}], "[", "10", "]"}], ",", 
   RowBox[{"ProductAlgebra", " ", "[", 
    RowBox[{"SU3", ",", "SU2", ",", "U1"}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[3]:=",
 CellID->1040201664],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ")", "\[InvisibleSpace]", 
    FormBox["\<\"(-6)\"\>",
     TraditionalForm]}], "+", 
   RowBox[{"(", 
    FormBox[
     StyleBox[
      OverscriptBox["3", "_"],
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ")", "\[InvisibleSpace]", 
    FormBox["\<\"(4)\"\>",
     TraditionalForm]}], "+", 
   RowBox[{"(", 
    FormBox[
     StyleBox["3",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["2",
      FontWeight->Bold],
     TraditionalForm], ")", "\[InvisibleSpace]", 
    FormBox["\<\"(-1)\"\>",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 ImageSize->{217, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[3]=",
 CellID->369550249]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[ButtonBox["MORE ABOUT",
 BaseStyle->None,
 Appearance->{Automatic, None},
 Evaluator->None,
 Method->"Preemptive",
 ButtonFunction:>(FrontEndExecute[{
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], All, ButtonCell], 
    FrontEndToken["OpenCloseGroup"], 
    FrontEnd`SelectionMove[
     FrontEnd`SelectedNotebook[], After, 
     CellContents]}]& )]], "MoreAboutSection",
 WholeCellGroupOpener->True,
 CellID->38303248],

Cell[TextData[ButtonBox["LieART: Lie Algebras and Representation Theory",
 BaseStyle->"Link",
 ButtonData->"paclet:LieART/guide/LieART"]], "MoreAbout",
 CellID->1280746216]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"ProductAlgebra - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "SymbolColor", "LinkTrails" -> 
  GridBox[{{
      RowBox[{
        ButtonBox[
        "LieART: Lie Algebras and Representation Theory", ButtonData -> 
         "paclet:LieART/guide/LieART", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "Metadata" -> {
   "built" -> "{2014, 8, 6, 10, 23, 57.714431}", 
    "history" -> {"0.9", "", "", ""}, "context" -> "LieART`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Lie A R T Symbol", "language" -> "en", "paclet" -> "LieART", "status" -> 
    "None", "summary" -> 
    "ProductAlgebra[algebras] gathers simple Lie algebras to a product \
algebra.", "synonyms" -> {}, "title" -> "ProductAlgebra", "titlemodifier" -> 
    "", "windowtitle" -> "ProductAlgebra", "type" -> "Symbol", "uri" -> 
    "LieART/ref/ProductAlgebra"}, "SearchTextTranslated" -> ""},
CellContext->"Global`",
FrontEndVersion->"9.0 for Linux x86 (64-bit) (February 7, 2013)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> "Global`"], 
   Cell[
    StyleData["Output"], CellContext -> "Global`"]}, Visible -> False, 
  FrontEndVersion -> "9.0 for Linux x86 (64-bit) (February 7, 2013)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "PrimaryExamplesSection"->{
  Cell[3537, 99, 491, 14, 29, "PrimaryExamplesSection",
   CellTags->"PrimaryExamplesSection",
   CellID->1576967512]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"PrimaryExamplesSection", 9096, 292}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[607, 21, 2349, 54, 49, "AnchorBarGrid",
 CellID->1],
Cell[2959, 77, 57, 1, 42, "ObjectName",
 CellID->1224892054],
Cell[3019, 80, 493, 15, 86, "Usage",
 CellID->982511436],
Cell[CellGroupData[{
Cell[3537, 99, 491, 14, 29, "PrimaryExamplesSection",
 CellTags->"PrimaryExamplesSection",
 CellID->1576967512],
Cell[CellGroupData[{
Cell[4053, 117, 195, 6, 25, "ExampleSection",
 CellID->503424393],
Cell[4251, 125, 115, 3, 28, "Input",
 CellID->184741312],
Cell[4369, 130, 131, 4, 34, "ExampleText",
 CellID->1691444968],
Cell[CellGroupData[{
Cell[4525, 138, 148, 4, 28, "Input",
 CellID->435008270],
Cell[4676, 144, 371, 13, 37, "Output",
 CellID->538414704]
}, Open  ]],
Cell[5062, 160, 168, 4, 49, "ExampleText",
 CellID->172946751],
Cell[CellGroupData[{
Cell[5255, 168, 287, 8, 28, "Input",
 CellID->1040201664],
Cell[5545, 178, 1023, 41, 37, "Output",
 CellID->369550249]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[6629, 226, 448, 13, 29, "MoreAboutSection",
 CellID->38303248],
Cell[7080, 241, 172, 3, 18, "MoreAbout",
 CellID->1280746216]
}, Open  ]],
Cell[7267, 247, 23, 0, 44, "FooterCell"]
}
]
*)

(* End of internal cache information *)

