(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[   2725950,      77254]
NotebookOptionsPosition[   2721208,      77104]
NotebookOutlinePosition[   2721699,      77122]
CellTagsIndexPosition[   2721656,      77119]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Tables of Branching Rules", "Title"],

Cell[TextData[{
 "following R. Slansky, ",
 StyleBox["\"Group Theory for Unified Model Building\"",
  FontSlant->"Italic"],
 ", Phys.Rept. ",
 StyleBox["79",
  FontWeight->"Bold"],
 " (1981) 1 - 128"
}], "Subsubtitle"],

Cell[CellGroupData[{

Cell["Initialization", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"<<", "LieART`Tables`"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 FormBox["\<\"LieART 1.1.4\"\>", TraditionalForm]], "Print",
 CellChangeTimes->{3.616237062650613*^9, 3.616238784339279*^9, 
  3.616299468971784*^9}],

Cell[BoxData[
 FormBox["\<\"last revised 5 August 2014\"\>", TraditionalForm]], "Print",
 CellChangeTimes->{3.616237062650613*^9, 3.616238784339279*^9, 
  3.6162994689724197`*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"$WriteToLaTeXFile", "=", "False"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
  "$BranchingRulesLaTeXFile", "=", "\"\<BranchingRulesTables.tex\>\""}], 
  ";"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"FileExistsQ", "[", "$BranchingRulesLaTeXFile", "]"}], ",", 
    RowBox[{"DeleteFile", "[", "$BranchingRulesLaTeXFile", "]"}]}], "]"}], 
  ";"}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["SU(N)", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU2", ",", 
   RowBox[{"{", "U1", "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "19"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "19"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"0.1\"\>",
      0.13781299999999999106314874097733991221`5.159890100096288,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    0.13781299999999999106314874097733991221`5.159890100096288, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615255836200057*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "1", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(2)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(2)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"U(1)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-9)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-11)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"13", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-12)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-13)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-14)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(15)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-15)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(16)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-16)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(17)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(15)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-1)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-3)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-5)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-7)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-9)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-11)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-13)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-15)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-17)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"19", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"(18)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(16)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(0)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-2)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-4)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-6)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-8)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-10)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-12)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-14)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-16)\"\>",
                TraditionalForm], "+", 
               FormBox["\<\"(-18)\"\>",
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615255836265291*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU3", ",", 
   RowBox[{"{", 
    RowBox[{"ProductAlgebra", "[", 
     RowBox[{"SU2", ",", "U1"}], "]"}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "63"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "9"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"0.3\"\>",
      0.30276900000000001034194951898825820535`5.501711319714829,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    0.30276900000000001034194951898825820535`5.501711319714829, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615255836586982*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "2", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(3)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(3)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"42", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"48", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"60", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"63", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6152558366514263`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU4", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "U1"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "105"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "6"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"0.7\"\>",
      0.67263899999999998691180280729895457625`5.848381957634634,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    0.67263899999999998691180280729895457625`5.848381957634634, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6152558377382307`*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "3", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(4)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(4)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"60", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(4)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"60", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615255837884325*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU5", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU3", ",", "SU2", ",", "U1"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "175"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "5"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"0.9\"\>",
      0.94820700000000002205524651799350976944`5.997503070390121,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    0.94820700000000002205524651799350976944`5.997503070390121, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615255838923415*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "4", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(5)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(5)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"40", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"75", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"160", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(5)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"40", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"50", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"75", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"160", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6152558390664577`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU6", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU3", ",", "SU3", ",", "U1"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "336"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "5"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"2.6\"\>",
      2.5680990000000001316493580816313624382`6.4302116750148235,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    2.5680990000000001316493580816313624382`6.4302116750148235, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6152558417189198`*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "5", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(6)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(6)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["70",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(25)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["70",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(6)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(6)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"15", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"20", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"175", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615255841973948*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU7", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SU3", ",", "U1"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "588"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "5"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"7.0\"\>",
      6.97053100000000025460167307755909860134`6.863865776253669,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    6.97053100000000025460167307755909860134`6.863865776253669, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6152558490761423`*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "6", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(7)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(7)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"48", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"112", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"196", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["105",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"392", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(30)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"490", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["105",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"490", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["105",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"588", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(7)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"48", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"112", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"196", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["70",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"392", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(25)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["70",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"490", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"490", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"588", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(7)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"48", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"112", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"140", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"196", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"392", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"490", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"490", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"588", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.61525584933768*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU8", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU7", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SU4", ",", "U1"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "790"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "5"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"23.7\"\>",
      23.7253830000000007771632226649671792984`7.395813145272273,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    23.7253830000000007771632226649671792984`7.395813145272273, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615255873415936*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "7", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(8)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"63", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["196", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"420", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"504", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["378", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["392",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"63", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"420", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"504", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"63", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"420", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"504", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"63", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"70", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"216", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"280", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"336", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"420", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"504", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615255873690638*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU9", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU8", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU7", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "SU4", ",", "U1"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "1040"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "4"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"35.3\"\>",
      35.25562099999999787769411341287195682526`7.567828282115348,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    35.25562099999999787769411341287195682526`7.567828282115348, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615255911079797*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "8", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(9)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(9)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"80", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"240", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"396", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-32)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["420",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["630",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1008", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["378", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["504",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(9)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"80", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"240", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"396", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["196", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["378", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1008", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["112", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["224",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(9)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"80", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"240", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"396", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1008", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(9)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"80", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"240", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"396", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"630", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1008", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(25)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6152559113416767`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU10", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU9", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU8", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU7", ",", "SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "SU4", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "SU5", ",", "U1"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "1000"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "4"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"171.0\"\>",
      171.44844299999999748251866549253463745117`8.254733458726045,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    171.44844299999999748251866549253463745117`8.254733458726045, {3, 1}], 
    "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615256085582604*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "9", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(10)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-27)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["396",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-36)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-26)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["630",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["15",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["196", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"99", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"715", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"990", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.61525608586572*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU11", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU10", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU9", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU8", ",", "SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU7", ",", "SU4", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "SU5", ",", "U1"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "594"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"104.0\"\>",
      103.77233900000000232921593124046921730042`8.03668151900223,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    103.77233900000000232921593124046921730042`8.03668151900223, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615256206060774*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "10", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(11)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-30)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-27)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-27)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["8", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["63",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["216",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(11)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"11", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"55", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"165", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"286", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-24)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(9)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-18)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"462", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-30)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(25)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615256206242445*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SU12", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU11", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU10", ",", "SU2", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU9", ",", "SU3", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU7", ",", "SU5", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "SU6", ",", "U1"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "792"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"449.0\"\>",
      449.4642630000000167456164490431547164917`8.673295079874126,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    449.4642630000000167456164490431547164917`8.673295079874126, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6152566638320503`*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "11", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(12)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(12)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(11)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-22)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"143", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["11", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-33)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["286",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"572", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["594",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(12)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"143", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"572", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["55",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["99",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["440",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["252",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(12)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(9)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"143", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["165",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"572", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["240",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["9", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["80",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(12)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(7)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"143", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-28)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"572", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-21)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["7", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-19)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(17)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-35)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-23)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(25)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SU(12)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"12", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"66", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"143", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"220", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"572", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"792", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615256664018915*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SO(N)", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO7", ",", 
   RowBox[{"{", "SU4", "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "2500"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "8"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.616224211698566*^9, 3.616224212094411*^9}}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"10.5\"\>",
      10.53910800000000058673776948126032948494`7.043403768267043,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    10.53910800000000058673776948126032948494`7.043403768267043, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615256718409889*^9, 3.616230142779251*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "1", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(7)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(7)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(4)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["4", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"21", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"48", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["4", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["20", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"77", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"105", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"112", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["20", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["36", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"112", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["36", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["4", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["20", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["60", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"168", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"182", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"189", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"294", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["70", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"330", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["70", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"378", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["196",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"448", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["4", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["20", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["60", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["140",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["20", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["36", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["60", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["140",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["140", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["36", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["84",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["140",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["140", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"616", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["126", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"672", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["56",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["56", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["120", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["160",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["160", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"693", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["300",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"714", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["196",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["336",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["60", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["140",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["140", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["160",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["160", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"819", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["384",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"825", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["300",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["300",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1008", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["84",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["120", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["140",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["140", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["160",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["160", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1008", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["4", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["20", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["60", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["140",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["280",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1254", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["196",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["336",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["540",
                  FontWeight->Bold], "\[Prime]\[Prime]\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1386", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["35", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["70", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["140",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["256", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["300",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1386", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["84",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["189", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["270",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["270", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["300",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1512", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["20", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["36", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["60", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["140",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["140", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["140",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["360",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["360", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["10",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["10", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["126", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["280",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["280", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["384",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1617", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["45",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["45", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["70", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["126", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["256", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["300",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["36", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["84",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["140",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["140", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["224",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["224", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["360",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["360", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1750", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["15",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["196",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["384",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["735",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1911", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["64",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["300",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["384",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["729",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2016", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["4",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["4", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["20",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["20", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["60",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["60", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["140",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["280",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["504",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["504", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2079", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["126", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["175",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["256", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["270",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["270", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["300",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["300",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2079", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["6",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["20",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["50",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["196",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["336",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["540",
                  FontWeight->Bold], "\[Prime]\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["825",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2310", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["140",
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold], "\[Prime]\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["189", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["256",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["256", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["270",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["270", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["300",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["300",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615256718535226*^9, 3.616230142963236*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO8", ",", 
   RowBox[{"{", "SO7", "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "2500"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "6"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"19.8\"\>",
      19.82567799999999991200638760346919298172`7.317827961557984,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    19.82567799999999991200638760346919298172`7.317827961557984, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615256738487112*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "13", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(8)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(8)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(7)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["8",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["8",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"28", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["35",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["35",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"35", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"112", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"112", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               SuperscriptBox[
                StyleBox["112",
                 FontWeight->Bold], "\[Prime]"],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"112", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               SuperscriptBox[
                StyleBox["112",
                 FontWeight->Bold], "\[Prime]"],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"160", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"160", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["112",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"160", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["112",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["112",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["168",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["112",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"224", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["168",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"294", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["294",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"294", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["294",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"294", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"300", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"350", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"567", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"567", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"567", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"672", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["294",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"672", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["294",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"672", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"672", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["168",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["448",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"672", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"672", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["168",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["448",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"672", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "s", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["378",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"672", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "v", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["672",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"672", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "c", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["672",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"840", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"840", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["112",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["168",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"840", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["112",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["168",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"840", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "s", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["294",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"840", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "c", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["616",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"840", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "v", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["616",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1296", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["616",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1296", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["112",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1296", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["112",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1386", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               SuperscriptBox[
                StyleBox["1386",
                 FontWeight->Bold], "\[Prime]"],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1386", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               SuperscriptBox[
                StyleBox["1386",
                 FontWeight->Bold], "\[Prime]"],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1386", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["378",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["714",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1400", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["168",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["693",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1400", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["168",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["720",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1400", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["168",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["720",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1568", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "s", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["7",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["21",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["27",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["105",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["182",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["819",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1568", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "v", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1008",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1568", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "c", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1008",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1680", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["294",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1386",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1680", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["294",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1386",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1680", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["672",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1008",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1680", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["168",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["448",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["1008",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1680", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["672",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1008",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1680", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["48",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["168",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["448",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["1008",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["77",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["330",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["693",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["825",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2400", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "sv", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["616",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2400", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "sc", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["35",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["189",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["616",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2400", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "vc", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["672",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["720",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1008",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2400", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "vs", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1728",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2400", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "cv", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["672",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["720",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1008",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2400", "\[InvisibleSpace]", 
                  SubsuperscriptBox["\[AlignmentMarker]", "cs", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SuperscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["560",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1728",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615256738557097*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO9", ",", 
   RowBox[{"{", 
    RowBox[{"SO8", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU4", ",", "SU2"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "5000"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "6"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"21.7\"\>",
      21.74027699999999896363078732974827289581`7.357864986553594,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    21.74027699999999896363078732974827289581`7.357864986553594, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615256761113978*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "14", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(9)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(9)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(8)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "c"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"44", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"128", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "s"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"156", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"231", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"432", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "c"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"450", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["294",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["300",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "vs"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "vc"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["350",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"672", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "sc"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "cs"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "sc"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "cs"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["567",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "cv"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "sv"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["350",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1122", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["294",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubsuperscriptBox[
                 StyleBox["672",
                  FontWeight->Bold], "v", "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["300",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["350",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["840",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "vs"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "vc"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["672",
                  FontWeight->Bold], "vs"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["672",
                  FontWeight->Bold], "vc"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1980", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["300",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["840",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubsuperscriptBox[
                 StyleBox["840",
                  FontWeight->Bold], "v", "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2457", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["350",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["567",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["1296",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2508", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["294",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubsuperscriptBox[
                 StyleBox["672",
                  FontWeight->Bold], "v", "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["1386",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "vs"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "vc"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["840",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["840",
                  FontWeight->Bold], "s"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2574", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["300",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["567",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["1400",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2772", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "cv"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "sv"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["350",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["567",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["567",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["840",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2772", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["294",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["294",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["672",
                  FontWeight->Bold], "cs"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["672",
                  FontWeight->Bold], "sc"],
                TraditionalForm], "+", 
               FormBox[
                SubsuperscriptBox[
                 StyleBox["840",
                  FontWeight->Bold], "v", "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2772", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["8",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["160",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["294",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["567",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["1568",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3900", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["35",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["56",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "cv"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "sv"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["350",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SubsuperscriptBox[
                 StyleBox["840",
                  FontWeight->Bold], "s", "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SubsuperscriptBox[
                 StyleBox["840",
                  FontWeight->Bold], "c", "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["1296",
                  FontWeight->Bold], "v"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["567",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["1400",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1925",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4158", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["567",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["567",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["672",
                  FontWeight->Bold], "cs"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["672",
                  FontWeight->Bold], "sc"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["840",
                  FontWeight->Bold], "v"],
                TraditionalForm], "+", 
               FormBox[
                SubsuperscriptBox[
                 StyleBox["840",
                  FontWeight->Bold], "v", "\[Prime]"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4608", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["112",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "sc"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "cs"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["672",
                  FontWeight->Bold], "sv"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["672",
                  FontWeight->Bold], "cv"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["1296",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["1296",
                  FontWeight->Bold], "c"],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4928", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "vs"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["224",
                  FontWeight->Bold], "vc"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["840",
                  FontWeight->Bold], "c"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["840",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["1400",
                  FontWeight->Bold], "s"],
                TraditionalForm], "+", 
               FormBox[
                SubscriptBox[
                 StyleBox["1400",
                  FontWeight->Bold], "c"],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(9)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"9", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"36", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"44", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"84", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"128", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"156", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"231", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"432", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"450", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"495", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"576", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"594", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"672", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"768", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"910", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"924", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1122", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["196",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1980", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2457", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2508", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["13",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["196",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2574", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2772", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2772", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2772", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  SuperscriptBox[
                   StyleBox["20",
                    FontWeight->Bold], "\[Prime]"],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["384",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3900", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["300",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["300",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4158", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["64",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4608", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["36",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["36", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["84",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4928", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["160", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6152567614274406`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO10", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SU2", ",", "SU4"}], "]"}], ",", "SO9", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU2", ",", "SO7"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "4000"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "5"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"22.1\"\>",
      22.14320299999999974716047290712594985962`7.365840354782301,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    22.14320299999999974716047290712594985962`7.365840354782301, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6152567844236727`*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "15", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(10)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"54", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"144", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"660", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["70",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["160", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["200",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"672", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(15)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["175", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"770", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["200",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"945", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1050", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["175", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1200", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1386", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["160", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["200",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-13)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["175",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["315",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1782", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["126",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["126", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["315",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["315", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["450",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["450", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["160", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["200",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["224", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["280",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["450",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["480", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2772", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(20)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(16)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["175", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["280",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["490",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["700",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2970", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["480",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["480", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3696", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(11)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(3)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["175",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["175", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["200",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["450",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["480", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3696", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(14)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(6)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["50",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["160",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["175", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["315", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["480", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(4)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"54", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"144", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"660", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"672", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"770", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"945", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1050", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1200", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1386", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold], "\[Prime]\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1782", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["196",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["140",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["140", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2772", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["35", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2970", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3696", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["4", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["4", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["20", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["20", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["36",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["36", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["60",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["60", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["140",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["140", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3696", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["20",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["64",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(9)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["16",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"54", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["126",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"144", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["156",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["432",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"660", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["156",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["450",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"672", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               StyleBox["672",
                FontWeight->Bold],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["576",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"770", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["495",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"945", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["594",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1050", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1200", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["432",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["768",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1386", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["156",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["910",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["672",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["768",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["126",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["594",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1782", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["9",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["44",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["156",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["450",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1122",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["16",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["576",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1920",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2772", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              FormBox[
               SuperscriptBox[
                StyleBox["2772",
                 FontWeight->Bold], "\[Prime]"],
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2970", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["231",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["495",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["594",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1650",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3696", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["432",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["576",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2560",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3696", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["924",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2772",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(10)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(7)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"10", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"16", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{"(", 
               FormBox[
                StyleBox["2",
                 FontWeight->Bold],
                TraditionalForm], ",", 
               FormBox[
                StyleBox["8",
                 FontWeight->Bold],
                TraditionalForm], ")"}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"45", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"54", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"120", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"126", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"144", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"210", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"560", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"660", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"672", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"720", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"770", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"945", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1050", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1200", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1386", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["7",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["330",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1440", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1782", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["9",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["182",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["378",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2640", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["448",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2772", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["294",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2970", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["7",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3696", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["4",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["48",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["48",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["112",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["168",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["512",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3696", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["168",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["378",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6152567849698277`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO14", ",", 
   RowBox[{"{", 
    RowBox[{"ProductAlgebra", "[", 
     RowBox[{"SU2", ",", "SU2", ",", "SO10"}], "]"}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "8000"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "4"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"18.4\"\>",
      18.35020700000000104523678601253777742386`7.2842408809658625,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    18.35020700000000104523678601253777742386`7.2842408809658625, {3, 1}], 
    "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6152568035231113`*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "16", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(14)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(14)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"64", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"91", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"104", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"364", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"546", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"832", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"896", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1001", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1716", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2002", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2275", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["660",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3080", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4004", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4928", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5265", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1386",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["720", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615256803629281*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO18", ",", 
   RowBox[{"{", 
    RowBox[{"ProductAlgebra", "[", 
     RowBox[{"SO8", ",", "SO10"}], "]"}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "5000"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"42.7\"\>",
      42.65793800000000146610545925796031951904`7.650599771967693,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    42.65793800000000146610545925796031951904`7.650599771967693, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6152568468496523`*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "17", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(18)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(18)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(8)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"18", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"153", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"170", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"256", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"816", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1122", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["112",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1920", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["160",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3060", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["35",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["28",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "v"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4352", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["56",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "s"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SubscriptBox[
                  StyleBox["8",
                   FontWeight->Bold], "c"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615256846893395*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO22", ",", 
   RowBox[{"{", 
    RowBox[{"ProductAlgebra", "[", 
     RowBox[{"SO12", ",", "SO10"}], "]"}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "17000"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"50.6\"\>",
      50.60687899999999928013494354672729969025`7.724809467840478,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    50.60687899999999928013494354672729969025`7.724809467840478, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615256899384322*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "18", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(22)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(22)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"22", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"231", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"252", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1024", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1540", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3520", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7315", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615256899454019*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"SO26", ",", 
   RowBox[{"{", 
    RowBox[{"ProductAlgebra", "[", 
     RowBox[{"SO16", ",", "SO10"}], "]"}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "700000"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "1"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"1070.0\"\>",
      1065.94330599999989317439030855894088745117`9.04833401989689,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    1065.94330599999989317439030855894088745117`9.04833401989689, {3, 1}], 
    "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615257966993093*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "19", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(26)\"", TraditionalForm], "\" Branching Rules\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox["\<\"SO(26)\"\>",
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(16)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"26", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"325", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2600", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"4096", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"14950", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"52325", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7020",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"65780", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4368",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"102400", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["128",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["128", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1920",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1920", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"230230", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4368",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8008",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"320320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7020",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24192",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"450450", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["135",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1344",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2970",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5304",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7020",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["36720",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"657800", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1820",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4368",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8008",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["11440",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615257967628932*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exceptional Algebras", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"E6", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO10", ",", "U1"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU3", ",", "SU3", ",", "SU3"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "19000"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"75.8\"\>",
      75.8222530000000034533513826318085193634`7.900396598288668,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    75.8222530000000034533513826318085193634`7.900396598288668, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.615258043599415*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "20", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox[
           SubscriptBox["\"E\"", "6"], TraditionalForm], 
          "\" Branching Rules\""}, "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "6"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(10)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"U(1)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["16",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(0)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-12)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["672", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1050", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-9)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["144", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-3)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(6)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(3)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1050", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(0)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1200", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-3)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7371", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1200",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7722", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-10)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["54",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["144",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["210",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["720", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["720",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1050", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1440",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["16", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["16",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["120",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(2)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["144", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["144",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(8)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["320",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["560",
                   FontWeight->Bold],
                  TraditionalForm], ")", "\[InvisibleSpace]", 
                 FormBox["\<\"(-1)\"\>",
                  TraditionalForm]}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-7)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["770",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["945",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1050",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-4)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1200", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(5)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1200",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2970",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(2)\"\>",
                 TraditionalForm]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3696",
                  FontWeight->Bold],
                 TraditionalForm], ")", "\[InvisibleSpace]", 
                FormBox["\<\"(-1)\"\>",
                 TraditionalForm]}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "6"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["384", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["405",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["490", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["896", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7371", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["384", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["384", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["840",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1050", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7722", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["210", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["384", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["384", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["420",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["840",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1176", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["2",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["315", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["384", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["384", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["840",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1050", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1050", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1260",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1701", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "6"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"78", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"351", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", "\[Prime]"]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"650", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1728", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2430", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"2925", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3003", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"5824", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7371", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"4", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7722", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"17550", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"5", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["24", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615258044118631*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"E7", ",", 
   RowBox[{"{", 
    RowBox[{"SU8", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SO12", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU6", ",", "SU3"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "28000"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.615287783986125*^9, 3.615287811418108*^9}, {
  3.615311804753172*^9, 3.615311808594244*^9}, {3.616237055029337*^9, 
  3.616237059634656*^9}, {3.616239089398303*^9, 3.6162390944929943`*^9}}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"85.3\"\>",
      85.34478000000000008640199666842818260193`7.951776876498113,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    85.34478000000000008640199666842818260193`7.951776876498113, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616299554560072*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "1", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox[
           SubscriptBox["\"E\"", "7"], TraditionalForm], 
          "\" Branching Rules\""}, "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "7"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(8)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"133", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"912", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["36", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["420",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["420", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1463", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["336",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["336", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["720",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1539", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["378", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["720",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6480", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["420",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["420", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1280",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1280", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1512", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7371", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["70",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["720",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1232",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1764",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3584",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8645", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["63",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["378",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["378", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["945",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["945", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2352",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3584",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["28",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["28", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1512",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1512", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox["2520",
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                SuperscriptBox[
                 StyleBox[
                  OverscriptBox["2520", "_"],
                  FontWeight->Bold], "\[Prime]"],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8100",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["8100", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27664", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["36",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["36", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["420",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["420", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1176",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1176", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1280",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1280", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2100",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["2100", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8820",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["8820", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "7"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SO(12)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"133", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"912", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1463", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["462",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1539", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["77",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6480", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7371", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["462", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1638",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1728", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8645", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["32", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["66",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["352", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["462", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["495",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1728", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2079",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["352",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4224",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4752",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27664", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["12",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["32",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["220",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["352",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["792",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2112",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["4928",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["4928",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "7"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(6)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"56", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"133", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"912", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1463", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"1539", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"6480", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"7371", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["105",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["384",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["384", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["405",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"8645", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["15",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["15", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["21",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["21", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["35",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["35",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["105",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["105", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["105",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["105", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["189",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["280",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["280", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["384",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["384", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"24320", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["336", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["840",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["840", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["980",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27664", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["6", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["6",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["20",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["20",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["56", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["70",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["70", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["84",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["84", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["84",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["84", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["120",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["120", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["210",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["210", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["336",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["336", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["540",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["560",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["560", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["840",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["840", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615288301833652*^9, 3.615793938621962*^9, 
  3.6162261123709106`*^9, 3.6162310137997503`*^9, 3.6162995562584267`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BranchingRulesTable", "[", 
  RowBox[{"E8", ",", 
   RowBox[{"{", 
    RowBox[{"SO16", ",", "SU9", ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"E7", ",", "SU2"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"E6", ",", "SU3"}], "]"}], ",", 
     RowBox[{"ProductAlgebra", "[", 
      RowBox[{"SU5", ",", "SU5"}], "]"}]}], "}"}], ",", 
   RowBox[{"MaxDim", "\[Rule]", "31000"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"82.8\"\>",
      82.83477899999999749525159131735563278198`7.938812631181554,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    82.83477899999999749525159131735563278198`7.938812631181554, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6152774953049173`*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "1", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox[
           SubscriptBox["\"E\"", "8"], TraditionalForm], 
          "\" Branching Rules\""}, "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "8"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SO(16)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["135",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1820",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1920",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["128",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1820",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5304",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["6435", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13312",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["120",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1920",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["7020",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["8008",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["13312",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "8"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              FormBox["\<\"SU(9)\"\>",
               TraditionalForm]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["84", "_"],
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["240",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["240", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1050",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1050", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1215",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["84", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1050",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1050", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1215",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1944",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["2520",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["2520", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5346",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["5346", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5760",
                 FontWeight->Bold],
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                StyleBox["1",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["80",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["84",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["84", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["240",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["240", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1050",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1050", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1215",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["1540",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["1540", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["3402",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["3402", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5346",
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox[
                 OverscriptBox["5346", "_"],
                 FontWeight->Bold],
                TraditionalForm], "+", 
               FormBox[
                StyleBox["5760",
                 FontWeight->Bold],
                TraditionalForm]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "8"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox["\<\"E\"\>", "7"],
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(2)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["133",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["133",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["912",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1539",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["133",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1463",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1539",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6480",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["7371",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["56",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["4",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["133",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["133",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["912",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1463",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1539",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["6480",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["2",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["8645",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "8"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox[
                SubscriptBox["\<\"E\"\>", "6"],
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(3)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["27", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["27", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["27", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["351", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["650",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["27", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["27", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["27", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["351", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["351",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["351", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["650",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["650",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1728", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2430",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["27",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["3",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["27", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["3", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["27", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["27",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["27", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["78",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["78",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["8",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["351", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox["351",
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 SuperscriptBox[
                  StyleBox[
                   OverscriptBox["351", "_"],
                   FontWeight->Bold], "\[Prime]"],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["351",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["6", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["351", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["6",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["650",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["650",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["8",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1728",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["3",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["1728", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["3", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["2925",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]},
     {
      TagBox[GridBox[{
         {
          TagBox[GridBox[{
             {
              FormBox[
               SubscriptBox["\<\"E\"\>", "8"],
               TraditionalForm], 
              FormBox["\<\"\[RightArrow]\"\>",
               TraditionalForm], 
              RowBox[{
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm], "\[Times]", 
               FormBox["\<\"SU(5)\"\>",
                TraditionalForm]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"248", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"3875", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"27000", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["200",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["200",
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]},
             {
              FormBox[
               TagBox[
                StyleBox[
                 RowBox[{"30380", "\[InvisibleSpace]", 
                  SuperscriptBox["\[AlignmentMarker]", ""]}],
                 StripOnInput->False,
                 FontWeight->Bold],
                DisplayForm],
               TraditionalForm], 
              FormBox["\<\"=\"\>",
               TraditionalForm], 
              RowBox[{
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["1",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["5",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["40", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["40",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["40", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["5", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"3", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["24",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox["45",
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox["10",
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"2", 
                RowBox[{"(", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["10", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ",", 
                 FormBox[
                  StyleBox[
                   OverscriptBox["45", "_"],
                   FontWeight->Bold],
                  TraditionalForm], ")"}]}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["15",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["15", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["50",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["50", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["70",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["10",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["10", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["70", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["45",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["40",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["40", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["45", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["75",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["24",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["126",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["1",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["126", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox["5",
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox["175",
                  FontWeight->Bold],
                 TraditionalForm], ")"}], "+", 
               RowBox[{"(", 
                FormBox[
                 StyleBox[
                  OverscriptBox["175", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ",", 
                FormBox[
                 StyleBox[
                  OverscriptBox["5", "_"],
                  FontWeight->Bold],
                 TraditionalForm], ")"}]}]}
            },
            AutoDelete->False,
            
            GridBoxAlignment->{
             "Columns" -> {"\[AlignmentMarker]", Center, Left}, 
              "Rows" -> {Center, {Center}}},
            GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {
                RGBColor[0.8, 0.8, 0.8], {
                 GrayLevel[1], 
                 RGBColor[0.9, 0.9, 0.9]}}},
            GridBoxDividers->{"Columns" -> {
                GrayLevel[1], {False}, 
                GrayLevel[1]}, "Rows" -> {True, True, {False}, True}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            
            GridBoxSpacings->{
             "Columns" -> {1, 0.25, 0.5, 2}, 
              "Rows" -> {1, 1, {Automatic}, 0.5}}],
           "Grid"]}
        },
        AutoDelete->False,
        GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
        GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
        GridBoxSpacings->{"Columns" -> {{0}}, "Rows" -> {{0}}}],
       "Grid"]}
    },
    DefaultBaseStyle->"Column",
    GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Automatic}}},
    GridBoxItemSize->{"Columns" -> {{All}}, "Rows" -> {{All}}},
    GridBoxSpacings->{"Columns" -> {{Automatic}}, "Rows" -> {0, 0.6, {1}, 0}}],
   "Column"], TraditionalForm]], "Output",
 CellChangeTimes->{3.615277497405105*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
CellGrouping->Manual,
WindowSize->{1056, 988},
WindowMargins->{{Automatic, 223}, {Automatic, 0}},
PrivateNotebookOptions->{"VersionedStylesheet"->{"Default.nb"[8.] -> True}},
CommonDefaultFormatTypes->{"Output"->TraditionalForm},
FrontEndVersion->"9.0 for Linux x86 (64-bit) (February 7, 2013)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 42, 0, 93, "Title"],
Cell[624, 24, 218, 8, 35, "Subsubtitle"],
Cell[CellGroupData[{
Cell[867, 36, 33, 0, 77, "Section"],
Cell[CellGroupData[{
Cell[925, 40, 85, 2, 29, "Input",
 InitializationCell->True],
Cell[1013, 44, 163, 3, 70, "Print"],
Cell[1179, 49, 179, 3, 70, "Print"]
}, Open  ]],
Cell[1373, 55, 334, 10, 68, "Input",
 InitializationCell->True],
Cell[1710, 67, 225, 6, 31, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[1972, 78, 24, 0, 47, "Section"],
Cell[CellGroupData[{
Cell[2021, 82, 222, 5, 31, "Input"],
Cell[2246, 89, 532, 14, 22, "Print"],
Cell[2781, 105, 24682, 667, 416, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[27500, 777, 292, 7, 31, "Input"],
Cell[27795, 786, 532, 14, 22, "Print"],
Cell[28330, 802, 48316, 1300, 416, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[76683, 2107, 406, 10, 50, "Input"],
Cell[77092, 2119, 534, 14, 22, "Print"],
Cell[77629, 2135, 108765, 2991, 955, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[186431, 5131, 406, 10, 50, "Input"],
Cell[186840, 5143, 532, 14, 22, "Print"],
Cell[187375, 5159, 98222, 2695, 889, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[285634, 7859, 502, 12, 50, "Input"],
Cell[286139, 7873, 534, 14, 22, "Print"],
Cell[286676, 7889, 174471, 4809, 1504, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[461184, 12703, 502, 12, 50, "Input"],
Cell[461689, 12717, 534, 14, 22, "Print"],
Cell[462226, 12733, 183627, 5058, 1540, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[645890, 17796, 598, 14, 50, "Input"],
Cell[646491, 17812, 533, 14, 22, "Print"],
Cell[647027, 17828, 186176, 5147, 1623, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[833240, 22980, 599, 14, 50, "Input"],
Cell[833842, 22996, 531, 13, 22, "Print"],
Cell[834376, 23011, 167370, 4629, 1497, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1001783, 27645, 696, 16, 50, "Input"],
Cell[1002482, 27663, 539, 14, 22, "Print"],
Cell[1003024, 27679, 168565, 4675, 1620, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1171626, 32359, 696, 16, 50, "Input"],
Cell[1172325, 32377, 532, 13, 22, "Print"],
Cell[1172860, 32392, 109004, 3029, 1224, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1281901, 35426, 697, 16, 50, "Input"],
Cell[1282601, 35444, 534, 13, 22, "Print"],
Cell[1283138, 35459, 109212, 3036, 1222, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1392399, 38501, 24, 0, 77, "Section"],
Cell[CellGroupData[{
Cell[1392448, 38505, 290, 6, 31, "Input"],
Cell[1392741, 38513, 553, 13, 22, "Print"],
Cell[1393297, 38528, 65231, 1932, 890, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1458565, 40465, 224, 5, 31, "Input"],
Cell[1458792, 40472, 531, 13, 22, "Print"],
Cell[1459326, 40487, 60100, 1785, 1410, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1519463, 42277, 324, 8, 31, "Input"],
Cell[1519790, 42287, 531, 13, 22, "Print"],
Cell[1520324, 42302, 214031, 6278, 1759, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1734392, 48585, 506, 12, 50, "Input"],
Cell[1734901, 48599, 533, 13, 22, "Print"],
Cell[1735437, 48614, 355698, 10197, 2915, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2091172, 58816, 309, 7, 31, "Input"],
Cell[2091484, 58825, 540, 14, 22, "Print"],
Cell[2092027, 58841, 73368, 2164, 520, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2165432, 61010, 297, 7, 31, "Input"],
Cell[2165732, 61019, 533, 13, 22, "Print"],
Cell[2166268, 61034, 19853, 576, 246, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2186158, 61615, 299, 7, 31, "Input"],
Cell[2186460, 61624, 531, 13, 22, "Print"],
Cell[2186994, 61639, 13857, 403, 208, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2200888, 62047, 300, 7, 31, "Input"],
Cell[2201191, 62056, 540, 14, 22, "Print"],
Cell[2201734, 62072, 39558, 1156, 386, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2241341, 63234, 39, 0, 77, "Section"],
Cell[CellGroupData[{
Cell[2241405, 63238, 494, 12, 50, "Input"],
Cell[2241902, 63252, 533, 14, 22, "Print"],
Cell[2242438, 63268, 278961, 8008, 1642, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2521436, 71281, 622, 13, 50, "Input"],
Cell[2522061, 71296, 531, 13, 22, "Print"],
Cell[2522595, 71311, 104907, 3062, 920, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2627539, 74378, 505, 12, 50, "Input"],
Cell[2628047, 74392, 533, 13, 22, "Print"],
Cell[2628583, 74407, 92585, 2692, 832, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
