(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[   1511634,      51532]
NotebookOptionsPosition[   1503515,      51289]
NotebookOutlinePosition[   1504214,      51313]
CellTagsIndexPosition[   1504171,      51310]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Tables of Tensor Products", "Title"],

Cell[TextData[{
 "following R. Slansky, ",
 StyleBox["\"Group Theory for Unified Model Building\"",
  FontSlant->"Italic"],
 ", Phys.Rept. ",
 StyleBox["79",
  FontWeight->"Bold"],
 " (1981) 1 - 128"
}], "Subsubtitle"],

Cell[CellGroupData[{

Cell["Initialization", "Section"],

Cell[BoxData[
 RowBox[{"<<", "LieART`Tables`"}]], "Input",
 InitializationCell->True],

Cell[BoxData[{
 RowBox[{
  RowBox[{"$WriteToLaTeXFile", "=", "False"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
  "$TensorProductsLaTeXFile", "=", "\"\<TensorProductsTables.tex\>\""}], 
  ";"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"FileExistsQ", "[", "$TensorProductsLaTeXFile", "]"}], ",", 
    RowBox[{"DeleteFile", "[", "$TensorProductsLaTeXFile", "]"}]}], "]"}], 
  ";"}]], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["SU(N)", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SU2", ",", 
   RowBox[{"MaxDim", "\[Rule]", "10"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "9"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "48"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "45"}],
   
   SequenceForm["Number of Tensor Products: ", 45],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6162181358183613`*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"0.8\"\>",
      0.76900100000000004563105449051363393664`5.9065268178329395,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    0.76900100000000004563105449051363393664`5.9065268178329395, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616218135841254*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "1", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(2)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["2",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["18",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616218135902975*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SU3", ",", 
   RowBox[{"MaxDim", "\[Rule]", "28"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "6"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "520"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "91"}],
   
   SequenceForm["Number of Tensor Products: ", 91],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616218144085372*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"8.2\"\>",
      8.16140299999999996316546457819640636444`6.932364736590155,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    8.16140299999999996316546457819640636444`6.932364736590155, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616218144104711*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "2", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(3)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["3", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["3", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["3", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["8",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["3", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["42", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["15",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["3", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["42", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["90", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["3", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["42", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["81", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["125",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["8",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["3", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["24", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["42", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["42", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["63", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["42", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["63", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["48", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["42", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["48", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["42", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["63", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["42", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["48", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["90", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["3", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["42", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["90", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["165", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["66", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["90", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["99", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["125",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["216",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["42", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["42", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["48", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["24",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["42", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["63", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["3", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["42", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["48", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["90", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["3", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["42", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["90", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["35",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["42", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["48", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["90", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["99", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["81", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["125",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["154", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["15", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["24", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["42", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["48", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["90", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["24", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["8",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["81", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["125",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["15",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["24",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["42",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["15",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["24",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["42",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["60",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["8",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["35",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["35", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["81", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["125",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["81", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["154", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["99",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["99",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["120",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["3",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["120",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["15",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["192",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["125",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["154",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["162",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["125",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["216",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["343",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["81", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["91",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["125",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["143",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["154",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["162",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6162181442845373`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SU4", ",", 
   RowBox[{"MaxDim", "\[Rule]", "50"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "4"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "1170"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "94"}],
   
   SequenceForm["Number of Tensor Products: ", 94],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616218154587075*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"10.2\"\>",
      10.1618969999999997355644154595211148262`7.027574701909566,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    10.1618969999999997355644154595211148262`7.027574701909566, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6162181546038733`*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "3", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(4)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["4", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["4", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["20", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["4", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["20", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["20", "_"],
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["15",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["20", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["20", "_"],
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["84", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["4", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["20", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["4", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["160", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["4", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["20", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["140", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["20", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["84",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["20", "_"],
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["84",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["20",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["84", "_"],
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["140", "_"],
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["140", "_"],
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["224",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["140",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["20", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["15",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["20", "_"],
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["300",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["20", "_"],
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["256",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["84",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["140",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["270", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["84",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["140",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["256",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["120",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["140",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["216",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["224",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["216",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["224",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["500",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["84",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["420",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["256",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["360",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["300",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["825",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["360",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["20", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["20", "_"],
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["140", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["20", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["20", "_"],
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["140", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["36",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["84",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["189", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["256", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["45", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["256", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["270", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["84",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["360",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["20", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["20", "_"],
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["140", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["420", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["540", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["84",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["280",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["360",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["420",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["15",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["84",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["256",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["256", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["300",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["70",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["140",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["270",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["84",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["140",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["45",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["256",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["140",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["216",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["224",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["140",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["140",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["224",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["540",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["36",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["84",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["360",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["256",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["256",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["300",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["875",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["256",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["360",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["20",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["60",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["140",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["360",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["540",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["60",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["84",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["140",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["360",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["420",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["15",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["84",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["256",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["256", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["300",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["729",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["175",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["256",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["360",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["140", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["384",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["20", "_"],
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["140", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["360", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["480", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["20", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["140", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["140", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["280", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["360", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["196",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["384",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["630",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["750",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["60", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["84", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["140", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["140", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["224", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["360", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["756", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["384",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["540",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["630",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["20",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["300",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["336",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["729",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["735",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616218154837469*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SU5", ",", 
   RowBox[{"MaxDim", "\[Rule]", "75"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "3000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "91"}],
   
   SequenceForm["Number of Tensor Products: ", 91],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616218168298293*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"13.4\"\>",
      13.38841500000000017678303265711292624474`7.1473290789773625,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    13.38841500000000017678303265711292624474`7.1473290789773625, {3, 1}], 
    "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616218168322723*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "4", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(5)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["5", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["5", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["40", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["5", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["50", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["5", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["40", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["200",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["50", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["70", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["24",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["200",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["160", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["70", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["280", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["224",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["5", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["450", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["126",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["175",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["224",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["315",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["175", "_"],
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["210", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["420", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["420", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["200",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1000",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["175", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["40", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["50", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["5", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["280", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["40", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["175",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["5", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["480", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["40", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["175",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["224",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["40",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["280", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["280", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["420", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["420", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["40", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["200",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["1050", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["50", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["175", "_"],
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["280", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["280", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["420", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["40", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["24",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["200",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1024",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["40", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["175", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["40", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["175", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["450", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["280",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["45",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["480",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["160", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["175", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["540", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["700", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["480",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["40", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["160", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["175", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["315", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["700", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["45",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["480",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["40",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["175",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["24",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["75",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["175",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["175", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["200",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1024",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["50", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["175",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["175", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["315", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["50", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["175",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["175", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["50", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["420",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["700", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["840", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["50", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["480",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1200",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["40", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["175", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["315", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["560", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["700", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["50", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["480",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1120",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["50", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["175",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["700", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1024",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["490",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["50", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["200",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1024",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1176",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["200",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["480",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["40", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["175", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["450", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["450",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["480",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["40", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["315", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["450", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["70",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["280",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["450",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["480",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["70",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["560",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["160", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["175", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["875", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1215", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["70",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["105",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["480",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["40", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["160", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["175", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["450", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1215", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["126", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["175", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["200",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["224", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1024",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["1050", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["175",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["700",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1215",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["175",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["175", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["224", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1024",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1701", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["700",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1890",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["24",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["200",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1000",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1024",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1050",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["1050", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["160",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["175",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["540",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["700",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["875",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1215",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["700",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["24",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["75",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["175",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["175", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1024",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1440",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["175",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["45",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["175",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["280",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["480",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1120",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["480",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["980",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1120",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["50",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["280",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["280",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["420",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["480",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2520",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["75",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["24",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["75",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["175",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["175", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["200",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["700",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["700", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1024",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1176",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616218168761383*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SU6", ",", 
   RowBox[{"MaxDim", "\[Rule]", "105"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "5670"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "90"}],
   
   SequenceForm["Number of Tensor Products: ", 90],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.61621818892944*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"20.1\"\>",
      20.07778599999999968872543831821531057358`7.323315734252836,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    20.07778599999999968872543831821531057358`7.323315734252836, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616218188945546*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "5", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(6)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["105", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["280", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["210", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["336", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["405",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["105", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["210", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["384",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["540",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["384",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["35",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["280", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["405",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["56", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["315", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["210", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["56", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["720", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["336", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["504", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["56", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["1050", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["252", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["420", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["504", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["700",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1134",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["56", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["405",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2695",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["490",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1050",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1134",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["384", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["105", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["210", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["840", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["336", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["420", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1260", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["336", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["420", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["504", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["70",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["540",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1134",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["405",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3200",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1050",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1134",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["35",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["280", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["405",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3675",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["490",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["896",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1134",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["384",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["540",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["840",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["105", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["384", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1050", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["540",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1134",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["840",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["84",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["336",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["840",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1260",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["210", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1701", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2688", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["384",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1575",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2430",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["105", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["105", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["210", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["384", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1050", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1701", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2205", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["384",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1176",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2430",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["105",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["105",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["384",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1176",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1701",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1800",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["35",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["280", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["405",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["896", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3675",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["105",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["540",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["105", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["420", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["336", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["105",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["384", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1176", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["384", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1050", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["105", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["490",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["105",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["840", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["1176", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["840", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["840", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["105",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["384", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1800", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["384", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1701", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["105", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1134",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["105",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["105",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2205",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["15",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["105",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["105",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["384",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1701",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["105",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1260", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4500", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["336", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1260", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4200", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["105", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2520",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["2520",
              FontWeight->Bold], "\[Prime]\[Prime]\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["840",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1800",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2520",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["105",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["840", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1260", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["5040", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["84", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["336", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["840", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1260", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4410", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["105", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["840",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1176",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1764",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2520",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["840",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1176",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2520",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["105",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["70", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["540",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1134", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1960",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4536", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["56", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["70", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["540",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1134", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1960",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["3240", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["105", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["336", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["420", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["840", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["2520", "_"],
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4410", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["6", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["84", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["210", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["336", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["420", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["840", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["840", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1260", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4410", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["105",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["105",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["105", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1050", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1176", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["1764", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["2520", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["4410", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["105",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["384", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1050", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1176", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1470", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2430", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["4410", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["15", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["105", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["315", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["384", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["1050", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1176", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1470", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2430", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["2520", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["105", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["280", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3675",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["5670", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["105", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["35",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["280", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["405",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["896", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3675",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3969",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6162181901232367`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SU7", ",", 
   RowBox[{"MaxDim", "\[Rule]", "112"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "3000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "46"}],
   
   SequenceForm["Number of Tensor Products: ", 46],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.61621820950602*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"19.2\"\>",
      19.22805800000000076011019700672477483749`7.304535336755058,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    19.22805800000000076011019700672477483749`7.304535336755058, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616218209524417*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "6", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(7)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["7", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["7", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["140", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["392",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["196", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["7", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["189", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["540", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["378", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["735",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["196", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["210", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["378", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["224", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["7", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["140", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["588", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["224", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["490", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["140", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["840", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["224", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["756", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["35", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["392",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["784",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["7", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["140", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["490", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["588", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["224",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["735",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["735",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1323",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["48",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["392",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["540",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["540", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["735",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["210", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["378", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["189", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1575", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["756", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1008", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["7", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["189", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2156", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["462", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["882", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1008", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["540", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2400", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["840", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2100", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2450",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["924", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1176", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2310", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["2646", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["112", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["21", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["735", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["196", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["378", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["112", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["7", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["140", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["189", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2016", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["224", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["490", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["756", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["882", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["112", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["7", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["140", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["189", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2800", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["490", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["756", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["882", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1008", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["112", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["392",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["540", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2940", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["140", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["588", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["840", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2352", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["112",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1260",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1323",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2450",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["2100", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2310", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2352", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["2646", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["490", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["588", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["840", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1176", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2100", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["2352", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["2646", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616218217101057*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SU8", ",", 
   RowBox[{"MaxDim", "\[Rule]", "168"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "5500"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "46"}],
   
   SequenceForm["Number of Tensor Products: ", 46],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6162183008166447`*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"82.8\"\>",
      82.80570400000000574891600990667939186096`7.93866016709835,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    82.80570400000000574891600990667939186096`7.93866016709835, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.61621830085221*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "7", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(8)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["8", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["8", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["216", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["336",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["8", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["280", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["945", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["630",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1232",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["336",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["630",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["56", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["420", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["56", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["8", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["216", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1344", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["56", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["504", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1008", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["56", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["216", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1800", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["504", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["1512", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["56", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2352",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["420", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1176", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1512", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["216",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["280",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["420",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1280",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1280",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["504",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2800",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["63",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["945", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1232",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["56", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["504", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["420", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1512", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["420", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2100", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["8", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["216", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1344", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["2352", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["378", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3584",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1764",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2352",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["924", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["630",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["280", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["3080", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["1512", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1848", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["8", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["280", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4032", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["792", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1680", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1848", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["2100", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4620", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["2520",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4752",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["168", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["28", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1280", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["336",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["630",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["168", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["8", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["216", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["280", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4200", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["504", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1008", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["1512", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1680", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["1008", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["1512", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1680", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1848", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["420", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1512", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2100", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["5376", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["63",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["168",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2520",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2800",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4752",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616218319740636*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SU9", ",", 
   RowBox[{"MaxDim", "\[Rule]", "240"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "2000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "22"}],
   
   SequenceForm["Number of Tensor Products: ", 22],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616218708060522*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"386.0\"\>",
      386.07716099999998959901859052479267120361`8.607274024292305,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    386.07716099999998959901859052479267120361`8.607274024292305, {3, 1}], 
    "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616218708085787*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "8", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(9)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["9", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["9", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["315", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["240",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1215",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["540",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["630",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["9", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["396", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["240",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1540", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["630",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["990",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1944",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["540",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["990",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["396",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["80",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1215",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1540",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1540", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1944",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["36", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["720", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["630",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1008", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1890", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["84", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1050", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1008", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["165", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1440", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["990",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["240",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["540",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["630",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["990",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6162187332560663`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SU10", ",", 
   RowBox[{"MaxDim", "\[Rule]", "210"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "2000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "15"}],
   
   SequenceForm["Number of Tensor Products: ", 15],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6162187854875298`*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"49.0\"\>",
      49.00505600000000328009264194406569004059`7.710840803096369,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    49.00505600000000328009264194406569004059`7.710840803096369, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616218785548381*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "9", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(10)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["99",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["440", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["99",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1925",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["825",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["990",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["55", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["10", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["540", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["990",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1485",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["825",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1485",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["99",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["440",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["540",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["45", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1155", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["990",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["210", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["120", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1980", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["252",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1848",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616218931785667*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SU11", ",", 
   RowBox[{"MaxDim", "\[Rule]", "165"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "1000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "7"}],
   SequenceForm["Number of Tensor Products: ", 7],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616219030229354*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"80.7\"\>",
      80.72253200000000106228981167078018188477`7.927594689109841,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    80.72253200000000106228981167078018188477`7.927594689109841, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616219030257701*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "10", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(11)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["11", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["55", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["11", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["594", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["440",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["66", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["11", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["715", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["440",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616219118273116*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SU12", ",", 
   RowBox[{"MaxDim", "\[Rule]", "220"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "1300"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "7"}],
   SequenceForm["Number of Tensor Products: ", 7],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6162231506138487`*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"230.0\"\>",
      230.47705199999998626481101382523775100708`8.383227603587036,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    230.47705199999998626481101382523775100708`8.383227603587036, {3, 1}], 
    "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6162231506532593`*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "1", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SU(12)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["12", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["143",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["66", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["12", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["780", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["572",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["78", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["12", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["924", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["364",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["572",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["143",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["780",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6162235375415707`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["SO(N)", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SO7", ",", 
   RowBox[{"MaxDim", "\[Rule]", "168"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "4"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "5000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "76"}],
   
   SequenceForm["Number of Tensor Products: ", 76],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6162235781373158`*^9, 3.616224176084648*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"4.2\"\>",
      4.22157400000000038176040106918662786484`6.64607431970658,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    4.22157400000000038176040106918662786484`6.64607431970658, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6162235781373158`*^9, 3.6162241761141157`*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "1", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(7)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["294",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["48",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["48",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["112",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["21",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["35",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["105",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["819",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["378",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["693",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["819",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1560",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1008",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["714",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["825",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1750",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1911",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["105",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["693",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["105",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["693",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["819",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1617",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["48",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["112",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["168",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["512",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["330",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1617",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1750",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1911",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["21",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["168",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["294",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["330",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["825",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1617",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1911",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["294",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1008",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["112",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["672",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1008",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["48",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["112",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["512",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1008",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["294",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["378",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1617",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["35",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["105",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["294",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["378",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["693",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1617",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4200",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3584",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["112",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["512",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1008",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["48",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["112",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["112",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["168",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["512",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1008",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2800",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["294",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["693",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["825",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1386",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1617",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2310",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["105",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["294",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["378",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["693",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1617",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2310",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["21",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["35",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["105",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["168",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["294",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["378",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["616",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["693",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["825",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1617",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["693",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["825",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1617",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["168",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["294",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1617",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1911",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1008",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["693",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1617",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["168",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["512",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["512",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1008",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2800",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["105",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["330",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["616",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["693",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["819",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1617",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["693",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["819",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4312",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4550",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1008",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2016",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2800",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3584",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["105",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["294",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["378",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["616",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["693",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["819",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1617",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3003",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4550",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["48",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["112",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["168",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["512",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1008",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1008",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1512",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2800",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3584",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["512",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1008",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2800",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["112",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["512",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["720",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1008",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2800",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3080",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["112",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["294",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["378",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["616",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["693",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1617",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["4095",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4550",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["112",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["105",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["168",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["294",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["330",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["378",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["616",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["693",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["819",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1617",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1911",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4550",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["168",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["21",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["35",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["105",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["168",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["330",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["378",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["378",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["616",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["693",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["819",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["825",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1617",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1750",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1911",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3375",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4550",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616224176368766*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SO8", ",", 
   RowBox[{"MaxDim", "\[Rule]", "160"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "4600"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "92"}],
   
   SequenceForm["Number of Tensor Products: ", 92],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616224185311417*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"8.9\"\>",
      8.86062699999999914268755674129351973534`6.968064368011541,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    8.86062699999999914268755674129351973534`6.968064368011541, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616224185331415*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "2", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(8)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sv"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cs"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cv"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vs"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vc"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "c"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "v", "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "c", "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["294",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "c"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s", "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["294",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sc"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sv"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vs"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cv"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vc"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["56",
               FontWeight->Bold], "c"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "c"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["56",
               FontWeight->Bold], "s"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["672",
              FontWeight->Bold], "cs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "c"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["672",
              FontWeight->Bold], "vs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "c"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["672",
              FontWeight->Bold], "cv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["672",
              FontWeight->Bold], "vc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["56",
               FontWeight->Bold], "c"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["160",
               FontWeight->Bold], "c"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "c"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["28",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["350",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "c", "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["56",
               FontWeight->Bold], "v"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["160",
               FontWeight->Bold], "v"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["28",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["350",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s", "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["294",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["672",
              FontWeight->Bold], "sv"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["672",
              FontWeight->Bold], "cs"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["672",
              FontWeight->Bold], "cv"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["672",
              FontWeight->Bold], "vs"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["672",
              FontWeight->Bold], "vc"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["294",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1568",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1568",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["2400",
              FontWeight->Bold], "sc"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["2400",
              FontWeight->Bold], "sv"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["672",
              FontWeight->Bold], "c", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1400",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1568",
              FontWeight->Bold], "c"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["2400",
              FontWeight->Bold], "cv"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["2400",
              FontWeight->Bold], "vs"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["2400",
              FontWeight->Bold], "vc"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["672",
              FontWeight->Bold], "v", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1400",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1568",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "c", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "v", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["3675",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "v", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["3675",
              FontWeight->Bold], "c"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["672",
              FontWeight->Bold], "cs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1680",
              FontWeight->Bold], "cv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["2800",
              FontWeight->Bold], "cv"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "c", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["3675",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["672",
              FontWeight->Bold], "vc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1680",
              FontWeight->Bold], "vs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["2800",
              FontWeight->Bold], "vs"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["672",
              FontWeight->Bold], "vs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1680",
              FontWeight->Bold], "vc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["2800",
              FontWeight->Bold], "vc"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["294",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1386",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1925",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["3696",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["4312",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["294",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1386",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1925",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["3696",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["4312",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "c"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "c"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["160",
               FontWeight->Bold], "s"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1400",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["160",
               FontWeight->Bold], "v"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1400",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["2800",
              FontWeight->Bold], "cs"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["2800",
              FontWeight->Bold], "vs"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["160",
               FontWeight->Bold], "c"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1400",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1568",
              FontWeight->Bold], "c"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "vc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["2800",
              FontWeight->Bold], "vc"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["2800",
              FontWeight->Bold], "sv"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["2800",
              FontWeight->Bold], "cv"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["160",
               FontWeight->Bold], "v"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["840",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1400",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1568",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["350",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "c", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "v", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["350",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "v", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "c"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["56",
               FontWeight->Bold], "v"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["160",
               FontWeight->Bold], "v"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["224",
               FontWeight->Bold], "cv"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sv"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["672",
              FontWeight->Bold], "cs"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["840",
               FontWeight->Bold], "v"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1400",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["2800",
              FontWeight->Bold], "cv"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["350",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "c", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "c"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["56",
               FontWeight->Bold], "s"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["160",
               FontWeight->Bold], "s"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["224",
               FontWeight->Bold], "vs"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "cs"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["672",
              FontWeight->Bold], "vc"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["840",
               FontWeight->Bold], "s"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1400",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["2800",
              FontWeight->Bold], "vs"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["56",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["8",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["56",
               FontWeight->Bold], "c"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["160",
               FontWeight->Bold], "c"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["224",
               FontWeight->Bold], "vc"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["224",
              FontWeight->Bold], "sc"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["672",
              FontWeight->Bold], "vs"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["840",
               FontWeight->Bold], "c"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1296",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["1400",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["2800",
              FontWeight->Bold], "vc"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["294",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["567",
               FontWeight->Bold], "s"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["3675",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["3696",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["4312",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["112",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["28",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["294",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["567",
               FontWeight->Bold], "v"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["3675",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["3696",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["4312",
              FontWeight->Bold], "v"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "s"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["28",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["35",
               FontWeight->Bold], "s"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["294",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["300",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["350",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["567",
               FontWeight->Bold], "s"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "c", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "v", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1925",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["3675",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["4312",
              FontWeight->Bold], "s"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SubscriptBox[
             StyleBox["160",
              FontWeight->Bold], "v"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["28",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["35",
               FontWeight->Bold], "v"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["35",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["294",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["300",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["350",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             SubscriptBox[
              StyleBox["567",
               FontWeight->Bold], "v"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "c"],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["567",
              FontWeight->Bold], "s"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "s", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "c", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SubsuperscriptBox[
             StyleBox["840",
              FontWeight->Bold], "v", "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1925",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["3675",
              FontWeight->Bold], "v"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SubscriptBox[
             StyleBox["4312",
              FontWeight->Bold], "v"],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6162192387834263`*^9, 3.616224185795022*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SO9", ",", 
   RowBox[{"MaxDim", "\[Rule]", "495"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "4"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "90000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "78"}],
   
   SequenceForm["Number of Tensor Products: ", 78],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616224198814884*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"12.5\"\>",
      12.46156000000000041438852349529042840004`7.116172326147207,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    12.46156000000000041438852349529042840004`7.116172326147207, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616224198840033*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "3", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(9)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["768",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1650",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2457",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1980",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2772",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["672",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["768",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2772",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3900",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2772",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4158",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1980",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2772",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["2772",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4158",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["128",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["768",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2560",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1920",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2560",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["128",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["432",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["672",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["768",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5040",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["128",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["432",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["672",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["768",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4608",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5040",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["36",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["84",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["126",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["231",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["594",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["924",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2457",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2772",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3900",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1920",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2457",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["2772",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1122",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2574",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["2772",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3900",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7700",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2457",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3900",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12375",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1920",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9504",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2508",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4004",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8748",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2560",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["231",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2457",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2574",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["231",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2457",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2574",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["2772",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["594",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2772",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3900",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9009",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["924",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2457",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2772",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3900",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15444",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["128",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["432",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["576",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["768",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1920",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5040",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9504",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["910",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["7140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7700",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8748",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9009",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["36",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["44",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["495",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["594",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["910",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1980",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2772",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3900",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4004",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7700",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8748",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["9009",
              FontWeight->Bold],
             TraditionalForm], ")"}]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["768",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2560",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2772",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["432",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["672",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["768",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5040",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["768",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5040",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9504",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["128",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["432",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["672",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["768",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["4608",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5040",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12672",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["128",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["432",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["672",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["768",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["4608",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["5040",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["12672",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["12672",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["84",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["126",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["231",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["594",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["924",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1650",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1980",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2457",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2574",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2772",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["3900",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4158",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9009",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15444",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1920",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5040",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9504",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19712",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27648",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["128",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["432",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["576",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["672",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["768",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1920",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["2560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["4608",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["5040",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["9504",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12672",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19712",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24192",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["36",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["84",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["126",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["231",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["495",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["594",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["924",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["1650",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1980",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2457",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2574",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["2772",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["2772",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["3900",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["4004",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4158",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["9009",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["12012",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["15444",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["15444",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["25740",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27456",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1122",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["2772",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1920",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5280",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7700",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2508",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8748",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["2457",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["2772",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12375",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20196",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["3900",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7700",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12375",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["32725",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1920",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9504",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["12672",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27648",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1122",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2574",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["2772",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5148",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16302",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["18018",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["23868",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1122",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2457",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2574",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["2772",
               FontWeight->Bold], "\[Prime]\[Prime]"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["16302",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20196",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["23868",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["31500",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1920",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9504",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19712",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27648",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["59136",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["68640",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2508",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4004",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8748",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9867",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["22932",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["33957",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["54978",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["56430",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2574",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2772",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4004",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9009",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1980",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8748",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9009",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2457",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2574",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2772",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15444",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["15444",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2772",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3900",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9009",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15444",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27456",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["768",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5040",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9504",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12672",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24192",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2457",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2574",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["2772",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12012",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["23868",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["31500",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["9",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["156",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["231",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1650",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2457",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2574",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["2772",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4158",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12012",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15444",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["15444",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["18018",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["31500",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["432",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["432",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["576",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["672",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["768",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1920",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["4608",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4928",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["5040",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["9504",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12672",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["12672",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19712",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24192",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34944",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65536",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7700",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8748",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9009",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44352",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["56430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["87516",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["495",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1980",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2772",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["2772",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3900",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4004",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7700",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8748",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["9009",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["22932",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["25740",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27456",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44352",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["71500",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616219238808076*^9, 3.616224199755629*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SO10", ",", 
   RowBox[{"MaxDim", "\[Rule]", "660"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "4"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "80000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "85"}],
   
   SequenceForm["Number of Tensor Products: ", 85],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616224253877178*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"54.0\"\>",
      54.00927000000000077761797001585364341736`7.753068220590177,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    54.00927000000000077761797001585364341736`7.753068220590177, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616224253903105*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "4", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(10)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["16", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["144", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["16", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["770",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["660",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["770",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["16", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["144", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1200", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2970",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4312",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["210",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["770",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1050", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4125",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5940",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["144", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["672", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1200", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["16", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1440", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["3696",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4950",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5940",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6930",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["770",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5940",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8910",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2772",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4125",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6930",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["16", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["144", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["720", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["144", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1050", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["144",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1200",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3696",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2640",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3696",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["16", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["144", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["560", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["720", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1200", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1440", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["3696", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["8800", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["144", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1200",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1440",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3696",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11088",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["144", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["720", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1200", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1440", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["5280", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["8800", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["144", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["45",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["210",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["770",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["945",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1050", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5940",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8085",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["120",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["320",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1728",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2970",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["3696",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4312",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4950",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["660",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1200",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1440",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["720", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2640", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["210",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1050", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5940",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4312",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4608",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1050", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8085",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1782",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4410",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4608",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["120",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1728",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2970",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["3696",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["3696", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10560",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8085",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14784",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2970",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["3696",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["6930",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10560",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["1050", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8085",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17325",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["144",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["672",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1200",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1440",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3696",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8800",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11088",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["144", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["720", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2640", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["3696", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["7920", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15120", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["45",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["210",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["770",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["945",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1050", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4125",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["5940",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["6930",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["6930", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8910",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4312",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4950",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4950", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28160",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["660",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["770",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7644",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12870",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16380",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["770",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["144", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["720", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["3696", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["320",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2970",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4312",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4410",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["320",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2970",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4312",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4410",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4608",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["770",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["945",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1050", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5940",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8085",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17920",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1050", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5940",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8085",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["23040",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["16", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["144", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["560", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["720", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1200", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2640", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["3696", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["8064", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["8800", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15120", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["1728",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2970",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["3696",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["3696", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4312",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4950",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4950", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36750",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["660",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["770",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1386",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["12870",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14784",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16380",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17920",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["45",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["54",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["660",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["770",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["945",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["1386",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["4125",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5940",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7644",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8085",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14784",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16380",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["17920",
              FontWeight->Bold],
             TraditionalForm], ")"}]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["144", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1200", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["3696", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2970",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["3696", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["770",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["945",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1050",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5940",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1200",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1440",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3696",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8064",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8800",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1200",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3696",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8800",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15120",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["16", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["144", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["560", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["672", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["720", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["1200", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1440", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["3696", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["8064", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["8800", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["11088", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["25200", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["16", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["144", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["560", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1200", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1440", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["3696", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["8064", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["8800", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["11088", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["34992", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["672",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1200",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["3696",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8800",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11088",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["25200",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["120",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["126", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["320",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["1728",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2970",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["3696",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["3696", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4312",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4410",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4950", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36750",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["210",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["770",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["945",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1050",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1050", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4125",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["5940",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["6930",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8085",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17920",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["23040",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["210",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["144",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1200",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1440",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["3696",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["5280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8064",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["8800",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["11088",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["25200",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34992",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["720", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2640", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["3696", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["8800", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15120", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["38016", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["48048", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["16", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["144", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["560", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["720", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["1200", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1440", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2640", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["3696", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["8064", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["8800", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["11088", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["15120", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["25200", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["38016", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["43680", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["120",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["126", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["320",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["1728",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["2970",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["3696",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox[
               OverscriptBox["3696", "_"],
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4312",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["4410",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4950",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4950", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["6930", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["10560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["27720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34398",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["36750",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["46800", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["48114", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["560", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["45",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["210",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["770",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["945",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1050",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["1050", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4125",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["5940",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["6930",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["6930", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7644",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["8085",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["8910",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["17920",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["23040",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["23040", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["72765",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["73710",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["660",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1782",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4608",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["660",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["2640",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7920",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["660",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["45",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["660",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12870",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14784",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["660",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["54",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["660",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12870",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16380",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["660",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4312",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4608",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42120",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["660",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["4950", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["50050",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["660",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["144",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2640",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7920",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20592",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48048",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["660",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1782",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4410",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4608",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9438",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["31680",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["37632",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48510",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["660",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["210",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1782",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4312",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4410",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4608",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["31680",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48510",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["68640",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6162192388311367`*^9, 3.616224256386231*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SO11", ",", 
   RowBox[{"MaxDim", "\[Rule]", "462"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "40000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "45"}],
   
   SequenceForm["Number of Tensor Products: ", 45],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616224268791657*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"12.1\"\>",
      12.10058500000000059060312196379527449608`7.103406279970497,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    12.10058500000000059060312196379527449608`7.103406279970497, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6162242688383512`*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "5", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(11)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1408",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["275",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1760",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2025",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["935",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2025",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1408",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3520",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3003",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5005",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3003",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7128",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7865",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11583",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["275",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["935",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2025",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["275",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1760",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7040",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["275",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["275",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7293",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["275",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["275",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2717",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7150",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7293",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["275",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2025",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15400",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["26520",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["275",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["275",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["935",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2025",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7007",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21945",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28798",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1408",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1760",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3003",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["320",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1408",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1760",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3520",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10240",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1408",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1760",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7040",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10240",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["320",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1408",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1760",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3520",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5280",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10240",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28512",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["55",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["165",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["275",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["330",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["429",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["462",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1430",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2025",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["3003",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4290",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["5005",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11583",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15400",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["22275",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3003",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1408",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3520",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5280",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11583",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15400",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3003",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5005",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["23595",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3003",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7865",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11583",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["23595",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["37752",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2025",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1408",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1760",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10240",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["275",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["429",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["3003",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5005",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7150",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["275",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["429",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["5005",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7128",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7150",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7293",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1430",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2025",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11583",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15400",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["33033",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["275",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["935",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1144",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2025",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["21945",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["26520",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28798",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["33033",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["55",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["65",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["935",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1144",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["1430",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["2025",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["7865",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11583",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15400",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["26520",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28798",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["33033",
              FontWeight->Bold],
             TraditionalForm], ")"}]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["11",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1408",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3520",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4224",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5280",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3003",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17160",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3003",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["22275",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3003",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11583",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["37752",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616219238853858*^9, 3.616224270162588*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SO12", ",", 
   RowBox[{"MaxDim", "\[Rule]", "560"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "48100"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "43"}],
   
   SequenceForm["Number of Tensor Products: ", 43],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616224313894742*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"43.5\"\>",
      43.54962700000000097588781500235199928284`7.659584352932479,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    43.54962700000000097588781500235199928284`7.659584352932479, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616224313960688*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "6", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(12)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["32", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["352", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["32", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["792",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["352",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1638",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["352",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["352",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2112",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2860",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1287",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1638",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2860",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["32", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["352", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1728", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["4928", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["792",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8008",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11232",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["462", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1638",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8085",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14014",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21021",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["352",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["32", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["352", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1728", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2112", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["352",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1287",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2860",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["352",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["462", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8085",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["352", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["792",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4752",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["352",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["2112", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["9152", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["352",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["352",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["4928",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13728",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["352",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["352",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["11088",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11232",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["352",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["352",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2112",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9152",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13728",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["352",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["352",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4004",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11088",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["11088",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["352",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["32", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["352", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["1728", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2112", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["4928", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["8800", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["13728", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["43680", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["352",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2860",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27456",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45045",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["352",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["352",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["66",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["462", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["495",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1638",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2079",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2860",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["8085",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["21021",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["21450", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["27027", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27456",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["352", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["352",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["220",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["352",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["792",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["4752",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4752", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4928",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["8008",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11232",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36036",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["45760",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["792",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4752",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["462", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["352",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["4928",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9504",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4224",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8800",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8085",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21450",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["462", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8085",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27027",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["792",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["352",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["4928",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8800",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["462", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8085",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21021",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8085",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27456",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["495",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["792",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["4752",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4752", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8008",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36036",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48048",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1638",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2860",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["32",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["352", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1728", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["2112", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["13728", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["66",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["352",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8008",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11088",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11232",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["352",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["560",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["8008",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11088",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["11088",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11232",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6162192388769608`*^9, 3.6162243168903503`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SO13", ",", 
   RowBox[{"MaxDim", "\[Rule]", "3000"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "4"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "19000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "25"}],
   
   SequenceForm["Number of Tensor Products: ", 25],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616224547898735*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"231.0\"\>",
      230.58741900000001123771653510630130767822`8.383435521494462,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    230.58741900000001123771653510630130767822`8.383435521494462, {3, 1}], 
    "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616224547948646*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "7", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(13)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["768",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["715", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1287",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1716",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["768",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4160",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["715", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2275",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["442",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["768",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4992",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3927",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1729",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2275",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3927",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["715", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["768",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13312",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1287",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7722",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12285",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7722",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17017",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["442",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1729",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3927",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["442",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["442",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16302",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17017",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["442",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["442",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5733",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16302",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16575",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2275",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3927",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["442",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["715",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["7722",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12285",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16575",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17017",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["442",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["715",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["12285",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16302",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16575",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17017",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["768",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["768",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4992",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1287",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["715", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1716",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14300",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1729",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["442",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5733",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16302",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2275",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12285",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16575",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["13",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7722",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12285",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17017",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616219238900237*^9, 3.616224550907241*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SO14", ",", 
   RowBox[{"MaxDim", "\[Rule]", "832"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "8000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "14"}],
   
   SequenceForm["Number of Tensor Products: ", 14],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6162246055155487`*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"54.2\"\>",
      54.22855200000000053250914788804948329926`7.7548279214251,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    54.22855200000000053250914788804948329926`7.7548279214251, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6162246055630093`*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "8", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(14)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["91",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["104",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["64", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["832", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["64", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["91",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1001",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3003",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["364",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1716",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2002",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["91",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["364",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["91",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["832",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["91",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["91",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["91",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["104",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1001",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3080",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4004",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["104",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["546",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["104",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["832",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5824",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["104",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["91",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["91",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["104",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4004",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5265",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["104",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["104",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["91",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["104",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2275",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3080",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5265",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["364",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["91",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1001",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4004",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["546",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["104",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2275",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5265",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["832",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["64", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["832", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4928", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["5824", "_"],
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616219238926488*^9, 3.616224611284068*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SO18", ",", 
   RowBox[{"MaxDim", "\[Rule]", "1920"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "5000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "4"}],
   SequenceForm["Number of Tensor Products: ", 4],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6162271459361067`*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"474.0\"\>",
      474.20279399999998304338078014552593231201`8.696564021789797,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    474.20279399999998304338078014552593231201`8.696564021789797, {3, 1}], 
    "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616227145966399*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "14", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(18)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["18",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["18",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["153",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["170",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["153",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["18",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["18",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["816",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1920",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["170",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["18",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["18",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1122",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1920",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["256",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["18",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["256", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["4352", "_"],
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6162192389298983`*^9, 3.616225113216341*^9, 
  3.6162266658999157`*^9, 3.616227146884817*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SO22", ",", 
   RowBox[{"MaxDim", "\[Rule]", "1024"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "17000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "3"}],
   SequenceForm["Number of Tensor Products: ", 3],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616227360175891*^9, 3.616227545343337*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"133.0\"\>",
      133.22144299999999361716618295758962631226`8.145174046725547,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    133.22144299999999361716618295758962631226`8.145174046725547, {3, 1}], 
    "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616227360175891*^9, 3.61622754535893*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "1", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(22)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["22",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["22",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["252",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["22",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["22",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1540",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3520",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["252",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["22",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["22",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2002",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3520",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616219238953658*^9, 3.6162252277561293`*^9, 
  3.616226286963456*^9, 3.616227400653014*^9, 3.616228304203558*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"SO26", ",", 
   RowBox[{"MaxDim", "\[Rule]", "2600"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "1"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "700000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "5"}],
   SequenceForm["Number of Tensor Products: ", 5],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616228334170198*^9, 3.6162301510940123`*^9, 
  3.6162303655655947`*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"4.2\"\>",
      4.19048300000000040199665818363428115845`6.642863996423833,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    4.19048300000000040199665818363428115845`6.642863996423833, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616228334170198*^9, 3.6162301510940123`*^9, 
  3.616230365583729*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "2", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"SO(26)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["325",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["325",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2600",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5824",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["325",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["325",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["325",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14950",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["37674",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["52325",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2600",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["325",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14950",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["52325",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2600",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["325",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2600",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5824",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65780",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["450450",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6162303526137257`*^9, 3.6162307562936993`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Sp(N)", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"Sp4", ",", 
   RowBox[{"MaxDim", "\[Rule]", "35"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "4"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "1800"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "45"}],
   
   SequenceForm["Number of Tensor Products: ", 45],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616226294513503*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"0.9\"\>",
      0.94881000000000004224176564093795605004`5.997779166578408,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    0.94881000000000004224176564093795605004`5.997779166578408, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616226294547021*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "2", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"Sp(4)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["16",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["10",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["35",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["35",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["154",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["91",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["154",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["140",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["256",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["140",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["260",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["35",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["35",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["154",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["16",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["20",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["40",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["16",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["40",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["4",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["20",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["80",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["140",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["81",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["154",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["260",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["154",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["390",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["10",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["14",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["35",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["35",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["81",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["105",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["154",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["35",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["81",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["105",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["154",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["30",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["35",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["35",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["154",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["231",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6162192390011*^9, 3.616226294658813*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"Sp6", ",", 
   RowBox[{"MaxDim", "\[Rule]", "100"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "16000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "45"}],
   
   SequenceForm["Number of Tensor Products: ", 45],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6162262964113092`*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"1.7\"\>",
      1.66365399999999996616395492310402914882`6.241662921823572,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    1.66365399999999996616395492310402914882`6.241662921823572, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.61622629645315*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "3", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"Sp(6)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["216",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["126",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["126",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["525",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["216",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["252",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["126",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["385",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["462",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["924",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["216",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["216",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["126",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["525",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["924",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["14",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["21",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["70",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["90",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["126",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["385",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["512",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["525",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["216",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["70",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["525",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["216",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1100",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["126",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["216",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["350",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["21",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["90",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["126",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["385",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["512",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1078",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["216",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1078",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2464",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["216",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2240",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["385",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["525",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1386",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2205",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["126",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1001",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1078",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2457",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["216",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["385",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["126",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["216",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2016",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["6",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["14",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["126",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["216",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["350",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["616",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1344",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1386",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["70",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["385",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["512",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["525",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2205",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1078",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2205",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2457",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["84",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["90",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["126",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["385",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["512",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["525",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1078",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2205",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.61621923902485*^9, 3.616226296565468*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"Sp8", ",", 
   RowBox[{"MaxDim", "\[Rule]", "308"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "12500"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "38"}],
   
   SequenceForm["Number of Tensor Products: ", 38],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616226299680443*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"3.0\"\>",
      2.98001499999999985845988703658804297447`6.494818363396428,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    2.98001499999999985845988703658804297447`6.494818363396428, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616226299715539*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "4", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"Sp(8)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["288",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["792",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1155",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["594",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["825",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["288",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["792",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["288",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1232",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["792",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1056",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["792",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["825",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1232",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["792",
              FontWeight->Bold], "\[Prime]\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["288",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1232",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3520",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1155",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3696",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1716",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2184",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4914",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["160",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["288",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["792",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1232",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["160",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["792",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1232",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["288",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["792",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1232",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4200",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["315",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["792",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1155",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["594",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["3696",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4914",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["36",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["308",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["315",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["330",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["594",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["792",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["825",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1155",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2184",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3696",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["4914",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["288",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["792",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1155",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["288",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["288",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["792",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1056",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1232",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4200",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["288",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["288",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["792",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1232",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3520",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4200",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["288",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["120",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["792",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1056",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["3696",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4752",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["288",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["594",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["792",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["825",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6237",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["8",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["792",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["792",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2184",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["36",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["594",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["825",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4914",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["42",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["315",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1155",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7020",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["308",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["48",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["288",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["792",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1232",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1512",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4200",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6552",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616219239049893*^9, 3.616226300017581*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"Sp10", ",", 
   RowBox[{"MaxDim", "\[Rule]", "320"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "50000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "36"}],
   
   SequenceForm["Number of Tensor Products: ", 36],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616226304538096*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"4.4\"\>",
      4.44292999999999960181185088003985583782`6.668269384104736,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    4.44292999999999960181185088003985583782`6.668269384104736, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616226304571557*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "5", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"Sp(10)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["780",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["891",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["891",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["780",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["891",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1408",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2860",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1408",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4212",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["780",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["891",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1155",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4004",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5005",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1155",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1408",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1408",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5720",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["891",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5005",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8580",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["780",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4004",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4719",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7865",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1408",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["891",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1155",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5005",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["891",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1155",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6864",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1408",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2860",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9152",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2860",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9152",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["9438",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["780",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["891",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4004",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5005",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7865",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8580",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4212",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2002",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4620",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["891",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6864",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15015",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1155",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6864",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21021",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1408",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4212",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24960",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["780",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["5005",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8250",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17820",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["780",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["891",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["320",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1408",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2860",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4212",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4620",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["10",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["320",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2860",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4212",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4620",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4928",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["780",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["891",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1155",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5005",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6864",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17920",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["891",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1155",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5005",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6864",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["28160",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["110",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["132",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1408",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2860",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4212",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4290",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5720",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["32340",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["220",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["44",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["55",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["780",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["891",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1430",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["14300",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15015",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17820",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17920",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["320",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["44",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["55",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["165",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["715",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["780",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["891",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["1430",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["4004",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5005",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6864",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8250",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["15015",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17820",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["17920",
              FontWeight->Bold],
             TraditionalForm], ")"}]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616219239079124*^9, 3.616226305782612*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"Sp12", ",", 
   RowBox[{"MaxDim", "\[Rule]", "429"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "25000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "17"}],
   
   SequenceForm["Number of Tensor Products: ", 17],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616226316778021*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"10.9\"\>",
      10.86339399999999955070961732417345046997`7.0565654443511265,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    10.86339399999999955070961732417345046997`7.0565654443511265, {3, 1}], 
    "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6162263168209867`*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "6", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"Sp(12)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["208",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2002",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["364",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2002",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1365",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["208",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2002",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["208",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["208",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["572",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4368",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7800",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["208",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["208",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4368",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11088",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["208",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["208",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["429",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2002",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6006",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["13650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["18954",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["364",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1365",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["364",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["364",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11088",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11648",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["364",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["364",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["4368",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11440",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["11648",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["429",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["572",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4576",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["12",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["208",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["572",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4368",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["429",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6006",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["21450",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["65",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["429",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["429",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2002",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6006",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["18954",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.61621923910837*^9, 3.616226320426228*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"Sp14", ",", 
   RowBox[{"MaxDim", "\[Rule]", "910"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "10000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "9"}],
   SequenceForm["Number of Tensor Products: ", 9],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.61622635336782*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"32.6\"\>",
      32.59517199999999803594619152136147022247`7.533753190370934,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    32.59517199999999803594619152136147022247`7.533753190370934, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616226353425243*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "7", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox["\"Sp(14)\"", TraditionalForm], "\" Tensor Products\""}, 
         "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3094",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3900",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3900",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5355",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2380",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3094",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5355",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["350",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["910",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3900",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["560",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2380",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5355",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["90",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3094",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3900",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5355",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6162192391326017`*^9, 3.616226359557969*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exceptional Algebras", "Section"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"E6", ",", 
   RowBox[{"MaxDim", "\[Rule]", "2430"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "970000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "36"}],
   
   SequenceForm["Number of Tensor Products: ", 36],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616226391738182*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"31.6\"\>",
      31.60571500000000000341060513164848089218`7.520365532877503,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    31.60571500000000000341060513164848089218`7.520365532877503, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.61622639179318*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "8", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox[
           SubscriptBox["\"E\"", "6"], TraditionalForm], 
          "\" Tensor Products\""}, "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["27", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["27", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["351", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["351", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["27", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1728", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["7722", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["27", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["351", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1728", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["7371", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["351", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5824",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["351", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3003",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5824",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7371",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19305",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7371",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17550",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["351", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["7371", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["7722", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["19305", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["34398", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["54054", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["351", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1728", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["7371", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["7722", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["51975", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["54054", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["27", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["351", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["351", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["1728", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["7371", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["7722", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["17550", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["34398", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["51975", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["351", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["650",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5824",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["5824", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70070",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["351", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["85293",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["351", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5824",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78975",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7371",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7722",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["650",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5824",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["5824", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7371",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7722",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17550",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19305",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["61425",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112320",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["351",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1728",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["7371",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["7722",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17550",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19305",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["51975",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112320",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["78",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["650",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2925",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["3003",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["3003", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["5824",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["5824", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["34749",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["70070",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["78975",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["78975", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["85293",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["1728", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["5824", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["351", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["351", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["1728", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["7371", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["17550", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["19305", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1728",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["7371",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7722",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17550",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["46332",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["51975",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["1728", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["3003", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5824",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["5824", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["70070",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["78975", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["146432", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["252252", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["1728", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["650",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2925",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["3003", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5824",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["5824", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["34749",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["70070",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["78975", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105600",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["252252", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["27", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["351", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["1728", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["7371", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["7722", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["17550", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["46332", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["51975", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["112320", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["359424", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["27", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["351", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox[
              OverscriptBox["351", "_"],
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["1728", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["7371", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["7722", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["17550", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["19305", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["46332", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["51975", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["112320", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["314496", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["351",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["1728",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["7371",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["7722",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["17550",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["19305",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34398",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["46332",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["51975",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["54054",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["112320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["314496",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["359424",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["1728", "_"],
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["78",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["650",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2430",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["2925",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["5824",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["5824", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["34749",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["43758",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["70070",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["78975",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["78975", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["85293",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["105600",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["252252",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["252252", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["812175",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["852930",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox[
             OverscriptBox["27", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["351", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox[
               OverscriptBox["351", "_"],
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["1728", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["7371", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["7722", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["17550", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["19305", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["34398", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["46332", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["51975", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["61425", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["112320", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox[
              OverscriptBox["314496", "_"],
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["386100", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["393822", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["459459", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["494208", "_"],
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2430",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17550",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["46332",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2430",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["78",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["43758",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105600",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2430",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["351",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7371",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17550",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19305",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34398",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["314496",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["459459",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2430",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["351",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7371",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17550",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19305",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["46332",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["51975",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["314496",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["393822",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2430",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["650",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2430",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2925",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["5824",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["5824", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["34749",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["70070",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["105600",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["252252",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox[
             OverscriptBox["252252", "_"],
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["812175",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616219239157033*^9, 3.616226395800343*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"E7", ",", 
   RowBox[{"MaxDim", "\[Rule]", "1539"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "2700000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "15"}],
   
   SequenceForm["Number of Tensor Products: ", 15],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616226436506714*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"40.3\"\>",
      40.30733800000000144336809171363711357117`7.625984030458088,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    40.30733800000000144336809171363711357117`7.625984030458088, {3, 1}], "s"],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6162264365679417`*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "9", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox[
           SubscriptBox["\"E\"", "7"], TraditionalForm], 
          "\" Tensor Products\""}, "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["133",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1463",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1539",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["133",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["912",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6480",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["133",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["133",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["133",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1539",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7371",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8645",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["912",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["133",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1539",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8645",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40755",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["912",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["133",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["912",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6480",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27664",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["86184",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["912",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["912",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["133",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1463",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1539",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7371",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8645",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40755",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["152152",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["253935",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["365750",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1463",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6480",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["24320",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["51072",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1463",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["133",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1463",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1539",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40755",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["150822",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1463",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["912",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["912",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6480",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27664",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["51072",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["362880",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["885248",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1463",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["1463",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["133",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1463",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1539",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7371",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["150822",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["152152",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["293930",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["617253",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["915705",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1539",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["912",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6480",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27664",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["51072",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1539",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["133",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["133",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1463",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1539",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8645",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40755",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["152152",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1539",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["912",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["56",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["912",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["6480",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["27664",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["51072",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["86184",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["362880",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["861840",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1539",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["1463",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["133",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1463",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1539",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8645",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["40755",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["150822",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["152152",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["915705",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["980343",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1539",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["1539",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["133",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1463",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1539",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["7371",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8645",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["40755",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["150822",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["152152",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["365750",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["617253",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["980343",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616219239184465*^9, 3.6162264460519238`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"E8", ",", 
   RowBox[{"MaxDim", "\[Rule]", "3875"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "2"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "79000000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "3"}],
   SequenceForm["Number of Tensor Products: ", 3],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616226545207756*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"98.6\"\>",
      98.57198300000000301679392578080296516418`8.01435340674486,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    98.57198300000000301679392578080296516418`8.01435340674486, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616226545280233*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "10", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox[
           SubscriptBox["\"E\"", "8"], TraditionalForm], 
          "\" Tensor Products\""}, "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["248",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["248",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["248",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3875",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27000",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["30380",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["3875",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["248",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["248",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3875",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["30380",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["147250",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["779247",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["3875",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["3875",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["248",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3875",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27000",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["30380",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["147250",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["779247",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2450240",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4881384",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["6696000",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6162192391880198`*^9, 3.616226545283214*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"F4", ",", 
   RowBox[{"MaxDim", "\[Rule]", "2652"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "3"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "32000000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "36"}],
   
   SequenceForm["Number of Tensor Products: ", 36],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.61622655523326*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"9.8\"\>",
      9.83611900000000005661604518536478281021`7.013423687588955,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    9.83611900000000005661604518536478281021`7.013423687588955, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616226555302967*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "11", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox[
           SubscriptBox["\"F\"", "4"], TraditionalForm], 
          "\" Tensor Products\""}, "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8424",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["273",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["324",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1053",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["8424",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10829",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19278",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19448",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10829",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["273",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1053",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["8424",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10829",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19278",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["324",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["8424",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10829",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16302",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8424",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10829",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8424",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17901",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1053",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8424",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17901",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19278",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10829",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12376",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["29172",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["273",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["324",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1053",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1274",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["8424",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["10829",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["17901",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19278",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["29172",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["106496",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8424",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10829",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17901",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19278",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["106496",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["119119",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["273",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["1053",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["8424",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["10829",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17901",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["19278",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["19448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["76076",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["106496",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8424",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10829",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["29172",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["106496",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["160056",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["52",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["273",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["324",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1053",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["1053",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["1274",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["8424",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["10829",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["12376",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16302",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17901",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["19278",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["19448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["29172",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["34749",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["106496",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["107406",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["119119",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160056",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["160056",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1053",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["8424",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["17901",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["19278",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["76076",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["106496",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["107406",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["119119",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["184756",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["379848",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["1053",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10829",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["12376",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16302",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["29172",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["100776",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160056",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["160056",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["226746",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["340119",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8424",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19278",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10829",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["29172",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1053",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["8424",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["10829",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17901",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19278",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["106496",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["107406",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1274",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["8424",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["10829",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["19278",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["29172",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["106496",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160056",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["273",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["1053",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2652",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["8424",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["10829",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["17901",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["19278",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["19448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["34749",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["76076",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["106496",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["107406",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["119119",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["205751",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["379848",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1274",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["10829",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["12376",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["29172",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["106496",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160056",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["160056",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["340119",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["420147",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["324",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["1053",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1274",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["8424",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["10829",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["12376",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16302",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19278",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["19448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["29172",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["106496",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["160056",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["160056",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["205751",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["226746",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["420147",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10829",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16302",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19278",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["76076",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["8424",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["10829",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["16302",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19278",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["34749",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["76076",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["106496",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160056",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["212992",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2652",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["8424",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["10829",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["16302",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17901",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19278",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["34749",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["76076",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81081",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["106496",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["205751",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["212992",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["324",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1274",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["8424",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["10829",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["16302",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["19278",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["19448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["29172",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["34749",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["76076",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["106496",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["107406",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["160056",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["160056",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["205751",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["212992",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["412776",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["787644",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["4096",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["8424",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17901",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["19278",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["19448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["34749",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["76076",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["106496",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["107406",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["205751",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["379848",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["787644",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1002456",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1053",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["8424",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["10829",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["17901",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["19278",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["19448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["34749",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["76076",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["106496",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["107406",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160056",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["205751",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["212992",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["379848",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["787644",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["952952",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["2652",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["26",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["52",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["324",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1053",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["1053",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1274",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["2652",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["4096",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["8424",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["10829",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["12376",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["16302",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["17901",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["19278",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["19448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["29172",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["34749",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["76076",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["81081",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["106496",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["107406",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["119119",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["160056",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["160056",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["205751",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["212992",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["342056",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["412776",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["420147",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["629356",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["787644",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1042899",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1341522",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.6162192392131968`*^9, 3.6162265557180243`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TensorProductsTable", "[", 
  RowBox[{"G2", ",", 
   RowBox[{"MaxDim", "\[Rule]", "189"}], ",", 
   RowBox[{"MaxDynkinDigit", "\[Rule]", "4"}], ",", 
   RowBox[{"MaxDimInResult", "\[Rule]", "96000"}]}], "]"}]], "Input"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Number of Tensor Products: \"\>", "\[InvisibleSpace]", "36"}],
   
   SequenceForm["Number of Tensor Products: ", 36],
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.6162265567165413`*^9}],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Total time: \"\>", "\[InvisibleSpace]", 
    TagBox[
     InterpretationBox["\<\"0.9\"\>",
      0.90127299999999999080557699926430359483`5.975456274109055,
      AutoDelete->True],
     NumberForm[#, {3, 1}]& ], "\[InvisibleSpace]", "\<\"s\"\>"}],
   SequenceForm["Total time: ", 
    NumberForm[
    0.90127299999999999080557699926430359483`5.975456274109055, {3, 1}], "s"],
   
   Editable->False], TraditionalForm]], "Print",
 CellChangeTimes->{3.616226556782691*^9}],

Cell[BoxData[
 FormBox[
  TagBox[GridBox[{
     {
      TemplateBox[{StyleBox[
         TemplateBox[{"\"Table\[MediumSpace]\"", "12", "\": \""}, 
          "RowDefault"], StripOnInput -> False, FontWeight -> Bold],
        TemplateBox[{
          FormBox[
           SubscriptBox["\"G\"", "2"], TraditionalForm], 
          "\" Tensor Products\""}, "RowDefault"]},
       "RowDefault"]},
     {
      TagBox[GridBox[{
         {
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["77",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["14",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["77",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["77",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["182",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["729",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["729",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["77",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["182",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["286",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["729",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["77",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["182",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["286",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["729",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["77",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["714",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["729",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["748",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1547",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["77",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["729",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1547",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["77",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["77",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["182",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["286",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["714",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["729",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["182",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["714",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["729",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["77",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["77",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["182",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["286",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["378",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["714",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["729",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["924",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1547",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["182",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["286",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["714",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["729",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["924",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1547",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2926",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["77",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["182",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["286",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["378",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["714",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["729",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["924",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1254",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1547",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["77",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["77",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["182",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["273",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["286",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["378",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["714",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["729",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["748",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["896",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["924",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1254",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1547",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1728",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["2079",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2926",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3003",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["286",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["448",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["729",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["77",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["182",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["286",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["378",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["729",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["924",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["64",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["77",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["77",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["182",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["286",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["378",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["714",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["729",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["924",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1547",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["27",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["77",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            SuperscriptBox[
             StyleBox["77",
              FontWeight->Bold], "\[Prime]"],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["182",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["286",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["378",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["714",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["729",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["924",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1547",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["77",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["77",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["77",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["182",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["273",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["286",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["378",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["714",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["729",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["896",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["924",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1547",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["1728",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["182",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["14",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["77",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["77",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["182",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"5", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["273",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["286",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["378",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"5", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["714",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["729",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["896",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["924",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1254",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["1547",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1728",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2926",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["3003",
             FontWeight->Bold],
            TraditionalForm]}]},
         {
          RowBox[{
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm], "\[CircleTimes]", 
           FormBox[
            StyleBox["189",
             FontWeight->Bold],
            TraditionalForm]}], "\<\" = \"\>", 
          RowBox[{
           FormBox[
            StyleBox["1",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["7",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["14",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["27",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["64",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["77",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"3", "(", 
            FormBox[
             SuperscriptBox[
              StyleBox["77",
               FontWeight->Bold], "\[Prime]"],
             TraditionalForm], ")"}], "+", 
           RowBox[{"5", "(", 
            FormBox[
             StyleBox["182",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"6", "(", 
            FormBox[
             StyleBox["189",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["273",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"5", "(", 
            FormBox[
             StyleBox["286",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"4", "(", 
            FormBox[
             StyleBox["378",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"6", "(", 
            FormBox[
             StyleBox["448",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["714",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"5", "(", 
            FormBox[
             StyleBox["729",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["748",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["896",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"5", "(", 
            FormBox[
             StyleBox["924",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["1254",
             FontWeight->Bold],
            TraditionalForm], "+", 
           RowBox[{"3", "(", 
            FormBox[
             StyleBox["1547",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           RowBox[{"2", "(", 
            FormBox[
             StyleBox["1728",
              FontWeight->Bold],
             TraditionalForm], ")"}], "+", 
           FormBox[
            StyleBox["2079",
             FontWeight->Bold],
            TraditionalForm], "+", 
           FormBox[
            StyleBox["2926",
             FontWeight->Bold],
            TraditionalForm]}]}
        },
        AutoDelete->False,
        GridBoxAlignment->{
         "Columns" -> {Right, Center, Left}, "Rows" -> {Bottom, {Center}}},
        GridBoxBackground->{"Columns" -> {{None}}, "Rows" -> {{
             GrayLevel[1], 
             RGBColor[0.9, 0.9, 0.9]}}},
        GridBoxDividers->{"Columns" -> {
            GrayLevel[1], {False}, 
            GrayLevel[1]}, "Rows" -> {True, {False}, True}},
        GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
        GridBoxSpacings->{
         "Columns" -> {2, 0.5, 0.5, 2}, "Rows" -> {Automatic}}],
       "Grid"]}
    },
    AutoDelete->False,
    GridBoxAlignment->{"Columns" -> {{Left}}},
    GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
   "Grid"], TraditionalForm]], "Output",
 CellChangeTimes->{3.616219239238017*^9, 3.61622655685837*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
CellGrouping->Manual,
WindowSize->{931, 669},
WindowMargins->{{194, Automatic}, {257, 89}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperOrientation"->"Portrait",
"PaperSize"->{594.75, 842.25},
"PostScriptOutputFile"->"/home/feger/print.pdf"},
PrivateNotebookOptions->{"VersionedStylesheet"->{"Default.nb"[8.] -> True}},
ShowSelection->True,
CommonDefaultFormatTypes->{"Output"->TraditionalForm},
FrontEndVersion->"9.0 for Linux x86 (64-bit) (February 7, 2013)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 42, 0, 93, "Title"],
Cell[624, 24, 218, 8, 35, "Subsubtitle"],
Cell[CellGroupData[{
Cell[867, 36, 33, 0, 77, "Section"],
Cell[903, 38, 85, 2, 29, "Input",
 InitializationCell->True],
Cell[991, 42, 334, 10, 67, "Input",
 InitializationCell->True],
Cell[1328, 54, 225, 6, 31, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[1590, 65, 24, 0, 47, "Section"],
Cell[CellGroupData[{
Cell[1639, 69, 240, 5, 31, "Input"],
Cell[1882, 76, 272, 7, 22, "Print"],
Cell[2157, 85, 530, 13, 22, "Print"],
Cell[2690, 100, 38699, 1367, 893, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[41426, 1472, 241, 5, 31, "Input"],
Cell[41670, 1479, 270, 7, 22, "Print"],
Cell[41943, 1488, 532, 14, 22, "Print"],
Cell[42478, 1504, 94087, 3238, 1781, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[136602, 4747, 242, 5, 31, "Input"],
Cell[136847, 4754, 270, 7, 22, "Print"],
Cell[137120, 4763, 535, 14, 22, "Print"],
Cell[137658, 4779, 91549, 3124, 1833, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[229244, 7908, 242, 5, 31, "Input"],
Cell[229489, 7915, 270, 7, 22, "Print"],
Cell[229762, 7924, 538, 14, 22, "Print"],
Cell[230303, 7940, 88917, 3053, 1777, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[319257, 10998, 243, 5, 31, "Input"],
Cell[319503, 11005, 269, 7, 22, "Print"],
Cell[319775, 11014, 531, 13, 22, "Print"],
Cell[320309, 11029, 86233, 2938, 1759, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[406579, 13972, 243, 5, 31, "Input"],
Cell[406825, 13979, 269, 7, 22, "Print"],
Cell[407097, 13988, 531, 13, 22, "Print"],
Cell[407631, 14003, 36040, 1237, 915, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[443708, 15245, 243, 5, 31, "Input"],
Cell[443954, 15252, 272, 7, 22, "Print"],
Cell[444229, 15261, 532, 14, 22, "Print"],
Cell[444764, 15277, 34326, 1184, 913, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[479127, 16466, 243, 5, 31, "Input"],
Cell[479373, 16473, 270, 7, 22, "Print"],
Cell[479646, 16482, 539, 14, 22, "Print"],
Cell[480188, 16498, 15130, 521, 457, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[495355, 17024, 244, 5, 31, "Input"],
Cell[495602, 17031, 272, 7, 22, "Print"],
Cell[495877, 17040, 531, 13, 22, "Print"],
Cell[496411, 17055, 10159, 347, 323, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[506607, 17407, 244, 5, 31, "Input"],
Cell[506854, 17414, 264, 6, 22, "Print"],
Cell[507121, 17422, 531, 13, 22, "Print"],
Cell[507655, 17437, 5306, 176, 171, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[512998, 17618, 244, 5, 31, "Input"],
Cell[513245, 17625, 266, 6, 22, "Print"],
Cell[513514, 17633, 541, 14, 22, "Print"],
Cell[514058, 17649, 5307, 176, 171, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[519414, 17831, 24, 0, 77, "Section"],
Cell[CellGroupData[{
Cell[519463, 17835, 243, 5, 31, "Input"],
Cell[519709, 17842, 294, 7, 22, "Print"],
Cell[520006, 17851, 552, 13, 22, "Print"],
Cell[520561, 17866, 101997, 3481, 1721, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[622595, 21352, 243, 5, 31, "Input"],
Cell[622841, 21359, 270, 7, 22, "Print"],
Cell[623114, 21368, 532, 14, 22, "Print"],
Cell[623649, 21384, 110728, 3727, 1829, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[734414, 25116, 244, 5, 31, "Input"],
Cell[734661, 25123, 270, 7, 22, "Print"],
Cell[734934, 25132, 531, 13, 22, "Print"],
Cell[735468, 25147, 98429, 3378, 1717, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[833934, 28530, 245, 5, 31, "Input"],
Cell[834182, 28537, 270, 7, 22, "Print"],
Cell[834455, 28546, 531, 13, 22, "Print"],
Cell[834989, 28561, 104755, 3562, 1805, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[939781, 32128, 245, 5, 31, "Input"],
Cell[940029, 32135, 270, 7, 22, "Print"],
Cell[940302, 32144, 533, 13, 22, "Print"],
Cell[940838, 32159, 48343, 1669, 951, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[989218, 33833, 245, 5, 31, "Input"],
Cell[989466, 33840, 270, 7, 22, "Print"],
Cell[989739, 33849, 531, 13, 22, "Print"],
Cell[990273, 33864, 42264, 1447, 879, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1032574, 35316, 246, 5, 31, "Input"],
Cell[1032823, 35323, 270, 7, 22, "Print"],
Cell[1033096, 35332, 539, 14, 22, "Print"],
Cell[1033638, 35348, 21574, 745, 513, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1055249, 36098, 244, 5, 31, "Input"],
Cell[1055496, 36105, 272, 7, 22, "Print"],
Cell[1055771, 36114, 529, 13, 22, "Print"],
Cell[1056303, 36129, 11508, 393, 305, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1067848, 36527, 245, 5, 31, "Input"],
Cell[1068096, 36534, 266, 6, 22, "Print"],
Cell[1068365, 36542, 539, 14, 22, "Print"],
Cell[1068907, 36558, 3851, 123, 115, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1072795, 36686, 246, 5, 31, "Input"],
Cell[1073044, 36693, 286, 6, 22, "Print"],
Cell[1073333, 36701, 560, 14, 22, "Print"],
Cell[1073896, 36717, 3268, 102, 95, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1077201, 36824, 247, 5, 31, "Input"],
Cell[1077451, 36831, 315, 7, 22, "Print"],
Cell[1077769, 36840, 581, 15, 22, "Print"],
Cell[1078353, 36857, 5217, 171, 133, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1083619, 37034, 24, 0, 77, "Section"],
Cell[CellGroupData[{
Cell[1083668, 37038, 242, 5, 31, "Input"],
Cell[1083913, 37045, 270, 7, 22, "Print"],
Cell[1084186, 37054, 532, 14, 22, "Print"],
Cell[1084721, 37070, 47889, 1653, 893, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1132647, 38728, 244, 5, 31, "Input"],
Cell[1132894, 38735, 272, 7, 22, "Print"],
Cell[1133169, 38744, 531, 14, 22, "Print"],
Cell[1133703, 38760, 50429, 1734, 893, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1184169, 40499, 244, 5, 31, "Input"],
Cell[1184416, 40506, 270, 7, 22, "Print"],
Cell[1184689, 40515, 532, 14, 22, "Print"],
Cell[1185224, 40531, 38904, 1344, 781, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1224165, 41880, 245, 5, 31, "Input"],
Cell[1224413, 41887, 270, 7, 22, "Print"],
Cell[1224686, 41896, 532, 14, 22, "Print"],
Cell[1225221, 41912, 36228, 1252, 743, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1261486, 43169, 245, 5, 31, "Input"],
Cell[1261734, 43176, 270, 7, 22, "Print"],
Cell[1262007, 43185, 540, 14, 22, "Print"],
Cell[1262550, 43201, 15306, 524, 361, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1277893, 43730, 245, 5, 31, "Input"],
Cell[1278141, 43737, 263, 6, 22, "Print"],
Cell[1278407, 43745, 531, 13, 22, "Print"],
Cell[1278941, 43760, 8141, 275, 209, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1287131, 44041, 39, 0, 77, "Section"],
Cell[CellGroupData[{
Cell[1287195, 44045, 245, 5, 31, "Input"],
Cell[1287443, 44052, 270, 7, 22, "Print"],
Cell[1287716, 44061, 530, 13, 22, "Print"],
Cell[1288249, 44076, 51415, 1719, 867, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1339701, 45800, 246, 5, 31, "Input"],
Cell[1339950, 45807, 270, 7, 22, "Print"],
Cell[1340223, 45816, 533, 13, 22, "Print"],
Cell[1340759, 45831, 17091, 581, 343, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1357887, 46417, 247, 5, 31, "Input"],
Cell[1358137, 46424, 264, 6, 22, "Print"],
Cell[1358404, 46432, 533, 14, 22, "Print"],
Cell[1358940, 46448, 4543, 146, 95, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1363520, 46599, 247, 5, 31, "Input"],
Cell[1363770, 46606, 269, 7, 22, "Print"],
Cell[1364042, 46615, 532, 14, 22, "Print"],
Cell[1364577, 46631, 70255, 2353, 1213, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1434869, 48989, 243, 5, 31, "Input"],
Cell[1435115, 48996, 272, 7, 22, "Print"],
Cell[1435390, 49005, 532, 14, 22, "Print"],
Cell[1435925, 49021, 67550, 2263, 941, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
