(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     72840,       2728]
NotebookOptionsPosition[     61684,       2354]
NotebookOutlinePosition[     64158,       2414]
CellTagsIndexPosition[     64115,       2411]
WindowTitle->LieART - Quick Start Tutorial - Wolfram Mathematica
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[GridBox[{
   {Cell["LIE A R T PACKAGE TUTORIAL", "PacletNameCell"], Cell[TextData[{
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"Functions \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"DecomposeIrrep\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/DecomposeIrrep"], "\<\"DecomposeProduct\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/DecomposeProduct"], "\<\"Irrep\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/Irrep"], "\<\"ProductIrrep\"\>":>
       Documentation`HelpLookup[
       "paclet:LieART/ref/ProductIrrep"], "\<\"YoungTableau\"\>":>
       Documentation`HelpLookup["paclet:LieART/ref/YoungTableau"]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}],
     "\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
     Cell[BoxData[
      ActionMenuBox[
       FrameBox["\<\"URL \[RightGuillemet]\"\>",
        StripOnInput->False], {"\<\"LieART/tutorial/QuickStartTutorial\"\>":>
       None, "\<\"Copy Mathematica url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell["LieART/tutorial/QuickStartTutorial"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; Null], 
       Delimiter, "\<\"Copy web url\"\>":>
       Module[{DocumentationSearch`Private`nb$}, 
        DocumentationSearch`Private`nb$ = NotebookPut[
           Notebook[{
             Cell[
              BoxData[
               MakeBoxes[
                Hyperlink[
                "http://reference.wolfram.com/mathematica/LieART/tutorial/\
QuickStartTutorial.html"], StandardForm]], "Input", TextClipboardType -> 
              "PlainText"]}, Visible -> False]]; 
        SelectionMove[DocumentationSearch`Private`nb$, All, Notebook]; 
        FrontEndTokenExecute[DocumentationSearch`Private`nb$, "Copy"]; 
        NotebookClose[DocumentationSearch`Private`nb$]; 
        Null], "\<\"Go to web url\"\>":>FrontEndExecute[{
         NotebookLocate[{
           URL[
            StringJoin[
            "http://reference.wolfram.com/mathematica/", 
             "LieART/tutorial/QuickStartTutorial", ".html"]], None}]}]},
       Appearance->None,
       MenuAppearance->Automatic]],
      LineSpacing->{1.4, 0}]
    }], "AnchorBar"]}
  }]], "AnchorBarGrid",
 GridBoxOptions->{GridBoxItemSize->{"Columns" -> {
     Scaled[0.65], {
      Scaled[0.34]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
   "RowsIndexed" -> {}}},
 CellID->1],

Cell[CellGroupData[{

Cell["LieART - Quick Start Tutorial", "Title",
 CellID->509267359],

Cell["\<\
The application LieART provides functions to perform the most common tasks \
associated with irreducible representations (irreps) of the classical and \
exceptional Lie algebas.\
\>", "Text",
 CellID->1534169418],

Cell["This loads the package:", "MathCaption",
 CellID->1167244701],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"<<", "LieART`"}]], "Input",
 CellLabel->"In[1]:=",
 CellID->1092165306],

Cell[CellGroupData[{

Cell[BoxData[
 FormBox["\<\"LieART 1.1.4\"\>", TraditionalForm]], "Print",
 CellLabel->"During evaluation of In[8]:=",
 CellID->1077823253],

Cell[BoxData[
 FormBox["\<\"last revised 6 August 2014\"\>", TraditionalForm]], "Print",
 CellLabel->"During evaluation of In[8]:=",
 CellID->61599756]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Entering Irreducible Representations", "Section",
 CellID->1449799695],

Cell[TextData[{
 "Irreducible representations (irreps) are internally decribed by their \
Dynkin label with a combined head of ",
 StyleBox["Irrep", "InlineCode"],
 " and the Lie algebra. "
}], "Text",
 CellID->374349434],

Cell[BoxData[GridBox[{
   {
    RowBox[{
     RowBox[{
      ButtonBox["Irrep",
       BaseStyle->"Link",
       ButtonData->"paclet:LieART/ref/Irrep"], "[", 
      StyleBox["algebraClass", "TI"], "]"}], "[", 
     StyleBox["label", "TI"], "]"}], Cell[TextData[{
     "irrep described by its ",
     StyleBox["algebraClass", "TI"],
     " and Dynkin ",
     StyleBox["label.", "TI"]
    }], "TableText"]}
  }]], "DefinitionBox",
 CellID->2096742444],

Cell["Entering irreps by Dynkin label.", "Caption",
 CellID->1891092685],

Cell[TextData[{
 "The ",
 Cell[BoxData[
  StyleBox["algebraClass", "TI"]], "InlineFormula"],
 " follows the Dynkin classification of simple Lie algebras and can only be \
",
 Cell[BoxData[
  ButtonBox["A",
   BaseStyle->"Link",
   ButtonData->"paclet:LieART/ref/A"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  ButtonBox["B",
   BaseStyle->"Link",
   ButtonData->"paclet:LieART/ref/B"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  ButtonBox["C",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/C"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  ButtonBox["D",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/D"]], "InlineFormula"],
 " for the classical algebras and ",
 Cell[BoxData[
  ButtonBox["E6",
   BaseStyle->"Link",
   ButtonData->"paclet:LieART/ref/E6"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  ButtonBox["E7",
   BaseStyle->"Link",
   ButtonData->"paclet:LieART/ref/E7"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  ButtonBox["E8",
   BaseStyle->"Link",
   ButtonData->"paclet:LieART/ref/E8"]], "InlineFormula"],
 ", ",
 Cell[BoxData[
  ButtonBox["F4",
   BaseStyle->"Link",
   ButtonData->"paclet:LieART/ref/F4"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  ButtonBox["G2",
   BaseStyle->"Link",
   ButtonData->"paclet:LieART/ref/G2"]], "InlineFormula"],
 " for the exceptional algebras. The precise classical algebra is determined \
by the length of the Dynkin label."
}], "Text",
 CellID->144687180],

Cell[TextData[{
 "Entering the ",
 Cell[BoxData[
  StyleBox[
   OverscriptBox["10", "_"],
   FontWeight->"Bold"]], "InlineFormula"],
 " of SU(5) by its Dynkin label and algebra class:"
}], "MathCaption",
 CellID->836781195],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Irrep", "[", "A", "]"}], "[", 
   RowBox[{"0", ",", "0", ",", "1", ",", "0"}], "]"}], "//", 
  "FullForm"}]], "Input",
 CellLabel->"In[18]:=",
 CellID->2058623809],

Cell[BoxData[
 FormBox[
  TagBox[
   StyleBox[
    RowBox[{
     RowBox[{"Irrep", "[", "A", "]"}], "[", 
     RowBox[{"0", ",", "0", ",", "1", ",", "0"}], "]"}],
    ShowSpecialCharacters->False,
    ShowStringCharacters->True,
    NumberMarks->True],
   FullForm], TraditionalForm]], "Output",
 ImageSize->{120, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[18]//FullForm=",
 CellID->1066064808]
}, Open  ]],

Cell[TextData[{
 "In ",
 Cell[BoxData[
  ButtonBox["StandardForm",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/StandardForm"]], "InlineFormula"],
 " the irrep is displayed in the textbook notation of Dynkin labels:"
}], "MathCaption",
 CellID->1221898746],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Irrep", "[", "A", "]"}], "[", 
   RowBox[{"0", ",", "0", ",", "1", ",", "0"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellLabel->"In[19]:=",
 CellID->711449157],

Cell[BoxData["\<\"(0010)\"\>"], "Output",
 ImageSize->{47, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[19]//StandardForm=",
 CellID->94736506]
}, Open  ]],

Cell[TextData[{
 "In ",
 Cell[BoxData[
  ButtonBox["TraditionalForm",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/TraditionalForm"]], "InlineFormula"],
 " (default) the irrep is displayed by its dimensional name:"
}], "MathCaption",
 CellID->681119358],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Irrep", "[", "A", "]"}], "[", 
  RowBox[{"0", ",", "0", ",", "1", ",", "0"}], "]"}]], "Input",
 CellLabel->"In[20]:=",
 CellID->937078051],

Cell[BoxData[
 FormBox[
  StyleBox[
   OverscriptBox["10", "_"],
   FontWeight->Bold], TraditionalForm]], "Output",
 ImageSize->{22, 14},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[20]=",
 CellID->1834936312]
}, Open  ]],

Cell[TextData[{
 "The default output format type of LieART is ",
 Cell[BoxData[
  ButtonBox["TraditionalForm",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/TraditionalForm"]], "InlineFormula"],
 ". The associated user setting is overwritten for the notebook LieART that \
is loaded in. For ",
 Cell[BoxData[
  ButtonBox["StandardForm",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/StandardForm"]], "InlineFormula"],
 " as output format type please set the global variable ",
 StyleBox["$DefaultOutputForm=StandardForm", "InlineCode"],
 "."
}], "Text",
 CellID->319667636],

Cell[TextData[{
 "As an example for entering an irrep of an exceptional algebra, consider the \
",
 Cell[BoxData[
  StyleBox["27",
   FontWeight->"Bold"]], "InlineFormula"],
 " of ",
 Cell[BoxData[
  SubscriptBox["E", "6"]], "InlineFormula"],
 ":"
}], "MathCaption",
 CellID->1881934180],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Irrep", "[", "E6", "]"}], "[", 
  RowBox[{"1", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0"}], 
  "]"}]], "Input",
 CellLabel->"In[21]:=",
 CellID->464169775],

Cell[BoxData[
 FormBox[
  StyleBox["27",
   FontWeight->Bold], TraditionalForm]], "Output",
 ImageSize->{23, 14},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[21]=",
 CellID->1025418079]
}, Open  ]],

Cell["\<\
Irreps may also be entered by their dimensional name. The package transforms \
the irrep into its Dynkin label. Since the algebra of an irrep of a classical \
Lie algebra becomes ambiguous with only the dimensional name, it has to be \
specified.\
\>", "Text",
 CellGroupingRules->{"GroupTogetherGrouping", 31},
 CellID->1468912962],

Cell[BoxData[GridBox[{
   {
    RowBox[{
     RowBox[{
      ButtonBox["Irrep",
       BaseStyle->"Link",
       ButtonData->"paclet:LieART/ref/Irrep"], "[", 
      StyleBox["algebra", "TI"], "]"}], "[", 
     StyleBox["dimname", "TI"], "]"}], Cell[TextData[{
     "irrep entered by its ",
     StyleBox["algebra", "TI"],
     " and dimensional name ",
     StyleBox["dimname.", "TI"]
    }], "TableText"]}
  }]], "DefinitionBox",
 CellID->792167334],

Cell["Entering irreps by dimensional name.", "Caption",
 CellID->1579371899],

Cell[TextData[{
 "Entering the ",
 Cell[BoxData[
  OverscriptBox["10", "_"]], "InlineFormula",
  FontWeight->"Bold"],
 StyleBox[" ",
  FontWeight->"Bold"],
 "of SU(5) by its dimensional name specifying the algebra by its Dynkin \
classification ",
 Cell[BoxData[
  SubscriptBox["A", "4"]], "InlineFormula"],
 ":"
}], "MathCaption",
 CellID->109632626],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Irrep", "[", "A4", "]"}], "[", 
   RowBox[{"Bar", "[", "10", "]"}], "]"}], "//", "InputForm"}]], "Input",
 CellLabel->"In[22]:=",
 CellID->333587606],

Cell["Irrep[A][0, 0, 1, 0]", "Output",
 ImageSize->{114, 14},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[22]//InputForm=",
 CellID->1531071855]
}, Open  ]],

Cell["The traditional name of the algebra SU(5) may also be used:", \
"MathCaption",
 CellID->1511260992],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Irrep", "[", "SU5", "]"}], "[", 
   RowBox[{"Bar", "[", "10", "]"}], "]"}], "//", "InputForm"}]], "Input",
 CellLabel->"In[23]:=",
 CellID->1425383264],

Cell["Irrep[A][0, 0, 1, 0]", "Output",
 ImageSize->{114, 14},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[23]//InputForm=",
 CellID->1118657973]
}, Open  ]],

Cell[TextData[{
 "Irreps of product algebras like SU(3)\[CircleTimes]SU(2)\[CircleTimes]U(1) \
are specified by ",
 Cell[BoxData[
  ButtonBox["ProductIrrep",
   BaseStyle->"Link",
   ButtonData->"paclet:LieART/ref/ProductIrrep"]], "InlineFormula"],
 " with the individual irreps of simple Lie algebras as arguments."
}], "Text",
 CellID->76966508],

Cell[BoxData[GridBox[{
   {
    RowBox[{
     ButtonBox[
      ButtonBox["ProductIrrep",
       BaseStyle->"Link",
       ButtonData->"paclet:LieART/ref/ProductIrrep"],
      BaseStyle->"Link",
      ButtonData->"paclet:LieART/ref/ProductIrrep"], "[", 
     StyleBox["irreps", "TI"], "]"}], Cell[TextData[{
     "head of product ",
     StyleBox["irreps", "TI"],
     ", gathering irreps of simple Lie algebras."
    }], "TableText"]}
  }]], "DefinitionBox",
 CellID->616137037],

Cell["Product irreps.", "Caption",
 CellID->722483301],

Cell[TextData[{
 "The product irrep ",
 Cell[BoxData[
  RowBox[{"(", 
   StyleBox[
    RowBox[{"3", ",", 
     OverscriptBox["3", "_"]}],
    FontWeight->"Bold"], ")"}]], "InlineFormula"],
 " of SU(3)\[CircleTimes]SU(3):"
}], "MathCaption",
 CellID->1501046753],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ProductIrrep", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Irrep", "[", "SU3", "]"}], "[", "3", "]"}], ",", 
   RowBox[{
    RowBox[{"Irrep", "[", "SU3", "]"}], "[", 
    RowBox[{"Bar", "[", "3", "]"}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[24]:=",
 CellID->1686390499],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    StyleBox["3",
     FontWeight->Bold],
    TraditionalForm], ",", 
   FormBox[
    StyleBox[
     OverscriptBox["3", "_"],
     FontWeight->Bold],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 ImageSize->{41, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[24]=",
 CellID->878148076]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellLabel->"In[25]:=",
 CellID->680021955],

Cell["ProductIrrep[Irrep[A][1, 0], Irrep[A][0, 1]]", "Output",
 ImageSize->{260, 14},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[25]//InputForm=",
 CellID->1800749348]
}, Open  ]],

Cell[CellGroupData[{

Cell["ProductIrrep[Irrep[A][1, 0], Irrep[A][0, 1]]", "Input",
 CellLabel->"In[26]:=",
 CellID->1308426208],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    StyleBox["3",
     FontWeight->Bold],
    TraditionalForm], ",", 
   FormBox[
    StyleBox[
     OverscriptBox["3", "_"],
     FontWeight->Bold],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 ImageSize->{41, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[26]=",
 CellID->1831185662]
}, Open  ]],

Cell["\<\
Take for example the left-handed quark doublet in the Standard-Model gauge \
group SU(3)\[CircleTimes]SU(2)\[CircleTimes]U(1) (The U(1) charge is not \
typeset in bold face):\
\>", "MathCaption",
 CellID->1021666716],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ProductIrrep", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Irrep", "[", "SU3", "]"}], "[", "3", "]"}], ",", 
   RowBox[{
    RowBox[{"Irrep", "[", "SU2", "]"}], "[", "2", "]"}], ",", 
   RowBox[{
    RowBox[{"Irrep", "[", "U1", "]"}], "[", 
    RowBox[{"1", "/", "3"}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[27]:=",
 CellID->1109740049],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    StyleBox["3",
     FontWeight->Bold],
    TraditionalForm], ",", 
   FormBox[
    StyleBox["2",
     FontWeight->Bold],
    TraditionalForm], ")", "\[InvisibleSpace]", 
   FormBox["\<\"(1/3)\"\>",
    TraditionalForm]}], TraditionalForm]], "Output",
 ImageSize->{76, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[27]=",
 CellID->643938528]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellLabel->"In[28]:=",
 CellID->1050432190],

Cell["\<\
ProductIrrep[Irrep[A][1, 0], Irrep[A][1], 
 Irrep[U][1/3]]\
\>", "Output",
 ImageSize->{247, 29},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[28]//InputForm=",
 CellID->874876204]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Decomposing Tensor Products ", "Section",
 CellGroupingRules->{"SectionGrouping", Inherited},
 CellID->354599395],

Cell[BoxData[GridBox[{
   {
    RowBox[{
     ButtonBox["DecomposeProduct",
      BaseStyle->"Link",
      ButtonData->"paclet:LieART/ref/DecomposeProduct"], "[", 
     StyleBox["irreps", "TI"], "]"}], Cell[TextData[{
     "decomposes the tensor product of several ",
     StyleBox["irreps.", "TI"]
    }], "TableText"]}
  }]], "DefinitionBox",
 CellID->1309055104],

Cell["Tensor product decomposition.", "Caption",
 CellID->1795200765],

Cell[TextData[{
 "Decompose the tensor product ",
 Cell[BoxData[
  RowBox[{
   StyleBox["3",
    FontWeight->"Bold"], "\[CircleTimes]", 
   StyleBox[
    OverscriptBox["3", "_"],
    FontWeight->"Bold"]}]], "InlineFormula"],
 " of SU(3):"
}], "MathCaption",
 CellID->1189993846],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DecomposeProduct", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Irrep", "[", "SU3", "]"}], "[", "3", "]"}], ",", 
   RowBox[{
    RowBox[{"Irrep", "[", "SU3", "]"}], "[", 
    RowBox[{"Bar", "[", "3", "]"}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[29]:=",
 CellID->1332771530],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    StyleBox["1",
     FontWeight->Bold],
    TraditionalForm], "+", 
   FormBox[
    StyleBox["8",
     FontWeight->Bold],
    TraditionalForm]}], TraditionalForm]], "Output",
 ImageSize->{35, 14},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[29]=",
 CellID->1013247494]
}, Open  ]],

Cell[TextData[{
 "Decompose the tensor product ",
 Cell[BoxData[
  RowBox[{
   StyleBox["27",
    FontWeight->"Bold"], "\[CircleTimes]", 
   StyleBox[
    OverscriptBox["27", "_"],
    FontWeight->"Bold"]}]], "InlineFormula"],
 " of ",
 Cell[BoxData[
  SubscriptBox["E", "6"]], "InlineFormula"],
 ":"
}], "MathCaption",
 CellID->133991959],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DecomposeProduct", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Irrep", "[", "E6", "]"}], "[", "27", "]"}], ",", " ", 
   RowBox[{
    RowBox[{"Irrep", "[", "E6", "]"}], "[", 
    RowBox[{"Bar", "[", "27", "]"}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[30]:=",
 CellID->101963549],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    StyleBox["1",
     FontWeight->Bold],
    TraditionalForm], "+", 
   FormBox[
    StyleBox["78",
     FontWeight->Bold],
    TraditionalForm], "+", 
   FormBox[
    StyleBox["650",
     FontWeight->Bold],
    TraditionalForm]}], TraditionalForm]], "Output",
 ImageSize->{84, 14},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[30]=",
 CellID->2099691517]
}, Open  ]],

Cell[TextData[{
 "Decompose the tensor product ",
 Cell[BoxData[
  RowBox[{
   StyleBox["3",
    FontWeight->"Bold"], "\[CircleTimes]", 
   StyleBox["3",
    FontWeight->"Bold"], "\[CircleTimes]", 
   StyleBox["3",
    FontWeight->"Bold"]}]], "InlineFormula"],
 " of SU(3):"
}], "MathCaption",
 CellID->1984894909],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DecomposeProduct", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Irrep", "[", "SU3", "]"}], "[", "3", "]"}], ",", 
   RowBox[{
    RowBox[{"Irrep", "[", "SU3", "]"}], "[", "3", "]"}], ",", 
   RowBox[{
    RowBox[{"Irrep", "[", "SU3", "]"}], "[", "3", "]"}]}], "]"}]], "Input",
 CellLabel->"In[31]:=",
 CellID->1356021807],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    StyleBox["1",
     FontWeight->Bold],
    TraditionalForm], "+", 
   RowBox[{"2", "(", 
    FormBox[
     StyleBox["8",
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   FormBox[
    StyleBox["10",
     FontWeight->Bold],
    TraditionalForm]}], TraditionalForm]], "Output",
 ImageSize->{89, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[31]=",
 CellID->697007864]
}, Open  ]],

Cell[TextData[{
 "Decompose the tensor product ",
 Cell[BoxData[
  RowBox[{
   StyleBox["8",
    FontWeight->"Bold"], "\[CircleTimes]", 
   StyleBox["8",
    FontWeight->"Bold"]}]], "InlineFormula"],
 " of SU(3):"
}], "MathCaption",
 CellID->1005626329],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DecomposeProduct", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Irrep", "[", "SU3", "]"}], "[", "8", "]"}], ",", 
   RowBox[{
    RowBox[{"Irrep", "[", "SU3", "]"}], "[", "8", "]"}]}], "]"}]], "Input",
 CellLabel->"In[32]:=",
 CellID->1887184167],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    StyleBox["1",
     FontWeight->Bold],
    TraditionalForm], "+", 
   RowBox[{"2", "(", 
    FormBox[
     StyleBox["8",
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   FormBox[
    StyleBox["10",
     FontWeight->Bold],
    TraditionalForm], "+", 
   FormBox[
    StyleBox[
     OverscriptBox["10", "_"],
     FontWeight->Bold],
    TraditionalForm], "+", 
   FormBox[
    StyleBox["27",
     FontWeight->Bold],
    TraditionalForm]}], TraditionalForm]], "Output",
 ImageSize->{152, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[32]=",
 CellID->1802103167]
}, Open  ]],

Cell[TextData[{
 "Internally a sum of irreps is represented by ",
 Cell[BoxData[
  ButtonBox["IrrepPlus",
   BaseStyle->"Link",
   ButtonData->"paclet:LieART/ref/IrrepPlus"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  ButtonBox["IrrepTimes",
   BaseStyle->"Link",
   ButtonData->"paclet:LieART/ref/IrrepTimes"]], "InlineFormula"],
 ", an analog of the built-in functions ",
 Cell[BoxData[
  ButtonBox["Plus",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Plus"]], "InlineFormula"],
 " and ",
 Cell[BoxData[
  ButtonBox["Times",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Times"]], "InlineFormula"],
 ":"
}], "MathCaption",
 CellID->683723966],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellLabel->"In[33]:=",
 CellID->997413459],

Cell["\<\
IrrepPlus[Irrep[A][0, 0], IrrepTimes[2, 
  Irrep[A][1, 1]], Irrep[A][3, 0], Irrep[A][0, 3], 
 Irrep[A][2, 2]]\
\>", "Output",
 ImageSize->{283, 44},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[33]//InputForm=",
 CellID->1579939891]
}, Open  ]],

Cell[TextData[{
 "Results can be transformed into a list of irreps with ",
 Cell[BoxData[
  ButtonBox["IrrepList",
   BaseStyle->"Link",
   ButtonData->"paclet:LieART/ref/IrrepList"]], "InlineFormula"],
 ", suitable for further processing with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " built-in functions like ",
 Cell[BoxData[
  ButtonBox["Select",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Select"]], "InlineFormula"],
 " or ",
 Cell[BoxData[
  ButtonBox["Cases",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Cases"]], "InlineFormula"],
 ":"
}], "MathCaption",
 CellID->1581190875],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "IrrepList"}]], "Input",
 CellLabel->"In[34]:=",
 CellID->863826808],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["8",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["8",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["10",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox[
      OverscriptBox["10", "_"],
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["27",
      FontWeight->Bold],
     TraditionalForm]}], "}"}], TraditionalForm]], "Output",
 ImageSize->{136, 18},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[34]=",
 CellID->865403289]
}, Open  ]],

Cell[TextData[{
 "Decompose the tensor product ",
 Cell[BoxData[
  RowBox[{
   StyleBox["4",
    FontWeight->"Bold"], "\[CircleTimes]", 
   StyleBox["4",
    FontWeight->"Bold"], "\[CircleTimes]", 
   StyleBox["6",
    FontWeight->"Bold"], "\[CircleTimes]", 
   StyleBox["15",
    FontWeight->"Bold"]}]], "InlineFormula"],
 " of SU(4):"
}], "MathCaption",
 CellID->902430666],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DecomposeProduct", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Irrep", "[", "SU4", "]"}], "[", "4", "]"}], ",", " ", 
   RowBox[{
    RowBox[{"Irrep", "[", "SU4", "]"}], "[", "4", "]"}], ",", " ", 
   RowBox[{
    RowBox[{"Irrep", "[", "SU4", "]"}], "[", "6", "]"}], ",", " ", 
   RowBox[{
    RowBox[{"Irrep", "[", "SU4", "]"}], "[", "15", "]"}]}], "]"}]], "Input",
 CellLabel->"In[35]:=",
 CellID->5612545],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", "(", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"7", "(", 
    FormBox[
     StyleBox["15",
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"4", "(", 
    FormBox[
     SuperscriptBox[
      StyleBox["20",
       FontWeight->Bold], "\[Prime]"],
     TraditionalForm], ")"}], "+", 
   FormBox[
    StyleBox["35",
     FontWeight->Bold],
    TraditionalForm], "+", 
   RowBox[{"5", "(", 
    FormBox[
     StyleBox["45",
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"3", "(", 
    FormBox[
     StyleBox[
      OverscriptBox["45", "_"],
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"3", "(", 
    FormBox[
     StyleBox["84",
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"2", "(", 
    FormBox[
     StyleBox["175",
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   FormBox[
    StyleBox["256",
     FontWeight->Bold],
    TraditionalForm]}], TraditionalForm]], "Output",
 ImageSize->{443, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[35]=",
 CellID->267790776]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " built-in command ",
 Cell[BoxData[
  ButtonBox["Times",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Times"]], "InlineFormula"],
 " for products is replaced by ",
 Cell[BoxData[
  ButtonBox["DecomposeProduct",
   BaseStyle->"Link",
   ButtonData->"paclet:LieART/ref/DecomposeProduct"]], "InlineFormula"],
 " for irreps as arguments. E.g., decompose the tensor product ",
 Cell[BoxData[
  RowBox[{
   StyleBox[
    OverscriptBox["10", "_"],
    FontWeight->"Bold"], "\[CircleTimes]", 
   StyleBox["24",
    FontWeight->"Bold"], "\[CircleTimes]", 
   StyleBox["45",
    FontWeight->"Bold"]}]], "InlineFormula"],
 " of SU(5):"
}], "MathCaption",
 CellID->1821879462],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Irrep", "[", "SU5", "]"}], "[", 
   RowBox[{"Bar", "[", "10", "]"}], "]"}], "*", 
  RowBox[{
   RowBox[{"Irrep", "[", "SU5", "]"}], "[", "24", "]"}], "*", 
  RowBox[{
   RowBox[{"Irrep", "[", "SU5", "]"}], "[", "45", "]"}]}]], "Input",
 CellLabel->"In[36]:=",
 CellID->1779904617],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"3", "(", 
    FormBox[
     StyleBox[
      OverscriptBox["5", "_"],
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"6", "(", 
    FormBox[
     StyleBox[
      OverscriptBox["45", "_"],
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"3", "(", 
    FormBox[
     StyleBox[
      OverscriptBox["50", "_"],
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"5", "(", 
    FormBox[
     StyleBox[
      OverscriptBox["70", "_"],
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"2", "(", 
    FormBox[
     StyleBox[
      OverscriptBox["105", "_"],
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   FormBox[
    SuperscriptBox[
     StyleBox[
      OverscriptBox["175", "_"],
      FontWeight->Bold], "\[Prime]\[Prime]"],
    TraditionalForm], "+", 
   RowBox[{"6", "(", 
    FormBox[
     StyleBox[
      OverscriptBox["280", "_"],
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"2", "(", 
    FormBox[
     SuperscriptBox[
      StyleBox[
       OverscriptBox["280", "_"],
       FontWeight->Bold], "\[Prime]"],
     TraditionalForm], ")"}], "+", 
   FormBox[
    StyleBox[
     OverscriptBox["420", "_"],
     FontWeight->Bold],
    TraditionalForm], "+", 
   FormBox[
    SuperscriptBox[
     StyleBox[
      OverscriptBox["450", "_"],
      FontWeight->Bold], "\[Prime]"],
    TraditionalForm], "+", 
   RowBox[{"3", "(", 
    FormBox[
     StyleBox[
      OverscriptBox["480", "_"],
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"2", "(", 
    FormBox[
     StyleBox[
      OverscriptBox["720", "_"],
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   FormBox[
    StyleBox[
     OverscriptBox["1120", "_"],
     FontWeight->Bold],
    TraditionalForm], "+", 
   FormBox[
    StyleBox[
     OverscriptBox["2520", "_"],
     FontWeight->Bold],
    TraditionalForm]}], TraditionalForm]], "Output",
 ImageSize->{383, 35},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[36]=",
 CellID->812961123]
}, Open  ]],

Cell[TextData[{
 "For powers of irreps the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " built-in command ",
 ButtonBox["Power",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Power"],
 " may be used. E.g., decompose the tensor product ",
 Cell[BoxData[
  RowBox[{
   StyleBox["27",
    FontWeight->"Bold"], "\[CircleTimes]", 
   StyleBox["27",
    FontWeight->"Bold"], "\[CircleTimes]", 
   StyleBox["27",
    FontWeight->"Bold"]}]], "InlineFormula"],
 " of ",
 Cell[BoxData[
  SubscriptBox["E", "6"]], "InlineFormula"],
 ":"
}], "MathCaption",
 CellID->1431754643],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Irrep", "[", "E6", "]"}], "[", "27", "]"}], "^", "3"}]], "Input",
 CellLabel->"In[37]:=",
 CellID->808921778],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    StyleBox["1",
     FontWeight->Bold],
    TraditionalForm], "+", 
   RowBox[{"2", "(", 
    FormBox[
     StyleBox["78",
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"3", "(", 
    FormBox[
     StyleBox["650",
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   FormBox[
    StyleBox["2925",
     FontWeight->Bold],
    TraditionalForm], "+", 
   FormBox[
    StyleBox["3003",
     FontWeight->Bold],
    TraditionalForm], "+", 
   RowBox[{"2", "(", 
    FormBox[
     StyleBox["5824",
      FontWeight->Bold],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 ImageSize->{301, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[37]=",
 CellID->1630197054]
}, Open  ]],

Cell[TextData[{
 "Decompose tensor products of product irreps",
 Cell[BoxData[
  RowBox[{" ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      StyleBox["3",
       FontWeight->"Bold"], ",", 
      StyleBox[
       OverscriptBox["3", "_"],
       FontWeight->"Bold"], 
      StyleBox[",",
       FontWeight->"Bold"], 
      StyleBox["1",
       FontWeight->"Bold"]}], ")"}], "\[CircleTimes]", 
    RowBox[{"(", 
     RowBox[{
      StyleBox[
       OverscriptBox["3", "_"],
       FontWeight->"Bold"], ",", 
      StyleBox["3",
       FontWeight->"Bold"], ",", 
      StyleBox["1",
       FontWeight->"Bold"]}], ")"}]}]}]], "InlineFormula"],
 " of SU(3)\[CircleTimes]SU(3)\[CircleTimes]SU(3):"
}], "MathCaption",
 CellID->328377256],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DecomposeProduct", "[", 
  RowBox[{
   RowBox[{"ProductIrrep", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Irrep", "[", "SU3", "]"}], "[", "3", "]"}], ",", 
     RowBox[{
      RowBox[{"Irrep", "[", "SU3", "]"}], "[", 
      RowBox[{"Bar", "[", "3", "]"}], "]"}], ",", 
     RowBox[{
      RowBox[{"Irrep", "[", "SU3", "]"}], "[", "1", "]"}]}], "]"}], ",", 
   RowBox[{"ProductIrrep", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Irrep", "[", "SU3", "]"}], "[", 
      RowBox[{"Bar", "[", "3", "]"}], "]"}], ",", 
     RowBox[{
      RowBox[{"Irrep", "[", "SU3", "]"}], "[", "3", "]"}], ",", 
     RowBox[{
      RowBox[{"Irrep", "[", "SU3", "]"}], "[", "1", "]"}]}], "]"}]}], 
  "]"}]], "Input",
 CellLabel->"In[38]:=",
 CellID->451431300],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"(", 
    FormBox[
     StyleBox["8",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"(", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["8",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"(", 
    FormBox[
     StyleBox["8",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["8",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 ImageSize->{259, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[38]=",
 CellID->158054236]
}, Open  ]],

Cell[TextData[{
 "Decompose the tensor products ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     StyleBox["3",
      FontWeight->"Bold"], ",", 
     StyleBox["2",
      FontWeight->"Bold"]}], ")"}], "\[CircleTimes]", 
   RowBox[{"(", 
    RowBox[{
     StyleBox[
      OverscriptBox["3", "_"],
      FontWeight->"Bold"], ",", 
     StyleBox["1",
      FontWeight->"Bold"]}], ")"}]}]], "InlineFormula"],
 " of SU(3)\[CircleTimes]SU(2):"
}], "MathCaption",
 CellID->1480557077],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DecomposeProduct", "[", 
  RowBox[{
   RowBox[{"ProductIrrep", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Irrep", "[", "SU3", "]"}], "[", "3", "]"}], ",", 
     RowBox[{
      RowBox[{"Irrep", "[", "SU2", "]"}], "[", "2", "]"}]}], "]"}], ",", 
   RowBox[{"ProductIrrep", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Irrep", "[", "SU3", "]"}], "[", 
      RowBox[{"Bar", "[", "3", "]"}], "]"}], ",", 
     RowBox[{
      RowBox[{"Irrep", "[", "SU2", "]"}], "[", "1", "]"}]}], "]"}]}], 
  "]"}]], "Input",
 CellLabel->"In[39]:=",
 CellID->343055154],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["2",
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"(", 
    FormBox[
     StyleBox["8",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["2",
      FontWeight->Bold],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 ImageSize->{93, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[39]=",
 CellID->1660930165]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Decomposition to Subalgebras", "Section",
 CellID->1642176862],

Cell[BoxData[GridBox[{
   {
    RowBox[{
     ButtonBox["DecomposeIrrep",
      BaseStyle->"Link",
      ButtonData->"paclet:LieART/ref/DecomposeIrrep"], "[", 
     RowBox[{
      StyleBox["irrep", "TI"], ",", 
      StyleBox["subalgebra", "TI"]}], "]"}], Cell[TextData[{
     "decomposes ",
     StyleBox["irrep", "TI"],
     " to the specified ",
     StyleBox["subalgebra.", "TI"]
    }], "TableText"]},
   {
    RowBox[{
     ButtonBox["DecomposeIrrep",
      BaseStyle->"Link",
      ButtonData->"paclet:LieART/ref/DecomposeIrrep"], "[", 
     RowBox[{
      StyleBox["pIrrep", "TI"], ",", 
      StyleBox["subalgebra", "TI"], ",", " ", 
      StyleBox["pos", "TI"]}], "]"}], Cell[TextData[{
     "decompose the product irrep ",
     StyleBox["pirrep", "TI"],
     " at position ",
     StyleBox["pos.", "TI"]
    }], "TableText"]}
  }]], "DefinitionBox",
 CellID->1279752208],

Cell["Decompose irreps and product irreps.", "Caption",
 CellID->1925211260],

Cell[TextData[{
 "Decompose the ",
 Cell[BoxData[
  OverscriptBox["10", "_"]], "InlineFormula",
  FontWeight->"Bold"],
 " of SU(5) to SU(3)\[CircleTimes]SU(2)\[CircleTimes]U(1):"
}], "MathCaption",
 CellID->442942768],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DecomposeIrrep", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Irrep", "[", "SU5", "]"}], "[", 
    RowBox[{"Bar", "[", "10", "]"}], "]"}], ",", 
   RowBox[{"ProductAlgebra", "[", 
    RowBox[{"SU3", ",", "SU2", ",", "U1"}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[40]:=",
 CellID->1304419061],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ")", "\[InvisibleSpace]", 
    FormBox["\<\"(6)\"\>",
     TraditionalForm]}], "+", 
   RowBox[{"(", 
    FormBox[
     StyleBox["3",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ")", "\[InvisibleSpace]", 
    FormBox["\<\"(-4)\"\>",
     TraditionalForm]}], "+", 
   RowBox[{"(", 
    FormBox[
     StyleBox[
      OverscriptBox["3", "_"],
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["2",
      FontWeight->Bold],
     TraditionalForm], ")", "\[InvisibleSpace]", 
    FormBox["\<\"(1)\"\>",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 ImageSize->{208, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[40]=",
 CellID->1523676287]
}, Open  ]],

Cell[TextData[{
 "Decompose the ",
 Cell[BoxData[
  StyleBox["10",
   FontWeight->"Bold"]], "InlineFormula"],
 " and the ",
 Cell[BoxData[
  StyleBox[
   OverscriptBox["5", "_"],
   FontWeight->"Bold"]], "InlineFormula"],
 " of SU(5) to SU(3)\[CircleTimes]SU(2)\[CircleTimes]U(1) (",
 Cell[BoxData[
  ButtonBox["DecomposeIrrep",
   BaseStyle->"Link",
   ButtonData->"paclet:LieART/ref/DecomposeIrrep"]], "InlineFormula"],
 " is ",
 Cell[BoxData[
  ButtonBox["Listable",
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Listable"]], "InlineFormula"],
 "):"
}], "MathCaption",
 CellID->455264099],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DecomposeIrrep", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Irrep", "[", "SU5", "]"}], "[", "10", "]"}], ",", 
     RowBox[{
      RowBox[{"Irrep", "[", "SU5", "]"}], "[", 
      RowBox[{"Bar", "[", "5", "]"}], "]"}]}], "}"}], ",", 
   RowBox[{"ProductAlgebra", "[", 
    RowBox[{"SU3", ",", "SU2", ",", "U1"}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[41]:=",
 CellID->2123210495],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      FormBox[
       StyleBox[
        OverscriptBox["3", "_"],
        FontWeight->Bold],
       TraditionalForm], ",", 
      FormBox[
       StyleBox["1",
        FontWeight->Bold],
       TraditionalForm], ")", "\[InvisibleSpace]", 
      FormBox["\<\"(4)\"\>",
       TraditionalForm]}], "+", 
     RowBox[{"(", 
      FormBox[
       StyleBox["3",
        FontWeight->Bold],
       TraditionalForm], ",", 
      FormBox[
       StyleBox["2",
        FontWeight->Bold],
       TraditionalForm], ")", "\[InvisibleSpace]", 
      FormBox["\<\"(-1)\"\>",
       TraditionalForm]}], "+", 
     RowBox[{"(", 
      FormBox[
       StyleBox["1",
        FontWeight->Bold],
       TraditionalForm], ",", 
      FormBox[
       StyleBox["1",
        FontWeight->Bold],
       TraditionalForm], ")", "\[InvisibleSpace]", 
      FormBox["\<\"(-6)\"\>",
       TraditionalForm]}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      FormBox[
       StyleBox[
        OverscriptBox["3", "_"],
        FontWeight->Bold],
       TraditionalForm], ",", 
      FormBox[
       StyleBox["1",
        FontWeight->Bold],
       TraditionalForm], ")", "\[InvisibleSpace]", 
      FormBox["\<\"(-2)\"\>",
       TraditionalForm]}], "+", 
     RowBox[{"(", 
      FormBox[
       StyleBox["1",
        FontWeight->Bold],
       TraditionalForm], ",", 
      FormBox[
       StyleBox["2",
        FontWeight->Bold],
       TraditionalForm], ")", "\[InvisibleSpace]", 
      FormBox["\<\"(3)\"\>",
       TraditionalForm]}]}]}], "}"}], TraditionalForm]], "Output",
 ImageSize->{371, 18},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[41]=",
 CellID->1049794380]
}, Open  ]],

Cell[TextData[{
 "Decompose the ",
 StyleBox["16",
  FontWeight->"Bold"],
 " of SO(10) to SU(5)\[CircleTimes]U(1):"
}], "MathCaption",
 CellID->1947924234],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DecomposeIrrep", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Irrep", "[", "SO10", "]"}], "[", "16", "]"}], ",", 
   RowBox[{"ProductAlgebra", "[", 
    RowBox[{"SU5", ",", "U1"}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[42]:=",
 CellID->277260301],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ")", "\[InvisibleSpace]", 
    FormBox["\<\"(-5)\"\>",
     TraditionalForm]}], "+", 
   RowBox[{"(", 
    FormBox[
     StyleBox[
      OverscriptBox["5", "_"],
      FontWeight->Bold],
     TraditionalForm], ")", "\[InvisibleSpace]", 
    FormBox["\<\"(3)\"\>",
     TraditionalForm]}], "+", 
   RowBox[{"(", 
    FormBox[
     StyleBox["10",
      FontWeight->Bold],
     TraditionalForm], ")", "\[InvisibleSpace]", 
    FormBox["\<\"(-1)\"\>",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 ImageSize->{174, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[42]=",
 CellID->9031146]
}, Open  ]],

Cell[TextData[{
 "Decompose the ",
 StyleBox["27",
  FontWeight->"Bold"],
 " of ",
 Cell[BoxData[
  SubscriptBox["E", "6"]], "InlineFormula"],
 " to SU(3)\[CircleTimes]SU(3)\[CircleTimes]SU(3):"
}], "MathCaption",
 CellID->906860298],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DecomposeIrrep", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Irrep", "[", "E6", "]"}], "[", "27", "]"}], ",", 
   RowBox[{"ProductAlgebra", "[", 
    RowBox[{"SU3", ",", "SU3", ",", "SU3"}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[43]:=",
 CellID->471735093],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    FormBox[
     StyleBox["3",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["3",
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"(", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["3",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox[
      OverscriptBox["3", "_"],
      FontWeight->Bold],
     TraditionalForm], ")"}], "+", 
   RowBox[{"(", 
    FormBox[
     StyleBox[
      OverscriptBox["3", "_"],
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox[
      OverscriptBox["3", "_"],
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 ImageSize->{192, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[43]=",
 CellID->479800872]
}, Open  ]],

Cell[TextData[{
 "Decompose the SU(3) irrep ",
 StyleBox["3",
  FontWeight->"Bold"],
 " in (",
 StyleBox["24",
  FontWeight->"Bold"],
 ",",
 StyleBox["3",
  FontWeight->"Bold"],
 ")(-3) of SU(5)\[CircleTimes]SU(3)\[CircleTimes]U(1) to \
SU(2)\[CircleTimes]U'(1), i.e., SU(5)\[CircleTimes]SU(3)\[CircleTimes]U(1)\
\[Rule]SU(5)\[CircleTimes]SU(2)\[CircleTimes]U'(1)\[CircleTimes]U(1):"
}], "MathCaption",
 CellID->1532918075],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DecomposeIrrep", "[", 
  RowBox[{
   RowBox[{"ProductIrrep", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Irrep", "[", "SU5", "]"}], "[", "24", "]"}], ",", 
     RowBox[{
      RowBox[{"Irrep", "[", "SU3", "]"}], "[", "3", "]"}], ",", 
     RowBox[{
      RowBox[{"Irrep", "[", "U1", "]"}], "[", 
      RowBox[{"-", "3"}], "]"}]}], "]"}], ",", 
   RowBox[{"ProductAlgebra", "[", 
    RowBox[{"SU2", ",", "U1"}], "]"}], ",", "2"}], "]"}]], "Input",
 CellLabel->"In[44]:=",
 CellID->1650450684],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    FormBox[
     StyleBox["24",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["1",
      FontWeight->Bold],
     TraditionalForm], ")", "\[InvisibleSpace]", 
    FormBox["\<\"(-2)\"\>",
     TraditionalForm], "\[InvisibleSpace]", 
    FormBox["\<\"(-3)\"\>",
     TraditionalForm]}], "+", 
   RowBox[{"(", 
    FormBox[
     StyleBox["24",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["2",
      FontWeight->Bold],
     TraditionalForm], ")", "\[InvisibleSpace]", 
    FormBox["\<\"(1)\"\>",
     TraditionalForm], "\[InvisibleSpace]", 
    FormBox["\<\"(-3)\"\>",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 ImageSize->{218, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[44]=",
 CellID->540895483]
}, Open  ]],

Cell["The same decomposition as above displayed as branching rule:", \
"MathCaption",
 CellID->981618749],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"IrrepRule", "[", 
    RowBox[{"#", ",", 
     RowBox[{"DecomposeIrrep", "[", 
      RowBox[{"#", ",", 
       RowBox[{"ProductAlgebra", "[", 
        RowBox[{"SU2", ",", "U1"}], "]"}], ",", "2"}], "]"}]}], "]"}], "&"}], 
  "@", 
  RowBox[{"ProductIrrep", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Irrep", "[", "SU5", "]"}], "[", "24", "]"}], ",", 
    RowBox[{
     RowBox[{"Irrep", "[", "SU3", "]"}], "[", "3", "]"}], ",", 
    RowBox[{
     RowBox[{"Irrep", "[", "U1", "]"}], "[", 
     RowBox[{"-", "3"}], "]"}]}], "]"}]}]], "Input",
 CellLabel->"In[45]:=",
 CellID->1394022133],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    FormBox[
     StyleBox["24",
      FontWeight->Bold],
     TraditionalForm], ",", 
    FormBox[
     StyleBox["3",
      FontWeight->Bold],
     TraditionalForm], ")", "\[InvisibleSpace]", 
    FormBox["\<\"(-3)\"\>",
     TraditionalForm]}], "\[RightArrow]", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      StyleBox["24",
       FontWeight->Bold],
      TraditionalForm], ",", 
     FormBox[
      StyleBox["1",
       FontWeight->Bold],
      TraditionalForm], ")", "\[InvisibleSpace]", 
     FormBox["\<\"(-2)\"\>",
      TraditionalForm], "\[InvisibleSpace]", 
     FormBox["\<\"(-3)\"\>",
      TraditionalForm]}], "+", 
    RowBox[{"(", 
     FormBox[
      StyleBox["24",
       FontWeight->Bold],
      TraditionalForm], ",", 
     FormBox[
      StyleBox["2",
       FontWeight->Bold],
      TraditionalForm], ")", "\[InvisibleSpace]", 
     FormBox["\<\"(1)\"\>",
      TraditionalForm], "\[InvisibleSpace]", 
     FormBox["\<\"(-3)\"\>",
      TraditionalForm]}]}]}], TraditionalForm]], "Output",
 ImageSize->{311, 16},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[45]=",
 CellID->95559239]
}, Open  ]],

Cell["\<\
Branching rules for all totally antisymmetric irreps, so-called basic irreps, \
of SU(6) to SU(3)\[CircleTimes]SU(3)\[CircleTimes]U(1):\
\>", "MathCaption",
 CellID->1987934892],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"IrrepRule", "[", 
     RowBox[{"#", ",", 
      RowBox[{"DecomposeIrrep", "[", 
       RowBox[{"#", ",", 
        RowBox[{"ProductAlgebra", "[", 
         RowBox[{"SU3", ",", "SU3", ",", "U1"}], "]"}]}], "]"}]}], "]"}], 
    "&"}], "/@", 
   RowBox[{"BasicIrreps", "[", "SU6", "]"}]}], "//", "TableForm"}]], "Input",
 CellLabel->"In[46]:=",
 CellID->1514165693],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {
       RowBox[{
        FormBox[
         StyleBox["6",
          FontWeight->Bold],
         TraditionalForm], "\[RightArrow]", 
        RowBox[{
         RowBox[{"(", 
          FormBox[
           StyleBox["3",
            FontWeight->Bold],
           TraditionalForm], ",", 
          FormBox[
           StyleBox["1",
            FontWeight->Bold],
           TraditionalForm], ")", "\[InvisibleSpace]", 
          FormBox["\<\"(1)\"\>",
           TraditionalForm]}], "+", 
         RowBox[{"(", 
          FormBox[
           StyleBox["1",
            FontWeight->Bold],
           TraditionalForm], ",", 
          FormBox[
           StyleBox["3",
            FontWeight->Bold],
           TraditionalForm], ")", "\[InvisibleSpace]", 
          FormBox["\<\"(-1)\"\>",
           TraditionalForm]}]}]}]},
      {
       RowBox[{
        FormBox[
         StyleBox["15",
          FontWeight->Bold],
         TraditionalForm], "\[RightArrow]", 
        RowBox[{
         RowBox[{"(", 
          FormBox[
           StyleBox[
            OverscriptBox["3", "_"],
            FontWeight->Bold],
           TraditionalForm], ",", 
          FormBox[
           StyleBox["1",
            FontWeight->Bold],
           TraditionalForm], ")", "\[InvisibleSpace]", 
          FormBox["\<\"(2)\"\>",
           TraditionalForm]}], "+", 
         RowBox[{"(", 
          FormBox[
           StyleBox["1",
            FontWeight->Bold],
           TraditionalForm], ",", 
          FormBox[
           StyleBox[
            OverscriptBox["3", "_"],
            FontWeight->Bold],
           TraditionalForm], ")", "\[InvisibleSpace]", 
          FormBox["\<\"(-2)\"\>",
           TraditionalForm]}], "+", 
         RowBox[{"(", 
          FormBox[
           StyleBox["3",
            FontWeight->Bold],
           TraditionalForm], ",", 
          FormBox[
           StyleBox["3",
            FontWeight->Bold],
           TraditionalForm], ")", "\[InvisibleSpace]", 
          FormBox["\<\"(0)\"\>",
           TraditionalForm]}]}]}]},
      {
       RowBox[{
        FormBox[
         StyleBox["20",
          FontWeight->Bold],
         TraditionalForm], "\[RightArrow]", 
        RowBox[{
         RowBox[{"(", 
          FormBox[
           StyleBox["1",
            FontWeight->Bold],
           TraditionalForm], ",", 
          FormBox[
           StyleBox["1",
            FontWeight->Bold],
           TraditionalForm], ")", "\[InvisibleSpace]", 
          FormBox["\<\"(3)\"\>",
           TraditionalForm]}], "+", 
         RowBox[{"(", 
          FormBox[
           StyleBox["1",
            FontWeight->Bold],
           TraditionalForm], ",", 
          FormBox[
           StyleBox["1",
            FontWeight->Bold],
           TraditionalForm], ")", "\[InvisibleSpace]", 
          FormBox["\<\"(-3)\"\>",
           TraditionalForm]}], "+", 
         RowBox[{"(", 
          FormBox[
           StyleBox["3",
            FontWeight->Bold],
           TraditionalForm], ",", 
          FormBox[
           StyleBox[
            OverscriptBox["3", "_"],
            FontWeight->Bold],
           TraditionalForm], ")", "\[InvisibleSpace]", 
          FormBox["\<\"(-1)\"\>",
           TraditionalForm]}], "+", 
         RowBox[{"(", 
          FormBox[
           StyleBox[
            OverscriptBox["3", "_"],
            FontWeight->Bold],
           TraditionalForm], ",", 
          FormBox[
           StyleBox["3",
            FontWeight->Bold],
           TraditionalForm], ")", "\[InvisibleSpace]", 
          FormBox["\<\"(1)\"\>",
           TraditionalForm]}]}]}]},
      {
       RowBox[{
        FormBox[
         StyleBox[
          OverscriptBox["15", "_"],
          FontWeight->Bold],
         TraditionalForm], "\[RightArrow]", 
        RowBox[{
         RowBox[{"(", 
          FormBox[
           StyleBox["3",
            FontWeight->Bold],
           TraditionalForm], ",", 
          FormBox[
           StyleBox["1",
            FontWeight->Bold],
           TraditionalForm], ")", "\[InvisibleSpace]", 
          FormBox["\<\"(-2)\"\>",
           TraditionalForm]}], "+", 
         RowBox[{"(", 
          FormBox[
           StyleBox["1",
            FontWeight->Bold],
           TraditionalForm], ",", 
          FormBox[
           StyleBox["3",
            FontWeight->Bold],
           TraditionalForm], ")", "\[InvisibleSpace]", 
          FormBox["\<\"(2)\"\>",
           TraditionalForm]}], "+", 
         RowBox[{"(", 
          FormBox[
           StyleBox[
            OverscriptBox["3", "_"],
            FontWeight->Bold],
           TraditionalForm], ",", 
          FormBox[
           StyleBox[
            OverscriptBox["3", "_"],
            FontWeight->Bold],
           TraditionalForm], ")", "\[InvisibleSpace]", 
          FormBox["\<\"(0)\"\>",
           TraditionalForm]}]}]}]},
      {
       RowBox[{
        FormBox[
         StyleBox[
          OverscriptBox["6", "_"],
          FontWeight->Bold],
         TraditionalForm], "\[RightArrow]", 
        RowBox[{
         RowBox[{"(", 
          FormBox[
           StyleBox[
            OverscriptBox["3", "_"],
            FontWeight->Bold],
           TraditionalForm], ",", 
          FormBox[
           StyleBox["1",
            FontWeight->Bold],
           TraditionalForm], ")", "\[InvisibleSpace]", 
          FormBox["\<\"(-1)\"\>",
           TraditionalForm]}], "+", 
         RowBox[{"(", 
          FormBox[
           StyleBox["1",
            FontWeight->Bold],
           TraditionalForm], ",", 
          FormBox[
           StyleBox[
            OverscriptBox["3", "_"],
            FontWeight->Bold],
           TraditionalForm], ")", "\[InvisibleSpace]", 
          FormBox["\<\"(1)\"\>",
           TraditionalForm]}]}]}]}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 ImageSize->{321, 96},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[46]//TableForm=",
 CellID->664155987]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Young Tableaux", "Section",
 CellID->431069882],

Cell[TextData[{
 "The irreps of SU(N) have a correspondence to Young tableaux, which can be \
displayed by ",
 Cell[BoxData[
  ButtonBox["YoungTableau",
   BaseStyle->"Link",
   ButtonData->"paclet:LieART/ref/YoungTableau"]], "InlineFormula"],
 "."
}], "Text",
 CellID->1099666059],

Cell[BoxData[GridBox[{
   {
    RowBox[{
     ButtonBox["YoungTableau",
      BaseStyle->"Link",
      ButtonData->"paclet:LieART/ref/YoungTableau"], "[", 
     StyleBox["irrep", "TI"], "]"}], Cell[TextData[{
     "Displays the Young tableau associated with ",
     StyleBox["irrep", "TI"],
     " "
    }], "TableText"]}
  }]], "DefinitionBox",
 CellID->1528399319],

Cell["Young tableaux.", "Caption",
 CellID->1616075793],

Cell[TextData[{
 "Young tableau of the ",
 StyleBox["720",
  FontWeight->"Bold"],
 " of SU(5):"
}], "MathCaption",
 CellID->381847914],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"YoungTableau", "[", 
  RowBox[{
   RowBox[{"Irrep", "[", "A", "]"}], "[", 
   RowBox[{"1", ",", "2", ",", "0", ",", "1"}], "]"}], "]"}]], "Input",
 CellLabel->"In[47]:=",
 CellID->1550153586],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzt2MGpwlAURdGgDmzDUrQAB7ECBZ0KUZBUr88WDPtc5O0DfvyDxWXHWXaX
+3hbDcPw2LY/4/m1n6bzfNq0fw7z83pZty+n9jm2z/f7u5sNPW3JU/ojaCl9
Nw8tpe/moaX03Ty0lL6bh5bSd/PQUvpuHlpK381DS+m7eWgpfTcPuyrtZ789
pYVPOAyrrKUcrLKWcrDKWsrBKmspB6uspRysspZysMpaysEqaykHq6ylHHz/
4aukJaUlNj9LaZufpbTNz1La5mcpbfOzlLb5WUrb/CylbX6W0jY/S2mbX9Vb
nfy6+k1LbH6W0jY/S2mbn6W0zc9S2uZnKW3zs5S2+VlK2/wspW1+/bxd6Wcf
LrWzYw==
    "], {{0, 83}, {78, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{78, 83},
  PlotRange->{{0, 78}, {0, 83}}]], "Output",
 ImageSize->{82, 83},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[47]=",
 CellID->239053307]
}, Open  ]],

Cell["\<\
Display Young tableaux of SU(4) irreps with a maximum of one column per box \
count:\
\>", "MathCaption",
 CellID->1293870584],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Row", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Row", "[", 
      RowBox[{"{", 
       RowBox[{"#", ",", "\"\<: \>\"", ",", 
        RowBox[{"YoungTableau", "[", "#", "]"}]}], "}"}], "]"}], "&"}], "/@", 
    
    RowBox[{"SortBy", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Irrep", "[", "A", "]"}], "@@@", 
       RowBox[{"Tuples", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0", ",", "1"}], "}"}], ",", "3"}], "]"}]}], ",", "Dim"}], 
     "]"}]}], ",", 
   RowBox[{"Spacer", "[", "10", "]"}]}], "]"}]], "Input",
 CellLabel->"In[48]:=",
 CellID->794726788],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztm2tsU2UYx9utazs3trHhFGEMHBu6qMRERC4yNIB+UESMuuBlDMFLlBnw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    "], {{0, 63}, {548, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSize->Automatic,
  ImageSizeRaw->{548, 63},
  PlotRange->{{0, 548}, {0, 63}}]], "Output",
 ImageSize->{552, 63},
 ImageMargins->{{0, 0}, {0, 0}},
 ImageRegion->{{0, 1}, {0, 1}},
 CellLabel->"Out[48]=",
 CellID->314369717]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[" ", "FooterCell"]
},
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{725, 750},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowTitle->"LieART - Quick Start Tutorial - Wolfram Mathematica",
TaggingRules->{
 "ModificationHighlight" -> False, "ColorType" -> "TutorialColor", 
  "LinkTrails" -> GridBox[{{
      RowBox[{
        ButtonBox[
        "LieART: Lie Algebras and Representation Theory", ButtonData -> 
         "paclet:LieART/guide/LieART", 
         BaseStyle -> {"Link", "DockedLinkTrail"}], 
        Cell[">", "DockedLinkTrailSeparator"]}]}}, ColumnAlignments -> Left], 
  "Metadata" -> {
   "built" -> "{2014, 8, 6, 10, 24, 10.886216}", 
    "history" -> {"0.9", "", "", ""}, "context" -> "LieART`", 
    "keywords" -> {}, "specialkeywords" -> {}, 
    "tutorialcollectionlinks" -> {}, "index" -> True, "label" -> 
    "Lie A R T Package Tutorial", "language" -> "en", "paclet" -> 
    "LieART Package", "status" -> "None", "summary" -> 
    "The application LieART provides functions to perform the most common \
tasks associated with irreducible representations (irreps) of the classical \
and exceptional Lie algebas. This loads the package: Irreducible \
representations (irreps) are internally decribed by their Dynkin label with a \
combined head of Irrep and the Lie algebra. ", "synonyms" -> {}, "title" -> 
    "LieART - Quick Start Tutorial", "titlemodifier" -> "", "windowtitle" -> 
    "LieART - Quick Start Tutorial", "type" -> "Tutorial", "uri" -> 
    "LieART/tutorial/QuickStartTutorial"}, "SearchTextTranslated" -> ""},
FrontEndVersion->"9.0 for Linux x86 (64-bit) (February 7, 2013)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> FrontEnd`FileName[{"Wolfram"}, "Reference.nb"]]], 
   Cell[
    StyleData["Input"], CellContext -> Notebook], 
   Cell[
    StyleData["Output"], CellContext -> Notebook], 
   Cell[
    StyleData["TutorialMoreAboutSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["RelatedTutorialsSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}], 
   Cell[
    StyleData["TutorialRelatedLinksSection"], 
    CellGroupingRules -> {"SectionGrouping", 30}]}, Visible -> False, 
  FrontEndVersion -> "9.0 for Linux x86 (64-bit) (February 7, 2013)", 
  StyleDefinitions -> "Default.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[622, 21, 2752, 61, 49, "AnchorBarGrid",
 CellID->1],
Cell[CellGroupData[{
Cell[3399, 86, 66, 1, 46, "Title",
 CellID->509267359],
Cell[3468, 89, 222, 5, 50, "Text",
 CellID->1534169418],
Cell[3693, 96, 67, 1, 32, "MathCaption",
 CellID->1167244701],
Cell[CellGroupData[{
Cell[3785, 101, 95, 3, 26, "Input",
 CellID->1092165306],
Cell[CellGroupData[{
Cell[3905, 108, 139, 3, 22, "Print",
 CellID->1077823253],
Cell[4047, 113, 151, 3, 24, "Print",
 CellID->61599756]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[4247, 122, 76, 1, 61, "Section",
 CellID->1449799695],
Cell[4326, 125, 221, 6, 52, "Text",
 CellID->374349434],
Cell[4550, 133, 449, 15, 61, "DefinitionBox",
 CellID->2096742444],
Cell[5002, 150, 72, 1, 32, "Caption",
 CellID->1891092685],
Cell[5077, 153, 1413, 53, 76, "Text",
 CellID->144687180],
Cell[6493, 208, 223, 8, 33, "MathCaption",
 CellID->836781195],
Cell[CellGroupData[{
Cell[6741, 220, 210, 7, 28, "Input",
 CellID->2058623809],
Cell[6954, 229, 437, 15, 50, "Output",
 CellID->1066064808]
}, Open  ]],
Cell[7406, 247, 260, 8, 33, "MathCaption",
 CellID->1221898746],
Cell[CellGroupData[{
Cell[7691, 259, 213, 7, 28, "Input",
 CellID->711449157],
Cell[7907, 268, 185, 5, 50, "Output",
 CellID->94736506]
}, Open  ]],
Cell[8107, 276, 257, 8, 33, "MathCaption",
 CellID->681119358],
Cell[CellGroupData[{
Cell[8389, 288, 173, 5, 28, "Input",
 CellID->937078051],
Cell[8565, 295, 247, 9, 35, "Output",
 CellID->1834936312]
}, Open  ]],
Cell[8827, 307, 579, 16, 75, "Text",
 CellID->319667636],
Cell[9409, 325, 287, 11, 33, "MathCaption",
 CellID->1881934180],
Cell[CellGroupData[{
Cell[9721, 340, 197, 6, 28, "Input",
 CellID->464169775],
Cell[9921, 348, 223, 8, 35, "Output",
 CellID->1025418079]
}, Open  ]],
Cell[10159, 359, 342, 7, 69, "Text",
 CellGroupingRules->{"GroupTogetherGrouping", 31},
 CellID->1468912962],
Cell[10504, 368, 450, 15, 80, "DefinitionBox",
 CellID->792167334],
Cell[10957, 385, 76, 1, 32, "Caption",
 CellID->1579371899],
Cell[11036, 388, 351, 13, 49, "MathCaption",
 CellID->109632626],
Cell[CellGroupData[{
Cell[11412, 405, 196, 6, 28, "Input",
 CellID->333587606],
Cell[11611, 413, 182, 5, 48, "Output",
 CellID->1531071855]
}, Open  ]],
Cell[11808, 421, 105, 2, 32, "MathCaption",
 CellID->1511260992],
Cell[CellGroupData[{
Cell[11938, 427, 198, 6, 28, "Input",
 CellID->1425383264],
Cell[12139, 435, 182, 5, 48, "Output",
 CellID->1118657973]
}, Open  ]],
Cell[12336, 443, 347, 9, 53, "Text",
 CellID->76966508],
Cell[12686, 454, 478, 15, 77, "DefinitionBox",
 CellID->616137037],
Cell[13167, 471, 54, 1, 32, "Caption",
 CellID->722483301],
Cell[13224, 474, 261, 10, 33, "MathCaption",
 CellID->1501046753],
Cell[CellGroupData[{
Cell[13510, 488, 295, 9, 28, "Input",
 CellID->1686390499],
Cell[13808, 499, 390, 16, 37, "Output",
 CellID->878148076]
}, Open  ]],
Cell[CellGroupData[{
Cell[14235, 520, 102, 3, 28, "Input",
 CellID->680021955],
Cell[14340, 525, 206, 5, 48, "Output",
 CellID->1800749348]
}, Open  ]],
Cell[CellGroupData[{
Cell[14583, 535, 106, 2, 26, "Input",
 CellID->1308426208],
Cell[14692, 539, 391, 16, 37, "Output",
 CellID->1831185662]
}, Open  ]],
Cell[15098, 558, 226, 5, 47, "MathCaption",
 CellID->1021666716],
Cell[CellGroupData[{
Cell[15349, 567, 362, 11, 28, "Input",
 CellID->1109740049],
Cell[15714, 580, 436, 17, 37, "Output",
 CellID->643938528]
}, Open  ]],
Cell[CellGroupData[{
Cell[16187, 602, 103, 3, 28, "Input",
 CellID->1050432190],
Cell[16293, 607, 227, 8, 63, "Output",
 CellID->874876204]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[16569, 621, 119, 2, 61, "Section",
 CellGroupingRules->{"SectionGrouping", Inherited},
 CellID->354599395],
Cell[16691, 625, 365, 11, 61, "DefinitionBox",
 CellID->1309055104],
Cell[17059, 638, 69, 1, 32, "Caption",
 CellID->1795200765],
Cell[17131, 641, 278, 11, 33, "MathCaption",
 CellID->1189993846],
Cell[CellGroupData[{
Cell[17434, 656, 299, 9, 28, "Input",
 CellID->1332771530],
Cell[17736, 667, 355, 15, 35, "Output",
 CellID->1013247494]
}, Open  ]],
Cell[18106, 685, 339, 14, 33, "MathCaption",
 CellID->133991959],
Cell[CellGroupData[{
Cell[18470, 703, 303, 9, 28, "Input",
 CellID->101963549],
Cell[18776, 714, 440, 19, 35, "Output",
 CellID->2099691517]
}, Open  ]],
Cell[19231, 736, 314, 12, 33, "MathCaption",
 CellID->1984894909],
Cell[CellGroupData[{
Cell[19570, 752, 342, 10, 28, "Input",
 CellID->1356021807],
Cell[19915, 764, 470, 20, 37, "Output",
 CellID->697007864]
}, Open  ]],
Cell[20400, 787, 253, 10, 33, "MathCaption",
 CellID->1005626329],
Cell[CellGroupData[{
Cell[20678, 801, 267, 8, 28, "Input",
 CellID->1887184167],
Cell[20948, 811, 664, 29, 37, "Output",
 CellID->1802103167]
}, Open  ]],
Cell[21627, 843, 654, 23, 49, "MathCaption",
 CellID->683723966],
Cell[CellGroupData[{
Cell[22306, 870, 102, 3, 28, "Input",
 CellID->997413459],
Cell[22411, 875, 279, 9, 78, "Output",
 CellID->1579939891]
}, Open  ]],
Cell[22705, 887, 607, 21, 49, "MathCaption",
 CellID->1581190875],
Cell[CellGroupData[{
Cell[23337, 912, 102, 3, 28, "Input",
 CellID->863826808],
Cell[23442, 917, 761, 33, 39, "Output",
 CellID->865403289]
}, Open  ]],
Cell[24218, 953, 375, 14, 33, "MathCaption",
 CellID->902430666],
Cell[CellGroupData[{
Cell[24618, 971, 430, 12, 45, "Input",
 CellID->5612545],
Cell[25051, 985, 1233, 52, 37, "Output",
 CellID->267790776]
}, Open  ]],
Cell[26299, 1040, 744, 26, 49, "MathCaption",
 CellID->1821879462],
Cell[CellGroupData[{
Cell[27068, 1070, 327, 10, 28, "Input",
 CellID->1779904617],
Cell[27398, 1082, 2151, 89, 56, "Output",
 CellID->812961123]
}, Open  ]],
Cell[29564, 1174, 575, 22, 48, "MathCaption",
 CellID->1431754643],
Cell[CellGroupData[{
Cell[30164, 1200, 156, 5, 28, "Input",
 CellID->808921778],
Cell[30323, 1207, 795, 34, 37, "Output",
 CellID->1630197054]
}, Open  ]],
Cell[31133, 1244, 733, 27, 33, "MathCaption",
 CellID->328377256],
Cell[CellGroupData[{
Cell[31891, 1275, 771, 23, 62, "Input",
 CellID->451431300],
Cell[32665, 1300, 1319, 59, 37, "Output",
 CellID->158054236]
}, Open  ]],
Cell[33999, 1362, 490, 19, 33, "MathCaption",
 CellID->1480557077],
Cell[CellGroupData[{
Cell[34514, 1385, 579, 18, 45, "Input",
 CellID->343055154],
Cell[35096, 1405, 583, 25, 37, "Output",
 CellID->1660930165]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[35728, 1436, 68, 1, 61, "Section",
 CellID->1642176862],
Cell[35799, 1439, 881, 29, 103, "DefinitionBox",
 CellID->1279752208],
Cell[36683, 1470, 76, 1, 32, "Caption",
 CellID->1925211260],
Cell[36762, 1473, 217, 7, 33, "MathCaption",
 CellID->442942768],
Cell[CellGroupData[{
Cell[37004, 1484, 315, 9, 28, "Input",
 CellID->1304419061],
Cell[37322, 1495, 1024, 41, 37, "Output",
 CellID->1523676287]
}, Open  ]],
Cell[38361, 1539, 594, 22, 33, "MathCaption",
 CellID->455264099],
Cell[CellGroupData[{
Cell[38980, 1565, 439, 13, 45, "Input",
 CellID->2123210495],
Cell[39422, 1580, 1757, 67, 39, "Output",
 CellID->1049794380]
}, Open  ]],
Cell[41194, 1650, 155, 6, 32, "MathCaption",
 CellID->1947924234],
Cell[CellGroupData[{
Cell[41374, 1660, 271, 8, 28, "Input",
 CellID->277260301],
Cell[41648, 1670, 765, 29, 37, "Output",
 CellID->9031146]
}, Open  ]],
Cell[42428, 1702, 233, 9, 33, "MathCaption",
 CellID->906860298],
Cell[CellGroupData[{
Cell[42686, 1715, 282, 8, 28, "Input",
 CellID->471735093],
Cell[42971, 1725, 1118, 49, 37, "Output",
 CellID->479800872]
}, Open  ]],
Cell[44104, 1777, 423, 14, 47, "MathCaption",
 CellID->1532918075],
Cell[CellGroupData[{
Cell[44552, 1795, 519, 15, 45, "Input",
 CellID->1650450684],
Cell[45074, 1812, 876, 33, 37, "Output",
 CellID->540895483]
}, Open  ]],
Cell[45965, 1848, 105, 2, 32, "MathCaption",
 CellID->981618749],
Cell[CellGroupData[{
Cell[46095, 1854, 631, 20, 45, "Input",
 CellID->1394022133],
Cell[46729, 1876, 1197, 45, 37, "Output",
 CellID->95559239]
}, Open  ]],
Cell[47941, 1924, 187, 4, 47, "MathCaption",
 CellID->1987934892],
Cell[CellGroupData[{
Cell[48153, 1932, 421, 13, 45, "Input",
 CellID->1514165693],
Cell[48577, 1947, 6502, 221, 130, "Output",
 CellID->664155987]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[55128, 2174, 53, 1, 61, "Section",
 CellID->431069882],
Cell[55184, 2177, 281, 9, 52, "Text",
 CellID->1099666059],
Cell[55468, 2188, 366, 12, 61, "DefinitionBox",
 CellID->1528399319],
Cell[55837, 2202, 55, 1, 32, "Caption",
 CellID->1616075793],
Cell[55895, 2205, 134, 6, 32, "MathCaption",
 CellID->381847914],
Cell[CellGroupData[{
Cell[56054, 2215, 215, 6, 28, "Input",
 CellID->1550153586],
Cell[56272, 2223, 859, 22, 104, "Output",
 CellID->239053307]
}, Open  ]],
Cell[57146, 2248, 136, 4, 32, "MathCaption",
 CellID->1293870584],
Cell[CellGroupData[{
Cell[57307, 2256, 623, 21, 45, "Input",
 CellID->794726788],
Cell[57933, 2279, 3685, 68, 80, "Output",
 CellID->314369717]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[61657, 2352, 23, 0, 44, "FooterCell"]
}
]
*)

(* End of internal cache information *)

